@@ -1,6 +1,4 @@
 use crate::command_prelude::*;
-
-use cargo::print_json;
 use serde::Serialize;
 
 pub fn cli() -> App {
@@ -21,7 +19,7 @@ pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
     let root = root
         .to_str()
         .ok_or_else(|| {
-            failure::format_err!(
+            anyhow::format_err!(
                 "your package path contains characters \
                  not representable in Unicode"
             )
@@ -30,6 +28,6 @@ pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
 
     let location = ProjectLocation { root };
 
-    print_json(&location);
+    config.shell().print_json(&location);
     Ok(())
 }
