@@ -3,8 +3,6 @@ use crate::command_prelude::*;
 use std::collections::HashMap;
 use std::process;
 
-use cargo::print_json;
-
 pub fn cli() -> App {
     subcommand("verify-project")
         .about("Check correctness of crate manifest")
@@ -13,19 +11,15 @@ pub fn cli() -> App {
 }
 
 pub fn exec(config: &mut Config, args: &ArgMatches<'_>) -> CliResult {
-    fn fail(reason: &str, value: &str) -> ! {
-        let mut h = HashMap::new();
-        h.insert(reason.to_string(), value.to_string());
-        print_json(&h);
-        process::exit(1)
-    }
-
     if let Err(e) = args.workspace(config) {
-        fail("invalid", &e.to_string())
+        let mut h = HashMap::new();
+        h.insert("invalid".to_string(), e.to_string());
+        config.shell().print_json(&h);
+        process::exit(1)
     }
 
     let mut h = HashMap::new();
     h.insert("success".to_string(), "true".to_string());
-    print_json(&h);
+    config.shell().print_json(&h);
     Ok(())
 }
