@@ -4,52 +4,59 @@ mod build_plan;
 mod compilation;
 mod compile_kind;
 mod context;
+mod crate_type;
 mod custom_build;
 mod fingerprint;
 mod job;
 mod job_queue;
 mod layout;
 mod links;
+mod lto;
 mod output_depinfo;
+pub mod rustdoc;
 pub mod standard_lib;
 mod timings;
 mod unit;
 pub mod unit_dependencies;
+pub mod unit_graph;
 
 use std::env;
 use std::ffi::{OsStr, OsString};
 use std::fs::{self, File};
-use std::io::Write;
+use std::io::{BufRead, Write};
 use std::path::PathBuf;
 use std::sync::Arc;
 
-use failure::Error;
+use anyhow::Error;
 use lazycell::LazyCell;
 use log::debug;
 
-pub use self::build_config::{BuildConfig, CompileMode, MessageFormat, ProfileKind};
-pub use self::build_context::{BuildContext, FileFlavor, TargetConfig, TargetInfo};
+pub use self::build_config::{BuildConfig, CompileMode, MessageFormat};
+pub use self::build_context::{BuildContext, FileFlavor, FileType, RustcTargetData, TargetInfo};
 use self::build_plan::BuildPlan;
 pub use self::compilation::{Compilation, Doctest};
 pub use self::compile_kind::{CompileKind, CompileTarget};
-pub use self::context::Context;
+pub use self::context::{Context, Metadata};
+pub use self::crate_type::CrateType;
 pub use self::custom_build::{BuildOutput, BuildScriptOutputs, BuildScripts};
 pub use self::job::Freshness;
 use self::job::{Job, Work};
 use self::job_queue::{JobQueue, JobState};
-pub use self::layout::is_bad_artifact_name;
+pub(crate) use self::layout::Layout;
+pub use self::lto::Lto;
 use self::output_depinfo::output_depinfo;
-use self::unit_dependencies::UnitDep;
+use self::unit_graph::UnitDep;
 pub use crate::core::compiler::unit::{Unit, UnitInterner};
 use crate::core::manifest::TargetSourcePath;
-use crate::core::profiles::{Lto, PanicStrategy, Profile};
-use crate::core::Feature;
-use crate::core::{PackageId, Target};
-use crate::util::errors::{self, CargoResult, CargoResultExt, Internal, ProcessError};
+use crate::core::profiles::{PanicStrategy, Profile, Strip};
+use crate::core::{Edition, Feature, PackageId, Target};
+use crate::util::errors::{self, CargoResult, CargoResultExt, ProcessError, VerboseError};
+use crate::util::interning::InternedString;
 use crate::util::machine_message::Message;
-use crate::util::paths;
 use crate::util::{self, machine_message, ProcessBuilder};
-use crate::util::{internal, join_paths, profile};
+use crate::util::{internal, join_paths, paths, profile};
+
+const RUSTDOC_CRATE_VERSION_FLAG: &str = "--crate-version";
 
 /// A glorified callback for executing calls to rustc. Rather than calling rustc
 /// directly, we'll use an `Executor`, giving clients an opportunity to intercept
@@ -58,13 +65,13 @@ pub trait Executor: Send + Sync + 'static {
     /// Called after a rustc process invocation is prepared up-front for a given
     /// unit of work (may still be modified for runtime-known dependencies, when
     /// the work is actually executed).
-    fn init<'a, 'cfg>(&self, _cx: &Context<'a, 'cfg>, _unit: &Unit<'a>) {}
+    fn init(&self, _cx: &Context<'_, '_>, _unit: &Unit) {}
 
     /// In case of an `Err`, Cargo will not continue with the build process for
     /// this package.
     fn exec(
         &self,
-        cmd: ProcessBuilder,
+        cmd: &ProcessBuilder,
         id: PackageId,
         target: &Target,
         mode: CompileMode,
@@ -74,7 +81,7 @@ pub trait Executor: Send + Sync + 'static {
 
     /// Queried when queuing each unit of work. If it returns true, then the
     /// unit will always be rebuilt, independent of whether it needs to be.
-    fn force_rebuild(&self, _unit: &Unit<'_>) -> bool {
+    fn force_rebuild(&self, _unit: &Unit) -> bool {
         false
     }
 }
@@ -87,7 +94,7 @@ pub struct DefaultExecutor;
 impl Executor for DefaultExecutor {
     fn exec(
         &self,
-        cmd: ProcessBuilder,
+        cmd: &ProcessBuilder,
         _id: PackageId,
         _target: &Target,
         _mode: CompileMode,
@@ -99,17 +106,17 @@ impl Executor for DefaultExecutor {
     }
 }
 
-fn compile<'a, 'cfg: 'a>(
-    cx: &mut Context<'a, 'cfg>,
-    jobs: &mut JobQueue<'a, 'cfg>,
+fn compile<'cfg>(
+    cx: &mut Context<'_, 'cfg>,
+    jobs: &mut JobQueue<'cfg>,
     plan: &mut BuildPlan,
-    unit: &Unit<'a>,
+    unit: &Unit,
     exec: &Arc<dyn Executor>,
     force_rebuild: bool,
 ) -> CargoResult<()> {
     let bcx = cx.bcx;
     let build_plan = bcx.build_config.build_plan;
-    if !cx.compiled.insert(*unit) {
+    if !cx.compiled.insert(unit.clone()) {
         return Ok(());
     }
 
@@ -136,13 +143,13 @@ fn compile<'a, 'cfg: 'a>(
             };
             work.then(link_targets(cx, unit, false)?)
         } else {
-            let work = if cx.bcx.show_warnings(unit.pkg.package_id()) {
+            let work = if unit.show_warnings(bcx.config) {
                 replay_output_cache(
                     unit.pkg.package_id(),
-                    unit.target,
+                    &unit.target,
                     cx.files().message_cache_path(unit),
                     cx.bcx.build_config.message_format,
-                    cx.bcx.config.shell().supports_color(),
+                    cx.bcx.config.shell().err_supports_color(),
                 )
             } else {
                 Work::noop()
@@ -168,11 +175,7 @@ fn compile<'a, 'cfg: 'a>(
     Ok(())
 }
 
-fn rustc<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
-    unit: &Unit<'a>,
-    exec: &Arc<dyn Executor>,
-) -> CargoResult<Work> {
+fn rustc(cx: &mut Context<'_, '_>, unit: &Unit, exec: &Arc<dyn Executor>) -> CargoResult<Work> {
     let mut rustc = prepare_rustc(cx, &unit.target.rustc_crate_types(), unit)?;
     let build_plan = cx.bcx.build_config.build_plan;
 
@@ -183,7 +186,6 @@ fn rustc<'a, 'cfg>(
 
     let outputs = cx.outputs(unit)?;
     let root = cx.files().out_dir(unit);
-    let kind = unit.kind;
 
     // Prepare the native lib state (extra `-L` and `-l` flags).
     let build_script_outputs = Arc::clone(&cx.build_script_outputs);
@@ -194,17 +196,12 @@ fn rustc<'a, 'cfg>(
     // don't pass the `-l` flags.
     let pass_l_flag = unit.target.is_lib() || !unit.pkg.targets().iter().any(|t| t.is_lib());
     let pass_cdylib_link_args = unit.target.is_cdylib();
-    let do_rename = unit.target.allows_underscores() && !unit.mode.is_any_test();
-    let real_name = unit.target.name().to_string();
-    let crate_name = unit.target.crate_name();
 
-    // Rely on `target_filenames` iterator as source of truth rather than rederiving filestem.
-    let rustc_dep_info_loc = if do_rename && cx.files().metadata(unit).is_none() {
-        root.join(&crate_name)
-    } else {
-        root.join(&cx.files().file_stem(unit))
-    }
-    .with_extension("d");
+    let dep_info_name = match cx.files().metadata(unit) {
+        Some(metadata) => format!("{}-{}.d", unit.target.crate_name(), metadata),
+        None => format!("{}.d", unit.target.crate_name()),
+    };
+    let rustc_dep_info_loc = root.join(dep_info_name);
     let dep_info_loc = fingerprint::dep_info_loc(cx, unit);
 
     rustc.args(cx.bcx.rustflags_args(unit));
@@ -213,7 +210,7 @@ fn rustc<'a, 'cfg>(
     }
     let mut output_options = OutputOptions::new(cx, unit);
     let package_id = unit.pkg.package_id();
-    let target = unit.target.clone();
+    let target = Target::clone(&unit.target);
     let mode = unit.mode;
 
     exec.init(cx, unit);
@@ -227,6 +224,8 @@ fn rustc<'a, 'cfg>(
         .unwrap_or_else(|| cx.bcx.config.cwd())
         .to_path_buf();
     let fingerprint_dir = cx.files().fingerprint_dir(unit);
+    let script_metadata = cx.find_build_script_metadata(unit.clone());
+    let is_local = unit.is_local();
 
     return Ok(Work::new(move |state| {
         // Only at runtime have we discovered what the extra -L and -l
@@ -249,7 +248,7 @@ fn rustc<'a, 'cfg>(
                 )?;
                 add_plugin_deps(&mut rustc, &script_outputs, &build_scripts, &root_output)?;
             }
-            add_custom_env(&mut rustc, &script_outputs, current_id, kind)?;
+            add_custom_env(&mut rustc, &script_outputs, current_id, script_metadata)?;
         }
 
         for output in outputs.iter() {
@@ -264,7 +263,7 @@ fn rustc<'a, 'cfg>(
             }
         }
 
-        fn internal_if_simple_exit_code(err: Error) -> Error {
+        fn verbose_if_simple_exit_code(err: Error) -> Error {
             // If a signal on unix (`code == None`) or an abnormal termination
             // on Windows (codes like `0xC0000409`), don't hide the error details.
             match err
@@ -272,7 +271,7 @@ fn rustc<'a, 'cfg>(
                 .as_ref()
                 .and_then(|perr| perr.exit.and_then(|e| e.code()))
             {
-                Some(n) if errors::is_simple_exit_code(n) => Internal::new(err).into(),
+                Some(n) if errors::is_simple_exit_code(n) => VerboseError::new(err).into(),
                 _ => err,
             }
         }
@@ -283,32 +282,17 @@ fn rustc<'a, 'cfg>(
             state.build_plan(buildkey, rustc.clone(), outputs.clone());
         } else {
             exec.exec(
-                rustc,
+                &rustc,
                 package_id,
                 &target,
                 mode,
                 &mut |line| on_stdout_line(state, line, package_id, &target),
                 &mut |line| on_stderr_line(state, line, package_id, &target, &mut output_options),
             )
-            .map_err(internal_if_simple_exit_code)
+            .map_err(verbose_if_simple_exit_code)
             .chain_err(|| format!("could not compile `{}`.", name))?;
         }
 
-        if do_rename && real_name != crate_name {
-            let dst = &outputs[0].path;
-            let src = dst.with_file_name(
-                dst.file_name()
-                    .unwrap()
-                    .to_str()
-                    .unwrap()
-                    .replace(&real_name, &crate_name),
-            );
-            if src.exists() && src.file_name() != dst.file_name() {
-                fs::rename(&src, &dst)
-                    .chain_err(|| internal(format!("could not rename crate {:?}", src)))?;
-            }
-        }
-
         if rustc_dep_info_loc.exists() {
             fingerprint::translate_dep_info(
                 &rustc_dep_info_loc,
@@ -316,8 +300,9 @@ fn rustc<'a, 'cfg>(
                 &cwd,
                 &pkg_root,
                 &target_dir,
+                &rustc,
                 // Do not track source files in the fingerprint for registry dependencies.
-                current_id.source_id().is_path(),
+                is_local,
             )
             .chain_err(|| {
                 internal(format!(
@@ -325,7 +310,9 @@ fn rustc<'a, 'cfg>(
                     rustc_dep_info_loc.display()
                 ))
             })?;
-            filetime::set_file_times(dep_info_loc, timestamp, timestamp)?;
+            // This mtime shift allows Cargo to detect if a source file was
+            // modified in the middle of the build.
+            paths::set_file_time_no_err(dep_info_loc, timestamp);
         }
 
         Ok(())
@@ -342,9 +329,9 @@ fn rustc<'a, 'cfg>(
         current_id: PackageId,
     ) -> CargoResult<()> {
         for key in build_scripts.to_link.iter() {
-            let output = build_script_outputs.get(key).ok_or_else(|| {
+            let output = build_script_outputs.get(key.0, key.1).ok_or_else(|| {
                 internal(format!(
-                    "couldn't find build script output for {}/{:?}",
+                    "couldn't find build script output for {}/{}",
                     key.0, key.1
                 ))
             })?;
@@ -377,10 +364,13 @@ fn rustc<'a, 'cfg>(
         rustc: &mut ProcessBuilder,
         build_script_outputs: &BuildScriptOutputs,
         current_id: PackageId,
-        kind: CompileKind,
+        metadata: Option<Metadata>,
     ) -> CargoResult<()> {
-        let key = (current_id, kind);
-        if let Some(output) = build_script_outputs.get(&key) {
+        let metadata = match metadata {
+            Some(metadata) => metadata,
+            None => return Ok(()),
+        };
+        if let Some(output) = build_script_outputs.get(current_id, metadata) {
             for &(ref name, ref value) in output.env.iter() {
                 rustc.env(name, value);
             }
@@ -391,11 +381,7 @@ fn rustc<'a, 'cfg>(
 
 /// Link the compiled target (often of form `foo-{metadata_hash}`) to the
 /// final target. This must happen during both "Fresh" and "Compile".
-fn link_targets<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
-    unit: &Unit<'a>,
-    fresh: bool,
-) -> CargoResult<Work> {
+fn link_targets(cx: &mut Context<'_, '_>, unit: &Unit, fresh: bool) -> CargoResult<Work> {
     let bcx = cx.bcx;
     let outputs = cx.outputs(unit)?;
     let export_dir = cx.files().export_dir();
@@ -405,7 +391,7 @@ fn link_targets<'a, 'cfg>(
     let features = unit.features.iter().map(|s| s.to_string()).collect();
     let json_messages = bcx.build_config.emit_json();
     let executable = cx.get_executable(unit)?;
-    let mut target = unit.target.clone();
+    let mut target = Target::clone(&unit.target);
     if let TargetSourcePath::Metabuild = target.src_path() {
         // Give it something to serialize.
         let path = unit.pkg.manifest().metabuild_path(cx.bcx.ws.target_dir());
@@ -479,10 +465,10 @@ fn add_plugin_deps(
     let var = util::dylib_path_envvar();
     let search_path = rustc.get_env(var).unwrap_or_default();
     let mut search_path = env::split_paths(&search_path).collect::<Vec<_>>();
-    for &id in build_scripts.plugins.iter() {
+    for (pkg_id, metadata) in &build_scripts.plugins {
         let output = build_script_outputs
-            .get(&(id, CompileKind::Host))
-            .ok_or_else(|| internal(format!("couldn't find libs for plugin dep {}", id)))?;
+            .get(*pkg_id, *metadata)
+            .ok_or_else(|| internal(format!("couldn't find libs for plugin dep {}", pkg_id)))?;
         search_path.append(&mut filter_dynamic_search_path(
             output.library_paths.iter(),
             root_output,
@@ -532,25 +518,33 @@ where
     search_path
 }
 
-fn prepare_rustc<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
-    crate_types: &[&str],
-    unit: &Unit<'a>,
+fn prepare_rustc(
+    cx: &mut Context<'_, '_>,
+    crate_types: &[CrateType],
+    unit: &Unit,
 ) -> CargoResult<ProcessBuilder> {
     let is_primary = cx.is_primary_package(unit);
+    let is_workspace = cx.bcx.ws.is_member(&unit.pkg);
 
     let mut base = cx
         .compilation
-        .rustc_process(unit.pkg, unit.target, is_primary)?;
-    base.inherit_jobserver(&cx.jobserver);
+        .rustc_process(unit, is_primary, is_workspace)?;
+    if cx.bcx.config.cli_unstable().jobserver_per_rustc {
+        let client = cx.new_jobserver()?;
+        base.inherit_jobserver(&client);
+        base.arg("-Zjobserver-token-requests");
+        assert!(cx.rustc_clients.insert(unit.clone(), client).is_none());
+    } else {
+        base.inherit_jobserver(&cx.jobserver);
+    }
     build_base_args(cx, &mut base, unit, crate_types)?;
     build_deps_args(&mut base, cx, unit)?;
     Ok(base)
 }
 
-fn rustdoc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult<Work> {
+fn rustdoc(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Work> {
     let bcx = cx.bcx;
-    let mut rustdoc = cx.compilation.rustdoc_process(unit.pkg, unit.target)?;
+    let mut rustdoc = cx.compilation.rustdoc_process(unit)?;
     rustdoc.inherit_jobserver(&cx.jobserver);
     rustdoc.arg("--crate-name").arg(&unit.target.crate_name());
     add_path_args(bcx, unit, &mut rustdoc);
@@ -575,28 +569,38 @@ fn rustdoc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult
 
     add_error_format_and_color(cx, &mut rustdoc, false)?;
 
-    if let Some(args) = bcx.extra_args_for(unit) {
+    if let Some(args) = cx.bcx.extra_args_for(unit) {
         rustdoc.args(args);
     }
 
     build_deps_args(&mut rustdoc, cx, unit)?;
+    rustdoc::add_root_urls(cx, unit, &mut rustdoc)?;
 
     rustdoc.args(bcx.rustdocflags_args(unit));
 
+    add_crate_versions_if_requested(bcx, unit, &mut rustdoc);
+
     let name = unit.pkg.name().to_string();
     let build_script_outputs = Arc::clone(&cx.build_script_outputs);
-    let key = (unit.pkg.package_id(), unit.kind);
     let package_id = unit.pkg.package_id();
-    let target = unit.target.clone();
+    let target = Target::clone(&unit.target);
     let mut output_options = OutputOptions::new(cx, unit);
+    let pkg_id = unit.pkg.package_id();
+    let script_metadata = cx.find_build_script_metadata(unit.clone());
 
     Ok(Work::new(move |state| {
-        if let Some(output) = build_script_outputs.lock().unwrap().get(&key) {
-            for cfg in output.cfgs.iter() {
-                rustdoc.arg("--cfg").arg(cfg);
-            }
-            for &(ref name, ref value) in output.env.iter() {
-                rustdoc.env(name, value);
+        if let Some(script_metadata) = script_metadata {
+            if let Some(output) = build_script_outputs
+                .lock()
+                .unwrap()
+                .get(pkg_id, script_metadata)
+            {
+                for cfg in output.cfgs.iter() {
+                    rustdoc.arg("--cfg").arg(cfg);
+                }
+                for &(ref name, ref value) in output.env.iter() {
+                    rustdoc.env(name, value);
+                }
             }
         }
         state.running(&rustdoc);
@@ -612,6 +616,31 @@ fn rustdoc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult
     }))
 }
 
+fn add_crate_versions_if_requested(
+    bcx: &BuildContext<'_, '_>,
+    unit: &Unit,
+    rustdoc: &mut ProcessBuilder,
+) {
+    if bcx.config.cli_unstable().crate_versions && !crate_version_flag_already_present(rustdoc) {
+        append_crate_version_flag(unit, rustdoc);
+    }
+}
+
+// The --crate-version flag could have already been passed in RUSTDOCFLAGS
+// or as an extra compiler argument for rustdoc
+fn crate_version_flag_already_present(rustdoc: &ProcessBuilder) -> bool {
+    rustdoc.get_args().iter().any(|flag| {
+        flag.to_str()
+            .map_or(false, |flag| flag.starts_with(RUSTDOC_CRATE_VERSION_FLAG))
+    })
+}
+
+fn append_crate_version_flag(unit: &Unit, rustdoc: &mut ProcessBuilder) {
+    rustdoc
+        .arg(RUSTDOC_CRATE_VERSION_FLAG)
+        .arg(unit.pkg.version().to_string());
+}
+
 // The path that we pass to rustc is actually fairly important because it will
 // show up in error messages (important for readability), debug information
 // (important for caching), etc. As a result we need to be pretty careful how we
@@ -626,7 +655,7 @@ fn rustdoc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult
 //
 // The first returned value here is the argument to pass to rustc, and the
 // second is the cwd that rustc should operate in.
-fn path_args(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>) -> (PathBuf, PathBuf) {
+fn path_args(bcx: &BuildContext<'_, '_>, unit: &Unit) -> (PathBuf, PathBuf) {
     let ws_root = bcx.ws.root();
     let src = match unit.target.src_path() {
         TargetSourcePath::Path(path) => path.to_path_buf(),
@@ -641,21 +670,21 @@ fn path_args(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>) -> (PathBuf, PathBuf)
     (src, unit.pkg.root().to_path_buf())
 }
 
-fn add_path_args(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>, cmd: &mut ProcessBuilder) {
+fn add_path_args(bcx: &BuildContext<'_, '_>, unit: &Unit, cmd: &mut ProcessBuilder) {
     let (arg, cwd) = path_args(bcx, unit);
     cmd.arg(arg);
     cmd.cwd(cwd);
 }
 
-fn add_cap_lints(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>, cmd: &mut ProcessBuilder) {
+fn add_cap_lints(bcx: &BuildContext<'_, '_>, unit: &Unit, cmd: &mut ProcessBuilder) {
     // If this is an upstream dep we don't want warnings from, turn off all
     // lints.
-    if !bcx.show_warnings(unit.pkg.package_id()) {
+    if !unit.show_warnings(bcx.config) {
         cmd.arg("--cap-lints").arg("allow");
 
     // If this is an upstream dep but we *do* want warnings, make sure that they
     // don't fail compilation.
-    } else if !unit.pkg.package_id().source_id().is_path() {
+    } else if !unit.is_local() {
         cmd.arg("--cap-lints").arg("warn");
     }
 }
@@ -690,18 +719,17 @@ fn add_error_format_and_color(
     Ok(())
 }
 
-fn build_base_args<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
+fn build_base_args(
+    cx: &mut Context<'_, '_>,
     cmd: &mut ProcessBuilder,
-    unit: &Unit<'a>,
-    crate_types: &[&str],
+    unit: &Unit,
+    crate_types: &[CrateType],
 ) -> CargoResult<()> {
     assert!(!unit.mode.is_run_custom_build());
 
     let bcx = cx.bcx;
     let Profile {
         ref opt_level,
-        ref lto,
         codegen_units,
         debuginfo,
         debug_assertions,
@@ -709,25 +737,31 @@ fn build_base_args<'a, 'cfg>(
         rpath,
         ref panic,
         incremental,
+        strip,
         ..
     } = unit.profile;
     let test = unit.mode.is_any_test();
 
     cmd.arg("--crate-name").arg(&unit.target.crate_name());
 
+    let edition = unit.target.edition();
+    if edition != Edition::Edition2015 {
+        cmd.arg(format!("--edition={}", edition));
+    }
+
     add_path_args(bcx, unit, cmd);
     add_error_format_and_color(cx, cmd, cx.rmeta_required(unit))?;
 
     if !test {
         for crate_type in crate_types.iter() {
-            cmd.arg("--crate-type").arg(crate_type);
+            cmd.arg("--crate-type").arg(crate_type.as_str());
         }
     }
 
     if unit.mode.is_check() {
         cmd.arg("--emit=dep-info,metadata");
     } else if !unit.requires_upstream_objects() {
-        // Always produce metdata files for rlib outputs. Metadata may be used
+        // Always produce metadata files for rlib outputs. Metadata may be used
         // in this session for a pipelined compilation, or it may be used in a
         // future Cargo session as part of a pipelined compile.
         cmd.arg("--emit=dep-info,metadata,link");
@@ -736,7 +770,7 @@ fn build_base_args<'a, 'cfg>(
     }
 
     let prefer_dynamic = (unit.target.for_host() && !unit.target.is_custom_build())
-        || (crate_types.contains(&"dylib") && bcx.ws.members().any(|p| p != unit.pkg));
+        || (crate_types.contains(&CrateType::Dylib) && bcx.ws.members().any(|p| *p != unit.pkg));
     if prefer_dynamic {
         cmd.arg("-C").arg("prefer-dynamic");
     }
@@ -749,16 +783,42 @@ fn build_base_args<'a, 'cfg>(
         cmd.arg("-C").arg(format!("panic={}", panic));
     }
 
-    // Disable LTO for host builds as prefer_dynamic and it are mutually
-    // exclusive.
-    if unit.target.can_lto() && !unit.target.for_host() {
-        match *lto {
-            Lto::Bool(false) => {}
-            Lto::Bool(true) => {
-                cmd.args(&["-C", "lto"]);
+    match cx.lto[unit] {
+        lto::Lto::Run(None) => {
+            cmd.arg("-C").arg("lto");
+        }
+        lto::Lto::Run(Some(s)) => {
+            cmd.arg("-C").arg(format!("lto={}", s));
+        }
+        lto::Lto::Off => {
+            cmd.arg("-C").arg("lto=off");
+        }
+        lto::Lto::ObjectAndBitcode => {} // this is rustc's default
+        lto::Lto::OnlyBitcode => {
+            // Note that this compiler flag, like the one below, is just an
+            // optimization in terms of build time. If we don't pass it then
+            // both object code and bitcode will show up. This is lagely just
+            // compat until the feature lands on stable and we can remove the
+            // conditional branch.
+            if cx
+                .bcx
+                .target_data
+                .info(CompileKind::Host)
+                .supports_embed_bitcode
+                .unwrap()
+            {
+                cmd.arg("-Clinker-plugin-lto");
             }
-            Lto::Named(ref s) => {
-                cmd.arg("-C").arg(format!("lto={}", s));
+        }
+        lto::Lto::OnlyObject => {
+            if cx
+                .bcx
+                .target_data
+                .info(CompileKind::Host)
+                .supports_embed_bitcode
+                .unwrap()
+            {
+                cmd.arg("-Cembed-bitcode=no");
             }
         }
     }
@@ -773,7 +833,7 @@ fn build_base_args<'a, 'cfg>(
         cmd.arg("-C").arg(format!("debuginfo={}", debuginfo));
     }
 
-    if let Some(args) = bcx.extra_args_for(unit) {
+    if let Some(args) = cx.bcx.extra_args_for(unit) {
         cmd.args(args);
     }
 
@@ -848,18 +908,21 @@ fn build_base_args<'a, 'cfg>(
         cmd.arg("--target").arg(n.rustc_target());
     }
 
-    opt(cmd, "-C", "ar=", bcx.ar(unit.kind).map(|s| s.as_ref()));
     opt(
         cmd,
         "-C",
         "linker=",
-        bcx.linker(unit.kind).map(|s| s.as_ref()),
+        bcx.linker(unit.kind).as_ref().map(|s| s.as_ref()),
     );
     if incremental {
         let dir = cx.files().layout(unit.kind).incremental().as_os_str();
         opt(cmd, "-C", "incremental=", Some(dir));
     }
 
+    if strip != Strip::None {
+        cmd.arg("-Z").arg(format!("strip={}", strip));
+    }
+
     if unit.is_std {
         // -Zforce-unstable-if-unmarked prevents the accidental use of
         // unstable crates within the sysroot (such as "extern crate libc" or
@@ -869,13 +932,30 @@ fn build_base_args<'a, 'cfg>(
         cmd.arg("-Zforce-unstable-if-unmarked")
             .env("RUSTC_BOOTSTRAP", "1");
     }
+
+    // Add `CARGO_BIN_` environment variables for building tests.
+    if unit.target.is_test() || unit.target.is_bench() {
+        for bin_target in unit
+            .pkg
+            .manifest()
+            .targets()
+            .iter()
+            .filter(|target| target.is_bin())
+        {
+            let exe_path = cx
+                .files()
+                .bin_link_for_target(bin_target, unit.kind, cx.bcx)?;
+            let key = format!("CARGO_BIN_EXE_{}", bin_target.name());
+            cmd.env(&key, exe_path);
+        }
+    }
     Ok(())
 }
 
-fn build_deps_args<'a, 'cfg>(
+fn build_deps_args(
     cmd: &mut ProcessBuilder,
-    cx: &mut Context<'a, 'cfg>,
-    unit: &Unit<'a>,
+    cx: &mut Context<'_, '_>,
+    unit: &Unit,
 ) -> CargoResult<()> {
     let bcx = cx.bcx;
     cmd.arg("-L").arg(&{
@@ -894,15 +974,14 @@ fn build_deps_args<'a, 'cfg>(
         });
     }
 
-    // Create Vec since mutable cx is needed in closure below.
-    let deps = Vec::from(cx.unit_deps(unit));
+    let deps = cx.unit_deps(unit);
 
     // If there is not one linkable target but should, rustc fails later
     // on if there is an `extern crate` for it. This may turn into a hard
     // error in the future (see PR #4797).
     if !deps
         .iter()
-        .any(|dep| !dep.unit.mode.is_doc() && dep.unit.target.linkable())
+        .any(|dep| !dep.unit.mode.is_doc() && dep.unit.target.is_linkable())
     {
         if let Some(dep) = deps
             .iter()
@@ -922,23 +1001,14 @@ fn build_deps_args<'a, 'cfg>(
 
     let mut unstable_opts = false;
 
-    if let Some(sysroot) = cx.files().layout(unit.kind).sysroot() {
-        if !unit.kind.is_host() {
-            cmd.arg("--sysroot").arg(sysroot);
-        }
-    }
-
     for dep in deps {
-        if !unit.is_std && dep.unit.is_std {
-            // Dependency to sysroot crate uses --sysroot.
-            continue;
-        }
         if dep.unit.mode.is_run_custom_build() {
             cmd.env("OUT_DIR", &cx.files().build_script_out_dir(&dep.unit));
         }
-        if dep.unit.target.linkable() && !dep.unit.mode.is_doc() {
-            link_to(cmd, cx, unit, &dep, &mut unstable_opts)?;
-        }
+    }
+
+    for arg in extern_args(cx, unit, &mut unstable_opts)? {
+        cmd.arg(arg);
     }
 
     // This will only be set if we're already using a feature
@@ -947,24 +1017,24 @@ fn build_deps_args<'a, 'cfg>(
         cmd.arg("-Z").arg("unstable-options");
     }
 
-    return Ok(());
+    Ok(())
+}
 
-    fn link_to<'a, 'cfg>(
-        cmd: &mut ProcessBuilder,
-        cx: &mut Context<'a, 'cfg>,
-        current: &Unit<'a>,
-        dep: &UnitDep<'a>,
-        need_unstable_opts: &mut bool,
-    ) -> CargoResult<()> {
-        let mut value = OsString::new();
-        value.push(dep.extern_crate_name.as_str());
-        value.push("=");
+/// Generates a list of `--extern` arguments.
+pub fn extern_args(
+    cx: &Context<'_, '_>,
+    unit: &Unit,
+    unstable_opts: &mut bool,
+) -> CargoResult<Vec<OsString>> {
+    let mut result = Vec::new();
+    let deps = cx.unit_deps(unit);
 
-        let mut pass = |file| {
-            let mut value = value.clone();
-            value.push(file);
-
-            if current
+    // Closure to add one dependency to `result`.
+    let mut link_to =
+        |dep: &UnitDep, extern_crate_name: InternedString, noprelude: bool| -> CargoResult<()> {
+            let mut value = OsString::new();
+            let mut opts = Vec::new();
+            if unit
                 .pkg
                 .manifest()
                 .features()
@@ -972,35 +1042,59 @@ fn build_deps_args<'a, 'cfg>(
                 .is_ok()
                 && !dep.public
             {
-                cmd.arg("--extern-private");
-                *need_unstable_opts = true;
-            } else {
-                cmd.arg("--extern");
+                opts.push("priv");
+                *unstable_opts = true;
             }
+            if noprelude {
+                opts.push("noprelude");
+                *unstable_opts = true;
+            }
+            if !opts.is_empty() {
+                value.push(opts.join(","));
+                value.push(":");
+            }
+            value.push(extern_crate_name.as_str());
+            value.push("=");
 
-            cmd.arg(&value);
-        };
+            let mut pass = |file| {
+                let mut value = value.clone();
+                value.push(file);
+                result.push(OsString::from("--extern"));
+                result.push(value);
+            };
 
-        let outputs = cx.outputs(&dep.unit)?;
-        let mut outputs = outputs.iter().filter_map(|output| match output.flavor {
-            FileFlavor::Linkable { rmeta } => Some((output, rmeta)),
-            _ => None,
-        });
+            let outputs = cx.outputs(&dep.unit)?;
 
-        if cx.only_requires_rmeta(current, &dep.unit) {
-            let (output, _rmeta) = outputs
-                .find(|(_output, rmeta)| *rmeta)
-                .expect("failed to find rlib dep for pipelined dep");
-            pass(&output.path);
-        } else {
-            for (output, rmeta) in outputs {
-                if !rmeta {
-                    pass(&output.path);
+            if cx.only_requires_rmeta(unit, &dep.unit) || dep.unit.mode.is_check() {
+                // Example: rlib dependency for an rlib, rmeta is all that is required.
+                let output = outputs
+                    .iter()
+                    .find(|output| output.flavor == FileFlavor::Rmeta)
+                    .expect("failed to find rmeta dep for pipelined dep");
+                pass(&output.path);
+            } else {
+                // Example: a bin needs `rlib` for dependencies, it cannot use rmeta.
+                for output in outputs.iter() {
+                    if output.flavor == FileFlavor::Linkable {
+                        pass(&output.path);
+                    }
                 }
             }
+            Ok(())
+        };
+
+    for dep in deps {
+        if dep.unit.target.is_linkable() && !dep.unit.mode.is_doc() {
+            link_to(dep, dep.extern_crate_name, dep.noprelude)?;
         }
-        Ok(())
     }
+    if unit.target.proc_macro() {
+        // Automatically import `proc_macro`.
+        result.push(OsString::from("--extern"));
+        result.push(OsString::from("proc_macro"));
+    }
+
+    Ok(result)
 }
 
 fn envify(s: &str) -> String {
@@ -1026,9 +1120,9 @@ struct OutputOptions {
 }
 
 impl OutputOptions {
-    fn new<'a>(cx: &Context<'a, '_>, unit: &Unit<'a>) -> OutputOptions {
+    fn new(cx: &Context<'_, '_>, unit: &Unit) -> OutputOptions {
         let look_for_metadata_directive = cx.rmeta_required(unit);
-        let color = cx.bcx.config.shell().supports_color();
+        let color = cx.bcx.config.shell().err_supports_color();
         let path = cx.files().message_cache_path(unit);
         // Remove old cache, ignore ENOENT, which is the common case.
         drop(fs::remove_file(&path));
@@ -1063,7 +1157,7 @@ fn on_stderr_line(
         // Check if caching is enabled.
         if let Some((path, cell)) = &mut options.cache_cell {
             // Cache the output, which will be replayed later when Fresh.
-            let f = cell.try_borrow_mut_with(|| File::create(path))?;
+            let f = cell.try_borrow_mut_with(|| paths::create(path))?;
             debug_assert!(!line.contains('\n'));
             f.write_all(line.as_bytes())?;
             f.write_all(&[b'\n'])?;
@@ -1187,6 +1281,31 @@ fn on_stderr_line_inner(
         }
     }
 
+    #[derive(serde::Deserialize)]
+    struct JobserverNotification {
+        jobserver_event: Event,
+    }
+
+    #[derive(Debug, serde::Deserialize)]
+    enum Event {
+        WillAcquire,
+        Release,
+    }
+
+    if let Ok(JobserverNotification { jobserver_event }) =
+        serde_json::from_str::<JobserverNotification>(compiler_message.get())
+    {
+        log::info!(
+            "found jobserver directive from rustc: `{:?}`",
+            jobserver_event
+        );
+        match jobserver_event {
+            Event::WillAcquire => state.will_acquire(),
+            Event::Release => state.release_token(),
+        }
+        return Ok(false);
+    }
+
     // And failing all that above we should have a legitimate JSON diagnostic
     // from the compiler, so wrap it in an external Cargo JSON message
     // indicating which package it came from and then emit it.
@@ -1223,9 +1342,20 @@ fn replay_output_cache(
             // No cached output, probably didn't emit anything.
             return Ok(());
         }
-        let contents = fs::read_to_string(&path)?;
-        for line in contents.lines() {
-            on_stderr_line(state, line, package_id, &target, &mut options)?;
+        // We sometimes have gigabytes of output from the compiler, so avoid
+        // loading it all into memory at once, as that can cause OOM where
+        // otherwise there would be none.
+        let file = paths::open(&path)?;
+        let mut reader = std::io::BufReader::new(file);
+        let mut line = String::new();
+        loop {
+            let length = reader.read_line(&mut line)?;
+            if length == 0 {
+                break;
+            }
+            let trimmed = line.trim_end_matches(&['\n', '\r'][..]);
+            on_stderr_line(state, trimmed, package_id, &target, &mut options)?;
+            line.clear();
         }
         Ok(())
     })
