@@ -23,7 +23,6 @@
 //! be detected via changes to `Cargo.lock`.
 
 use std::collections::{BTreeSet, HashSet};
-use std::fs::File;
 use std::io::{BufWriter, Write};
 use std::path::{Path, PathBuf};
 
@@ -44,17 +43,17 @@ fn render_filename<P: AsRef<Path>>(path: P, basedir: Option<&str>) -> CargoResul
     };
     relpath
         .to_str()
-        .ok_or_else(|| internal("path not utf-8"))
+        .ok_or_else(|| internal(format!("path `{:?}` not utf-8", relpath)))
         .map(|f| f.replace(" ", "\\ "))
 }
 
-fn add_deps_for_unit<'a, 'b>(
+fn add_deps_for_unit(
     deps: &mut BTreeSet<PathBuf>,
-    context: &mut Context<'a, 'b>,
-    unit: &Unit<'a>,
-    visited: &mut HashSet<Unit<'a>>,
+    cx: &mut Context<'_, '_>,
+    unit: &Unit,
+    visited: &mut HashSet<Unit>,
 ) -> CargoResult<()> {
-    if !visited.insert(*unit) {
+    if !visited.insert(unit.clone()) {
         return Ok(());
     }
 
@@ -62,13 +61,11 @@ fn add_deps_for_unit<'a, 'b>(
     // generate a dep info file, so we just keep on going below
     if !unit.mode.is_run_custom_build() {
         // Add dependencies from rustc dep-info output (stored in fingerprint directory)
-        let dep_info_loc = fingerprint::dep_info_loc(context, unit);
-        if let Some(paths) = fingerprint::parse_dep_info(
-            unit.pkg.root(),
-            context.files().host_root(),
-            &dep_info_loc,
-        )? {
-            for path in paths {
+        let dep_info_loc = fingerprint::dep_info_loc(cx, unit);
+        if let Some(paths) =
+            fingerprint::parse_dep_info(unit.pkg.root(), cx.files().host_root(), &dep_info_loc)?
+        {
+            for path in paths.files {
                 deps.insert(path);
             }
         } else {
@@ -82,19 +79,24 @@ fn add_deps_for_unit<'a, 'b>(
     }
 
     // Add rerun-if-changed dependencies
-    let key = (unit.pkg.package_id(), unit.kind);
-    if let Some(output) = context.build_script_outputs.lock().unwrap().get(&key) {
-        for path in &output.rerun_if_changed {
-            deps.insert(path.into());
+    if let Some(metadata) = cx.find_build_script_metadata(unit.clone()) {
+        if let Some(output) = cx
+            .build_script_outputs
+            .lock()
+            .unwrap()
+            .get(unit.pkg.package_id(), metadata)
+        {
+            for path in &output.rerun_if_changed {
+                deps.insert(path.into());
+            }
         }
     }
 
     // Recursively traverse all transitive dependencies
-    let unit_deps = Vec::from(context.unit_deps(unit)); // Create vec due to mutable borrow.
+    let unit_deps = Vec::from(cx.unit_deps(unit)); // Create vec due to mutable borrow.
     for dep in unit_deps {
-        let source_id = dep.unit.pkg.package_id().source_id();
-        if source_id.is_path() {
-            add_deps_for_unit(deps, context, &dep.unit, visited)?;
+        if unit.is_local() {
+            add_deps_for_unit(deps, cx, &dep.unit, visited)?;
         }
     }
     Ok(())
@@ -103,7 +105,7 @@ fn add_deps_for_unit<'a, 'b>(
 /// Save a `.d` dep-info file for the given unit.
 ///
 /// This only saves files for uplifted artifacts.
-pub fn output_depinfo<'a, 'b>(cx: &mut Context<'a, 'b>, unit: &Unit<'a>) -> CargoResult<()> {
+pub fn output_depinfo(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<()> {
     let bcx = cx.bcx;
     let mut deps = BTreeSet::new();
     let mut visited = HashSet::new();
@@ -115,7 +117,7 @@ pub fn output_depinfo<'a, 'b>(cx: &mut Context<'a, 'b>, unit: &Unit<'a>) -> Carg
                 .resolve_path(bcx.config)
                 .as_os_str()
                 .to_str()
-                .ok_or_else(|| internal("build.dep-info-basedir path not utf-8"))?
+                .ok_or_else(|| anyhow::format_err!("build.dep-info-basedir path not utf-8"))?
                 .to_string();
             Some(basedir_string.as_str())
         }
@@ -129,7 +131,7 @@ pub fn output_depinfo<'a, 'b>(cx: &mut Context<'a, 'b>, unit: &Unit<'a>) -> Carg
     for output in cx
         .outputs(unit)?
         .iter()
-        .filter(|o| o.flavor != FileFlavor::DebugInfo)
+        .filter(|o| !matches!(o.flavor, FileFlavor::DebugInfo | FileFlavor::Auxiliary))
     {
         if let Some(ref link_dst) = output.hardlink {
             let output_path = link_dst.with_extension("d");
@@ -139,13 +141,13 @@ pub fn output_depinfo<'a, 'b>(cx: &mut Context<'a, 'b>, unit: &Unit<'a>) -> Carg
                 // If nothing changed don't recreate the file which could alter
                 // its mtime
                 if let Ok(previous) = fingerprint::parse_rustc_dep_info(&output_path) {
-                    if previous.len() == 1 && previous[0].0 == target_fn && previous[0].1 == deps {
+                    if previous.files.iter().eq(deps.iter().map(Path::new)) {
                         continue;
                     }
                 }
 
                 // Otherwise write it all out
-                let mut outfile = BufWriter::new(File::create(output_path)?);
+                let mut outfile = BufWriter::new(paths::create(output_path)?);
                 write!(outfile, "{}:", target_fn)?;
                 for dep in &deps {
                     write!(outfile, " {}", dep)?;
