@@ -1,11 +1,15 @@
-use crate::core::compiler::{CompileKind, CompileMode};
-use crate::core::{profiles::Profile, InternedString, Package, Target};
+use crate::core::compiler::{CompileKind, CompileMode, CrateType};
+use crate::core::manifest::{Target, TargetKind};
+use crate::core::{profiles::Profile, Package};
 use crate::util::hex::short_hash;
+use crate::util::interning::InternedString;
+use crate::util::Config;
 use std::cell::RefCell;
 use std::collections::HashSet;
 use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::ops::Deref;
+use std::rc::Rc;
 
 /// All information needed to define a unit.
 ///
@@ -21,21 +25,21 @@ use std::ops::Deref;
 /// example, it needs to know the target architecture (OS, chip arch etc.) and it needs to know
 /// whether you want a debug or release build. There is enough information in this struct to figure
 /// all that out.
-#[derive(Clone, Copy, PartialOrd, Ord)]
-pub struct Unit<'a> {
-    inner: &'a UnitInner<'a>,
+#[derive(Clone, PartialOrd, Ord)]
+pub struct Unit {
+    inner: Rc<UnitInner>,
 }
 
 /// Internal fields of `Unit` which `Unit` will dereference to.
 #[derive(Clone, Hash, PartialEq, Eq, PartialOrd, Ord)]
-pub struct UnitInner<'a> {
+pub struct UnitInner {
     /// Information about available targets, which files to include/exclude, etc. Basically stuff in
     /// `Cargo.toml`.
-    pub pkg: &'a Package,
+    pub pkg: Package,
     /// Information about the specific target to build, out of the possible targets in `pkg`. Not
     /// to be confused with *target-triple* (or *target architecture* ...), the target arch for a
     /// build.
-    pub target: &'a Target,
+    pub target: Target,
     /// The profile contains information about *how* the build should be run, including debug
     /// level, etc.
     pub profile: Profile,
@@ -55,7 +59,7 @@ pub struct UnitInner<'a> {
     pub is_std: bool,
 }
 
-impl UnitInner<'_> {
+impl UnitInner {
     /// Returns whether compilation of this unit requires all upstream artifacts
     /// to be available.
     ///
@@ -65,39 +69,52 @@ impl UnitInner<'_> {
     pub fn requires_upstream_objects(&self) -> bool {
         self.mode.is_any_test() || self.target.kind().requires_upstream_objects()
     }
+
+    /// Returns whether or not this is a "local" package.
+    ///
+    /// A "local" package is one that the user can likely edit, or otherwise
+    /// wants warnings, etc.
+    pub fn is_local(&self) -> bool {
+        self.pkg.package_id().source_id().is_path() && !self.is_std
+    }
+
+    /// Returns whether or not warnings should be displayed for this unit.
+    pub fn show_warnings(&self, config: &Config) -> bool {
+        self.is_local() || config.extra_verbose()
+    }
 }
 
-impl<'a> Unit<'a> {
+impl Unit {
     pub fn buildkey(&self) -> String {
         format!("{}-{}", self.pkg.name(), short_hash(self))
     }
 }
 
 // Just hash the pointer for fast hashing
-impl<'a> Hash for Unit<'a> {
+impl Hash for Unit {
     fn hash<H: Hasher>(&self, hasher: &mut H) {
-        (self.inner as *const UnitInner<'a>).hash(hasher)
+        std::ptr::hash(&*self.inner, hasher)
     }
 }
 
 // Just equate the pointer since these are interned
-impl<'a> PartialEq for Unit<'a> {
-    fn eq(&self, other: &Unit<'a>) -> bool {
-        self.inner as *const UnitInner<'a> == other.inner as *const UnitInner<'a>
+impl PartialEq for Unit {
+    fn eq(&self, other: &Unit) -> bool {
+        std::ptr::eq(&*self.inner, &*other.inner)
     }
 }
 
-impl<'a> Eq for Unit<'a> {}
+impl Eq for Unit {}
 
-impl<'a> Deref for Unit<'a> {
-    type Target = UnitInner<'a>;
+impl Deref for Unit {
+    type Target = UnitInner;
 
-    fn deref(&self) -> &UnitInner<'a> {
-        self.inner
+    fn deref(&self) -> &UnitInner {
+        &*self.inner
     }
 }
 
-impl<'a> fmt::Debug for Unit<'a> {
+impl fmt::Debug for Unit {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("Unit")
             .field("pkg", &self.pkg)
@@ -117,17 +134,17 @@ impl<'a> fmt::Debug for Unit<'a> {
 /// efficient hash/equality implementation for `Unit`. All units are
 /// manufactured through an interner which guarantees that each equivalent value
 /// is only produced once.
-pub struct UnitInterner<'a> {
-    state: RefCell<InternerState<'a>>,
+pub struct UnitInterner {
+    state: RefCell<InternerState>,
 }
 
-struct InternerState<'a> {
-    cache: HashSet<Box<UnitInner<'a>>>,
+struct InternerState {
+    cache: HashSet<Rc<UnitInner>>,
 }
 
-impl<'a> UnitInterner<'a> {
+impl UnitInterner {
     /// Creates a new blank interner
-    pub fn new() -> UnitInterner<'a> {
+    pub fn new() -> UnitInterner {
         UnitInterner {
             state: RefCell::new(InternerState {
                 cache: HashSet::new(),
@@ -139,17 +156,38 @@ impl<'a> UnitInterner<'a> {
     /// will all be equivalent to the provided arguments, although they may not
     /// be the exact same instance.
     pub fn intern(
-        &'a self,
-        pkg: &'a Package,
-        target: &'a Target,
+        &self,
+        pkg: &Package,
+        target: &Target,
         profile: Profile,
         kind: CompileKind,
         mode: CompileMode,
         features: Vec<InternedString>,
         is_std: bool,
-    ) -> Unit<'a> {
+    ) -> Unit {
+        let target = match (is_std, target.kind()) {
+            // This is a horrible hack to support build-std. `libstd` declares
+            // itself with both rlib and dylib. We don't want the dylib for a
+            // few reasons:
+            //
+            // - dylibs don't have a hash in the filename. If you do something
+            //   (like switch rustc versions), it will stomp on the dylib
+            //   file, invalidating the entire cache (because std is a dep of
+            //   everything).
+            // - We don't want to publicize the presence of dylib for the
+            //   standard library.
+            //
+            // At some point in the future, it would be nice to have a
+            // first-class way of overriding or specifying crate-types.
+            (true, TargetKind::Lib(crate_types)) if crate_types.contains(&CrateType::Dylib) => {
+                let mut new_target = Target::clone(target);
+                new_target.set_kind(TargetKind::Lib(vec![CrateType::Rlib]));
+                new_target
+            }
+            _ => target.clone(),
+        };
         let inner = self.intern_inner(&UnitInner {
-            pkg,
+            pkg: pkg.clone(),
             target,
             profile,
             kind,
@@ -160,34 +198,13 @@ impl<'a> UnitInterner<'a> {
         Unit { inner }
     }
 
-    // Ok so interning here is a little unsafe, hence the usage of `unsafe`
-    // internally. The primary issue here is that we've got an internal cache of
-    // `UnitInner` instances added so far, but we may need to mutate it to add
-    // it, and the mutation for an interner happens behind a shared borrow.
-    //
-    // Our goal though is to escape the lifetime `borrow_mut` to the same
-    // lifetime as the borrowed passed into this function. That's where `unsafe`
-    // comes into play. What we're subverting here is resizing internally in the
-    // `HashSet` as well as overwriting previous keys in the `HashSet`.
-    //
-    // As a result we store `Box<UnitInner>` internally to have an extra layer
-    // of indirection. That way `*const UnitInner` is a stable address that
-    // doesn't change with `HashSet` resizing. Furthermore we're careful to
-    // never overwrite an entry once inserted.
-    //
-    // Ideally we'd use an off-the-shelf interner from crates.io which avoids a
-    // small amount of unsafety here, but at the time this was written one
-    // wasn't obviously available.
-    fn intern_inner(&'a self, item: &UnitInner<'a>) -> &'a UnitInner<'a> {
+    fn intern_inner(&self, item: &UnitInner) -> Rc<UnitInner> {
         let mut me = self.state.borrow_mut();
         if let Some(item) = me.cache.get(item) {
-            // note that `item` has type `&Box<UnitInner<'a>`. Use `&**` to
-            // convert that to `&UnitInner<'a>`, then do some trickery to extend
-            // the lifetime to the `'a` on the function here.
-            return unsafe { &*(&**item as *const UnitInner<'a>) };
+            return item.clone();
         }
-        me.cache.insert(Box::new(item.clone()));
-        let item = me.cache.get(item).unwrap();
-        unsafe { &*(&**item as *const UnitInner<'a>) }
+        let item = Rc::new(item.clone());
+        me.cache.insert(item.clone());
+        item
     }
 }
