@@ -82,6 +82,11 @@ impl FromStr for Edition {
         match s {
             "2015" => Ok(Edition::Edition2015),
             "2018" => Ok(Edition::Edition2018),
+            s if s.parse().map_or(false, |y: u16| y > 2020 && y < 2050) => bail!(
+                "this version of Cargo is older than the `{}` edition, \
+                 and only supports `2015` and `2018` editions.",
+                s
+            ),
             s => bail!(
                 "supported edition values are `2015` or `2018`, but `{}` \
                  is unknown",
@@ -206,6 +211,12 @@ features! {
 
         // Allow to specify profiles other than 'dev', 'release', 'test', etc.
         [unstable] named_profiles: bool,
+
+        // Opt-in new-resolver behavior.
+        [unstable] resolver: bool,
+
+        // Allow to specify whether binaries should be stripped.
+        [unstable] strip: bool,
     }
 }
 
@@ -343,6 +354,9 @@ pub struct CliUnstable {
     pub jobserver_per_rustc: bool,
     pub features: Option<Vec<String>>,
     pub crate_versions: bool,
+    pub separate_nightlies: bool,
+    pub multitarget: bool,
+    pub rustdoc_map: bool,
 }
 
 impl CliUnstable {
@@ -420,6 +434,9 @@ impl CliUnstable {
             "jobserver-per-rustc" => self.jobserver_per_rustc = parse_empty(k, v)?,
             "features" => self.features = Some(parse_features(v)),
             "crate-versions" => self.crate_versions = parse_empty(k, v)?,
+            "separate-nightlies" => self.separate_nightlies = parse_empty(k, v)?,
+            "multitarget" => self.multitarget = parse_empty(k, v)?,
+            "rustdoc-map" => self.rustdoc_map = parse_empty(k, v)?,
             _ => bail!("unknown `-Z` flag specified: {}", k),
         }
 
