@@ -1,12 +1,11 @@
 use std::collections::HashMap;
 use std::num::NonZeroU64;
-use std::rc::Rc;
 
-use failure::format_err;
+use anyhow::format_err;
 use log::debug;
 
-use crate::core::interning::InternedString;
 use crate::core::{Dependency, PackageId, SourceId, Summary};
+use crate::util::interning::InternedString;
 use crate::util::Graph;
 
 use super::dep_cache::RegistryQueryer;
@@ -35,7 +34,7 @@ pub struct Context {
 
     /// a way to look up for a package in activations what packages required it
     /// and all of the exact deps that it fulfilled.
-    pub parents: Graph<PackageId, Rc<Vec<Dependency>>>,
+    pub parents: Graph<PackageId, im_rc::HashSet<Dependency>>,
 }
 
 /// When backtracking it can be useful to know how far back to go.
@@ -164,20 +163,21 @@ impl Context {
             }
         }
         debug!("checking if {} is already activated", summary.package_id());
-        if opts.all_features {
+        if opts.features.all_features {
             return Ok(false);
         }
 
         let has_default_feature = summary.features().contains_key("default");
         Ok(match self.resolve_features.get(&id) {
             Some(prev) => {
-                opts.features.is_subset(prev)
-                    && (!opts.uses_default_features
+                opts.features.features.is_subset(prev)
+                    && (!opts.features.uses_default_features
                         || prev.contains("default")
                         || !has_default_feature)
             }
             None => {
-                opts.features.is_empty() && (!opts.uses_default_features || !has_default_feature)
+                opts.features.features.is_empty()
+                    && (!opts.features.uses_default_features || !has_default_feature)
             }
         })
     }
@@ -254,8 +254,8 @@ impl Context {
             .collect()
     }
 
-    pub fn graph(&self) -> Graph<PackageId, Vec<Dependency>> {
-        let mut graph: Graph<PackageId, Vec<Dependency>> = Graph::new();
+    pub fn graph(&self) -> Graph<PackageId, std::collections::HashSet<Dependency>> {
+        let mut graph: Graph<PackageId, std::collections::HashSet<Dependency>> = Graph::new();
         self.activations
             .values()
             .for_each(|(r, _)| graph.add(r.package_id()));
@@ -264,14 +264,14 @@ impl Context {
             for (o, e) in self.parents.edges(i) {
                 let old_link = graph.link(*o, *i);
                 assert!(old_link.is_empty());
-                *old_link = e.to_vec();
+                *old_link = e.iter().cloned().collect();
             }
         }
         graph
     }
 }
 
-impl Graph<PackageId, Rc<Vec<Dependency>>> {
+impl Graph<PackageId, im_rc::HashSet<Dependency>> {
     pub fn parents_of(&self, p: PackageId) -> impl Iterator<Item = (PackageId, bool)> + '_ {
         self.edges(&p)
             .map(|(grand, d)| (*grand, d.iter().any(|x| x.is_public())))
@@ -337,7 +337,7 @@ impl PublicDependency {
         parent_pid: PackageId,
         is_public: bool,
         age: ContextAge,
-        parents: &Graph<PackageId, Rc<Vec<Dependency>>>,
+        parents: &Graph<PackageId, im_rc::HashSet<Dependency>>,
     ) {
         // one tricky part is that `candidate_pid` may already be active and
         // have public dependencies of its own. So we not only need to mark
@@ -382,7 +382,7 @@ impl PublicDependency {
         b_id: PackageId,
         parent: PackageId,
         is_public: bool,
-        parents: &Graph<PackageId, Rc<Vec<Dependency>>>,
+        parents: &Graph<PackageId, im_rc::HashSet<Dependency>>,
     ) -> Result<
         (),
         (
@@ -398,7 +398,7 @@ impl PublicDependency {
             // for each (transitive) parent that can newly see `t`
             let mut stack = vec![(parent, is_public)];
             while let Some((p, public)) = stack.pop() {
-                // TODO: dont look at the same thing more then once
+                // TODO: don't look at the same thing more then once
                 if let Some(o) = self.inner.get(&p).and_then(|x| x.get(&t.name())) {
                     if o.0 != t {
                         // the (transitive) parent can already see a different version by `t`s name.
