@@ -62,20 +62,23 @@ use crate::util::profile;
 
 use self::context::Context;
 use self::dep_cache::RegistryQueryer;
+use self::features::RequestedFeatures;
 use self::types::{ConflictMap, ConflictReason, DepsFrame};
 use self::types::{FeaturesSet, RcVecIter, RemainingDeps, ResolverProgress};
 
 pub use self::encode::Metadata;
 pub use self::encode::{EncodableDependency, EncodablePackageId, EncodableResolve};
 pub use self::errors::{ActivateError, ActivateResult, ResolveError};
+pub use self::features::{ForceAllTargets, HasDevUnits};
 pub use self::resolve::{Resolve, ResolveVersion};
-pub use self::types::ResolveOpts;
+pub use self::types::{ResolveBehavior, ResolveOpts};
 
 mod conflict_cache;
 mod context;
 mod dep_cache;
 mod encode;
 mod errors;
+pub mod features;
 mod resolve;
 mod types;
 
@@ -106,9 +109,6 @@ mod types;
 /// * `config` - a location to print warnings and such, or `None` if no warnings
 ///   should be printed
 ///
-/// * `print_warnings` - whether or not to print backwards-compatibility
-///   warnings and such
-///
 /// * `check_public_visible_dependencies` - a flag for whether to enforce the restrictions
 ///     introduced in the "public & private dependencies" RFC (1977). The current implementation
 ///     makes sure that there is only one version of each name visible to each package.
@@ -141,17 +141,27 @@ pub fn resolve(
         let cksum = summary.checksum().map(|s| s.to_string());
         cksums.insert(summary.package_id(), cksum);
     }
+    let graph = cx.graph();
+    let replacements = cx.resolve_replacements(&registry);
+    let features = cx
+        .resolve_features
+        .iter()
+        .map(|(k, v)| (*k, v.iter().cloned().collect()))
+        .collect();
+    let summaries = cx
+        .activations
+        .into_iter()
+        .map(|(_key, (summary, _age))| (summary.package_id(), summary))
+        .collect();
     let resolve = Resolve::new(
-        cx.graph(),
-        cx.resolve_replacements(&registry),
-        cx.resolve_features
-            .iter()
-            .map(|(k, v)| (*k, v.iter().map(|x| x.to_string()).collect()))
-            .collect(),
+        graph,
+        replacements,
+        features,
         cksums,
         BTreeMap::new(),
         Vec::new(),
         ResolveVersion::default_for_new_lockfiles(),
+        summaries,
     );
 
     check_cycles(&resolve)?;
@@ -161,11 +171,11 @@ pub fn resolve(
     Ok(resolve)
 }
 
-/// Recursively activates the dependencies for `top`, in depth-first order,
+/// Recursively activates the dependencies for `summaries`, in depth-first order,
 /// backtracking across possible candidates for each dependency as necessary.
 ///
 /// If all dependencies can be activated and resolved to a version in the
-/// dependency graph, cx.resolve is returned.
+/// dependency graph, `cx` is returned.
 fn activate_deps_loop(
     mut cx: Context,
     registry: &mut RegistryQueryer<'_>,
@@ -368,9 +378,11 @@ fn activate_deps_loop(
             let pid = candidate.package_id();
             let opts = ResolveOpts {
                 dev_deps: false,
-                features: Rc::clone(&features),
-                all_features: false,
-                uses_default_features: dep.uses_default_features(),
+                features: RequestedFeatures {
+                    features: Rc::clone(&features),
+                    all_features: false,
+                    uses_default_features: dep.uses_default_features(),
+                },
             };
             trace!(
                 "{}[{}]>{} trying {}",
@@ -597,12 +609,11 @@ fn activate(
     cx.age += 1;
     if let Some((parent, dep)) = parent {
         let parent_pid = parent.package_id();
-        Rc::make_mut(
-            // add a edge from candidate to parent in the parents graph
-            cx.parents.link(candidate_pid, parent_pid),
-        )
-        // and associate dep with that edge
-        .push(dep.clone());
+        // add a edge from candidate to parent in the parents graph
+        cx.parents
+            .link(candidate_pid, parent_pid)
+            // and associate dep with that edge
+            .insert(dep.clone());
         if let Some(public_dependency) = cx.public_dependency.as_mut() {
             public_dependency.add_edge(
                 candidate_pid,
@@ -644,7 +655,7 @@ fn activate(
 
     let now = Instant::now();
     let (used_features, deps) =
-        &*registry.build_deps(parent.map(|p| p.0.package_id()), &candidate, &opts)?;
+        &*registry.build_deps(cx, parent.map(|p| p.0.package_id()), &candidate, &opts)?;
 
     // Record what list of features is active for this package.
     if !used_features.is_empty() {
@@ -833,7 +844,7 @@ fn generalize_conflicting(
     for (critical_parent, critical_parents_deps) in
         cx.parents.edges(&backtrack_critical_id).filter(|(p, _)| {
             // it will only help backjump further if it is older then the critical_age
-            cx.is_active(*p).expect("parent not currently active!?") < backtrack_critical_age
+            cx.is_active(**p).expect("parent not currently active!?") < backtrack_critical_age
         })
     {
         for critical_parents_dep in critical_parents_deps.iter() {
@@ -868,7 +879,7 @@ fn generalize_conflicting(
                 let mut con = conflicting_activations.clone();
                 // It is always valid to combine previously inserted conflicts.
                 // A, B are both known bad states each that can never be activated.
-                // A + B is redundant but cant be activated, as if
+                // A + B is redundant but can't be activated, as if
                 // A + B is active then A is active and we know that is not ok.
                 for (_, other) in &others {
                     con.extend(other.iter().map(|(&id, re)| (id, re.clone())));
@@ -928,7 +939,7 @@ fn find_candidate(
     // If we backtracked we do not know how our `conflicting_activations` related to
     // the cause of that backtrack, so we do not update it.
     let age = if !backtracked {
-        // we dont have abnormal situations. So we can ask `cx` for how far back we need to go.
+        // we don't have abnormal situations. So we can ask `cx` for how far back we need to go.
         let a = cx.is_conflicting(Some(parent.package_id()), conflicting_activations);
         // If the `conflicting_activations` does not apply to `cx`, then something went very wrong
         // in building it. But we will just fall back to laboriously trying all possibilities witch
@@ -1012,7 +1023,7 @@ fn check_cycles(resolve: &Resolve) -> CargoResult<()> {
         path.push(id);
         // See if we visited ourselves
         if !visited.insert(id) {
-            failure::bail!(
+            anyhow::bail!(
                 "cyclic package dependency: package `{}` depends on itself. Cycle:\n{}",
                 id,
                 errors::describe_path(&path.iter().rev().collect::<Vec<_>>()),
@@ -1062,7 +1073,7 @@ fn check_duplicate_pkgs_in_lockfile(resolve: &Resolve) -> CargoResult<()> {
     for pkg_id in resolve.iter() {
         let encodable_pkd_id = encode::encodable_package_id(pkg_id, &state);
         if let Some(prev_pkg_id) = unique_pkg_ids.insert(encodable_pkd_id, pkg_id) {
-            failure::bail!(
+            anyhow::bail!(
                 "package collision in the lockfile: packages {} and {} are different, \
                  but only one can be written to lockfile unambiguously",
                 prev_pkg_id,
