@@ -7,8 +7,8 @@ use std::rc::Rc;
 
 use serde::{Serialize, Serializer};
 
-use crate::core::interning::InternedString;
 use crate::core::{Dependency, PackageId, SourceId};
+use crate::util::interning::InternedString;
 use semver::Version;
 
 use crate::util::CargoResult;
@@ -46,14 +46,14 @@ impl Summary {
         for dep in dependencies.iter() {
             let feature = dep.name_in_toml();
             if !namespaced_features && features.get(&*feature).is_some() {
-                failure::bail!(
+                anyhow::bail!(
                     "Features and dependencies cannot have the \
                      same name: `{}`",
                     feature
                 )
             }
             if dep.is_optional() && !dep.is_transitive() {
-                failure::bail!(
+                anyhow::bail!(
                     "Dev-dependencies are not allowed to be optional: `{}`",
                     feature
                 )
@@ -91,7 +91,7 @@ impl Summary {
         &self.inner.features
     }
     pub fn checksum(&self) -> Option<&str> {
-        self.inner.checksum.as_ref().map(|s| &s[..])
+        self.inner.checksum.as_deref()
     }
     pub fn links(&self) -> Option<InternedString> {
         self.inner.links
@@ -115,8 +115,7 @@ impl Summary {
     {
         {
             let slot = &mut Rc::make_mut(&mut self.inner).dependencies;
-            let deps = mem::replace(slot, Vec::new());
-            *slot = deps.into_iter().map(f).collect();
+            *slot = mem::take(slot).into_iter().map(f).collect();
         }
         self
     }
@@ -180,7 +179,7 @@ where
             match dep_map.get(feature.borrow()) {
                 Some(dep_data) => {
                     if !dep_data.iter().any(|d| d.is_optional()) {
-                        failure::bail!(
+                        anyhow::bail!(
                             "Feature `{}` includes the dependency of the same name, but this is \
                              left implicit in the features included by this feature.\n\
                              Additionally, the dependency must be marked as optional to be \
@@ -249,7 +248,7 @@ where
                 (&Feature(feat), dep_exists, false) => {
                     if namespaced && !features.contains_key(&*feat) {
                         if dep_exists {
-                            failure::bail!(
+                            anyhow::bail!(
                                 "Feature `{}` includes `{}` which is not defined as a feature.\n\
                                  A non-optional dependency of the same name is defined; consider \
                                  adding `optional = true` to its definition",
@@ -257,7 +256,7 @@ where
                                 feat
                             )
                         } else {
-                            failure::bail!(
+                            anyhow::bail!(
                                 "Feature `{}` includes `{}` which is not defined as a feature",
                                 feature,
                                 feat
@@ -272,7 +271,7 @@ where
                 // just to provide the correct string for the crate dependency in the error.
                 (&Crate(ref dep), true, false) => {
                     if namespaced {
-                        failure::bail!(
+                        anyhow::bail!(
                             "Feature `{}` includes `crate:{}` which is not an \
                              optional dependency.\nConsider adding \
                              `optional = true` to the dependency",
@@ -280,7 +279,7 @@ where
                             dep
                         )
                     } else {
-                        failure::bail!(
+                        anyhow::bail!(
                             "Feature `{}` depends on `{}` which is not an \
                              optional dependency.\nConsider adding \
                              `optional = true` to the dependency",
@@ -295,14 +294,14 @@ where
                 // namespaced here is just to provide the correct string in the error.
                 (&Crate(ref dep), false, _) => {
                     if namespaced {
-                        failure::bail!(
+                        anyhow::bail!(
                             "Feature `{}` includes `crate:{}` which is not a known \
                              dependency",
                             feature,
                             dep
                         )
                     } else {
-                        failure::bail!(
+                        anyhow::bail!(
                             "Feature `{}` includes `{}` which is neither a dependency nor \
                              another feature",
                             feature,
@@ -313,7 +312,7 @@ where
                 (&Crate(_), true, true) => {}
                 // If the value is a feature for one of the dependencies, bail out if no such
                 // dependency is actually defined in the manifest.
-                (&CrateFeature(ref dep, _), false, _) => failure::bail!(
+                (&CrateFeature(ref dep, _), false, _) => anyhow::bail!(
                     "Feature `{}` requires a feature of `{}` which is not a \
                      dependency",
                     feature,
@@ -327,7 +326,7 @@ where
         if !dependency_found {
             // If we have not found the dependency of the same-named feature, we should
             // bail here.
-            failure::bail!(
+            anyhow::bail!(
                 "Feature `{}` includes the optional dependency of the \
                  same name, but this is left implicit in the features \
                  included by this feature.\nConsider adding \
