@@ -7,14 +7,13 @@
 //! rough outline is:
 //!
 //! - Resolve the dependency graph (see `ops::resolve`).
-//! - Download any packages needed (see `PackageSet`). Note that dependency
-//!   downloads are deferred until `build_unit_dependencies`.
+//! - Download any packages needed (see `PackageSet`).
 //! - Generate a list of top-level "units" of work for the targets the user
 //!   requested on the command-line. Each `Unit` corresponds to a compiler
 //!   invocation. This is done in this module (`generate_targets`).
+//! - Build the graph of `Unit` dependencies (see
+//!   `core::compiler::context::unit_dependencies`).
 //! - Create a `Context` which will perform the following steps:
-//!     - Build the graph of `Unit` dependencies (see
-//!       `core::compiler::context::unit_dependencies`).
 //!     - Prepare the `target` directory (see `Layout`).
 //!     - Create a job queue (see `JobQueue`). The queue checks the
 //!       fingerprint of each `Unit` to determine if it should run or be
@@ -25,18 +24,17 @@
 
 use std::collections::{BTreeSet, HashMap, HashSet};
 use std::iter::FromIterator;
-use std::path::PathBuf;
 use std::sync::Arc;
 
-use crate::core::compiler::standard_lib;
 use crate::core::compiler::unit_dependencies::build_unit_dependencies;
+use crate::core::compiler::{standard_lib, unit_graph};
 use crate::core::compiler::{BuildConfig, BuildContext, Compilation, Context};
 use crate::core::compiler::{CompileKind, CompileMode, RustcTargetData, Unit};
 use crate::core::compiler::{DefaultExecutor, Executor, UnitInterner};
 use crate::core::profiles::{Profiles, UnitFor};
 use crate::core::resolver::features::{self, FeaturesFor};
 use crate::core::resolver::{HasDevUnits, Resolve, ResolveOpts};
-use crate::core::{LibKind, Package, PackageSet, Target};
+use crate::core::{Package, PackageSet, Target};
 use crate::core::{PackageId, PackageIdSpec, TargetKind, Workspace};
 use crate::ops;
 use crate::ops::resolve::WorkspaceResolve;
@@ -44,9 +42,15 @@ use crate::util::config::Config;
 use crate::util::{closest_msg, profile, CargoResult};
 
 /// Contains information about how a package should be compiled.
+///
+/// Note on distinction between `CompileOptions` and `BuildConfig`:
+/// `BuildConfig` contains values that need to be retained after
+/// `BuildContext` is created. The other fields are no longer necessary. Think
+/// of it as `CompileOptions` are high-level settings requested on the
+/// command-line, and `BuildConfig` are low-level settings for actually
+/// driving `rustc`.
 #[derive(Debug)]
-pub struct CompileOptions<'a> {
-    pub config: &'a Config,
+pub struct CompileOptions {
     /// Configuration information for a rustc build
     pub build_config: BuildConfig,
     /// Extra features to build for the root package
@@ -70,19 +74,12 @@ pub struct CompileOptions<'a> {
     /// Whether the `--document-private-items` flags was specified and should
     /// be forwarded to `rustdoc`.
     pub rustdoc_document_private_items: bool,
-    /// The directory to copy final artifacts to. Note that even if `out_dir` is
-    /// set, a copy of artifacts still could be found a `target/(debug\release)`
-    /// as usual.
-    // Note that, although the cmd-line flag name is `out-dir`, in code we use
-    // `export_dir`, to avoid confusion with out dir at `target/debug/deps`.
-    pub export_dir: Option<PathBuf>,
 }
 
-impl<'a> CompileOptions<'a> {
-    pub fn new(config: &'a Config, mode: CompileMode) -> CargoResult<CompileOptions<'a>> {
+impl<'a> CompileOptions {
+    pub fn new(config: &Config, mode: CompileMode) -> CargoResult<CompileOptions> {
         Ok(CompileOptions {
-            config,
-            build_config: BuildConfig::new(config, None, &None, mode)?,
+            build_config: BuildConfig::new(config, None, &[], mode)?,
             features: Vec::new(),
             all_features: false,
             no_default_features: false,
@@ -94,7 +91,6 @@ impl<'a> CompileOptions<'a> {
             target_rustc_args: None,
             local_rustdoc_args: None,
             rustdoc_document_private_items: false,
-            export_dir: None,
         })
     }
 }
@@ -241,10 +237,7 @@ pub enum CompileFilter {
     },
 }
 
-pub fn compile<'a>(
-    ws: &Workspace<'a>,
-    options: &CompileOptions<'a>,
-) -> CargoResult<Compilation<'a>> {
+pub fn compile<'a>(ws: &Workspace<'a>, options: &CompileOptions) -> CargoResult<Compilation<'a>> {
     let exec: Arc<dyn Executor> = Arc::new(DefaultExecutor);
     compile_with_exec(ws, options, &exec)
 }
@@ -253,7 +246,7 @@ pub fn compile<'a>(
 /// calls and add custom logic. `compile` uses `DefaultExecutor` which just passes calls through.
 pub fn compile_with_exec<'a>(
     ws: &Workspace<'a>,
-    options: &CompileOptions<'a>,
+    options: &CompileOptions,
     exec: &Arc<dyn Executor>,
 ) -> CargoResult<Compilation<'a>> {
     ws.emit_warnings()?;
@@ -262,11 +255,27 @@ pub fn compile_with_exec<'a>(
 
 pub fn compile_ws<'a>(
     ws: &Workspace<'a>,
-    options: &CompileOptions<'a>,
+    options: &CompileOptions,
     exec: &Arc<dyn Executor>,
 ) -> CargoResult<Compilation<'a>> {
+    let interner = UnitInterner::new();
+    let bcx = create_bcx(ws, options, &interner)?;
+    if options.build_config.unit_graph {
+        unit_graph::emit_serialized_unit_graph(&bcx.roots, &bcx.unit_graph)?;
+        return Ok(Compilation::new(&bcx)?);
+    }
+
+    let _p = profile::start("compiling");
+    let cx = Context::new(&bcx)?;
+    cx.compile(exec)
+}
+
+pub fn create_bcx<'a, 'cfg>(
+    ws: &'a Workspace<'cfg>,
+    options: &'a CompileOptions,
+    interner: &'a UnitInterner,
+) -> CargoResult<BuildContext<'a, 'cfg>> {
     let CompileOptions {
-        config,
         ref build_config,
         ref spec,
         ref features,
@@ -277,8 +286,8 @@ pub fn compile_ws<'a>(
         ref target_rustc_args,
         ref local_rustdoc_args,
         rustdoc_document_private_items,
-        ref export_dir,
     } = *options;
+    let config = ws.config();
 
     match build_config.mode {
         CompileMode::Test
@@ -301,13 +310,7 @@ pub fn compile_ws<'a>(
         }
     }
 
-    let profiles = Profiles::new(
-        ws.profiles(),
-        config,
-        build_config.requested_profile,
-        ws.features(),
-    )?;
-    let target_data = RustcTargetData::new(ws, build_config.requested_kind)?;
+    let target_data = RustcTargetData::new(ws, &build_config.requested_kinds)?;
 
     let specs = spec.to_package_id_specs(ws)?;
     let dev_deps = ws.require_optional_deps() || filter.need_dev_deps(build_config.mode);
@@ -320,10 +323,11 @@ pub fn compile_ws<'a>(
     let resolve = ops::resolve_ws_with_opts(
         ws,
         &target_data,
-        build_config.requested_kind,
+        &build_config.requested_kinds,
         &opts,
         &specs,
         has_dev_units,
+        crate::core::resolver::features::ForceAllTargets::No,
     )?;
     let WorkspaceResolve {
         mut pkg_set,
@@ -338,15 +342,14 @@ pub fn compile_ws<'a>(
                 .shell()
                 .warn("-Zbuild-std does not currently fully support --build-plan")?;
         }
-        if build_config.requested_kind.is_host() {
+        if build_config.requested_kinds[0].is_host() {
             // TODO: This should eventually be fixed. Unfortunately it is not
             // easy to get the host triple in BuildConfig. Consider changing
             // requested_target to an enum, or some other approach.
             anyhow::bail!("-Zbuild-std requires --target");
         }
-        let (mut std_package_set, std_resolve, std_features) =
-            standard_lib::resolve_std(ws, &target_data, build_config.requested_kind, crates)?;
-        remove_dylib_crate_type(&mut std_package_set)?;
+        let (std_package_set, std_resolve, std_features) =
+            standard_lib::resolve_std(ws, &target_data, &build_config.requested_kinds, crates)?;
         pkg_set.add_set(std_package_set);
         Some((std_resolve, std_features))
     } else {
@@ -355,11 +358,8 @@ pub fn compile_ws<'a>(
 
     // Find the packages in the resolver that the user wants to build (those
     // passed in with `-p` or the defaults from the workspace), and convert
-    // Vec<PackageIdSpec> to a Vec<&PackageId>.
-    let to_build_ids = specs
-        .iter()
-        .map(|s| s.query(resolve.iter()))
-        .collect::<CargoResult<Vec<_>>>()?;
+    // Vec<PackageIdSpec> to a Vec<PackageId>.
+    let to_build_ids = resolve.specs_to_ids(&specs)?;
     // Now get the `Package` for each `PackageId`. This may trigger a download
     // if the user specified `-p` for a dependency that is not downloaded.
     // Dependencies will be downloaded during build_unit_dependencies.
@@ -398,32 +398,29 @@ pub fn compile_ws<'a>(
         );
     }
 
+    let profiles = Profiles::new(
+        ws.profiles(),
+        config,
+        build_config.requested_profile,
+        ws.features(),
+    )?;
     profiles.validate_packages(
         ws.profiles(),
         &mut config.shell(),
         workspace_resolve.as_ref().unwrap_or(&resolve),
     )?;
 
-    let interner = UnitInterner::new();
-    let mut bcx = BuildContext::new(
-        ws,
-        &pkg_set,
-        config,
-        build_config,
-        profiles,
-        &interner,
-        HashMap::new(),
-        target_data,
-    )?;
-
     let units = generate_targets(
         ws,
         &to_builds,
         filter,
-        build_config.requested_kind,
+        &build_config.requested_kinds,
+        build_config.mode,
         &resolve,
         &resolved_features,
-        &bcx,
+        &pkg_set,
+        &profiles,
+        interner,
     )?;
 
     let std_roots = if let Some(crates) = &config.cli_unstable().build_std {
@@ -441,16 +438,19 @@ pub fn compile_ws<'a>(
         }
         let (std_resolve, std_features) = std_resolve_features.as_ref().unwrap();
         standard_lib::generate_std_roots(
-            &bcx,
             &crates,
             std_resolve,
             std_features,
-            build_config.requested_kind,
+            &build_config.requested_kinds,
+            &pkg_set,
+            interner,
+            &profiles,
         )?
     } else {
-        Vec::new()
+        Default::default()
     };
 
+    let mut extra_compiler_args = HashMap::new();
     if let Some(args) = extra_args {
         if units.len() != 1 {
             anyhow::bail!(
@@ -460,7 +460,7 @@ pub fn compile_ws<'a>(
                 extra_args_name
             );
         }
-        bcx.extra_compiler_args.insert(units[0], args);
+        extra_compiler_args.insert(units[0].clone(), args);
     }
     for unit in &units {
         if unit.mode.is_doc() || unit.mode.is_doc_test() {
@@ -476,27 +476,40 @@ pub fn compile_ws<'a>(
             }
 
             if let Some(args) = extra_args {
-                bcx.extra_compiler_args.insert(*unit, args.clone());
+                extra_compiler_args
+                    .entry(unit.clone())
+                    .or_default()
+                    .extend(args);
             }
         }
     }
 
-    let unit_dependencies = build_unit_dependencies(
-        &bcx,
+    let unit_graph = build_unit_dependencies(
+        ws,
+        &pkg_set,
         &resolve,
         &resolved_features,
         std_resolve_features.as_ref(),
         &units,
         &std_roots,
+        build_config.mode,
+        &target_data,
+        &profiles,
+        interner,
     )?;
 
-    let ret = {
-        let _p = profile::start("compiling");
-        let cx = Context::new(config, &bcx, unit_dependencies, build_config.requested_kind)?;
-        cx.compile(&units, export_dir.clone(), exec)?
-    };
+    let bcx = BuildContext::new(
+        ws,
+        pkg_set,
+        build_config,
+        profiles,
+        extra_compiler_args,
+        target_data,
+        units,
+        unit_graph,
+    )?;
 
-    Ok(ret)
+    Ok(bcx)
 }
 
 impl FilterRule {
@@ -607,16 +620,18 @@ impl CompileFilter {
     pub fn need_dev_deps(&self, mode: CompileMode) -> bool {
         match mode {
             CompileMode::Test | CompileMode::Doctest | CompileMode::Bench => true,
-            CompileMode::Build | CompileMode::Doc { .. } | CompileMode::Check { .. } => match *self
-            {
-                CompileFilter::Default { .. } => false,
-                CompileFilter::Only {
-                    ref examples,
-                    ref tests,
-                    ref benches,
-                    ..
-                } => examples.is_specific() || tests.is_specific() || benches.is_specific(),
-            },
+            CompileMode::Check { test: true } => true,
+            CompileMode::Build | CompileMode::Doc { .. } | CompileMode::Check { test: false } => {
+                match *self {
+                    CompileFilter::Default { .. } => false,
+                    CompileFilter::Only {
+                        ref examples,
+                        ref tests,
+                        ref benches,
+                        ..
+                    } => examples.is_specific() || tests.is_specific() || benches.is_specific(),
+                }
+            }
             CompileMode::RunCustomBuild => panic!("Invalid mode"),
         }
     }
@@ -679,90 +694,103 @@ struct Proposal<'a> {
 
 /// Generates all the base targets for the packages the user has requested to
 /// compile. Dependencies for these targets are computed later in `unit_dependencies`.
-fn generate_targets<'a>(
+fn generate_targets(
     ws: &Workspace<'_>,
-    packages: &[&'a Package],
+    packages: &[&Package],
     filter: &CompileFilter,
-    default_arch_kind: CompileKind,
-    resolve: &'a Resolve,
+    requested_kinds: &[CompileKind],
+    mode: CompileMode,
+    resolve: &Resolve,
     resolved_features: &features::ResolvedFeatures,
-    bcx: &BuildContext<'a, '_>,
-) -> CargoResult<Vec<Unit<'a>>> {
-    // Helper for creating a `Unit` struct.
-    let new_unit = |pkg: &'a Package, target: &'a Target, target_mode: CompileMode| {
-        let unit_for = if target_mode.is_any_test() {
-            // NOTE: the `UnitFor` here is subtle. If you have a profile
-            // with `panic` set, the `panic` flag is cleared for
-            // tests/benchmarks and their dependencies. If this
-            // was `normal`, then the lib would get compiled three
-            // times (once with panic, once without, and once with
-            // `--test`).
-            //
-            // This would cause a problem for doc tests, which would fail
-            // because `rustdoc` would attempt to link with both libraries
-            // at the same time. Also, it's probably not important (or
-            // even desirable?) for rustdoc to link with a lib with
-            // `panic` set.
-            //
-            // As a consequence, Examples and Binaries get compiled
-            // without `panic` set. This probably isn't a bad deal.
-            //
-            // Forcing the lib to be compiled three times during `cargo
-            // test` is probably also not desirable.
-            UnitFor::new_test(bcx.config)
-        } else if target.for_host() {
-            // Proc macro / plugin should not have `panic` set.
-            UnitFor::new_compiler()
-        } else {
-            UnitFor::new_normal()
-        };
-        // Custom build units are added in `build_unit_dependencies`.
-        assert!(!target.is_custom_build());
-        let target_mode = match target_mode {
-            CompileMode::Test => {
-                if target.is_example() && !filter.is_specific() && !target.tested() {
-                    // Examples are included as regular binaries to verify
-                    // that they compile.
-                    CompileMode::Build
-                } else {
-                    CompileMode::Test
+    package_set: &PackageSet<'_>,
+    profiles: &Profiles,
+    interner: &UnitInterner,
+) -> CargoResult<Vec<Unit>> {
+    let config = ws.config();
+    // Helper for creating a list of `Unit` structures
+    let new_unit =
+        |units: &mut HashSet<Unit>, pkg: &Package, target: &Target, target_mode: CompileMode| {
+            let unit_for = if target_mode.is_any_test() {
+                // NOTE: the `UnitFor` here is subtle. If you have a profile
+                // with `panic` set, the `panic` flag is cleared for
+                // tests/benchmarks and their dependencies. If this
+                // was `normal`, then the lib would get compiled three
+                // times (once with panic, once without, and once with
+                // `--test`).
+                //
+                // This would cause a problem for doc tests, which would fail
+                // because `rustdoc` would attempt to link with both libraries
+                // at the same time. Also, it's probably not important (or
+                // even desirable?) for rustdoc to link with a lib with
+                // `panic` set.
+                //
+                // As a consequence, Examples and Binaries get compiled
+                // without `panic` set. This probably isn't a bad deal.
+                //
+                // Forcing the lib to be compiled three times during `cargo
+                // test` is probably also not desirable.
+                UnitFor::new_test(config)
+            } else if target.for_host() {
+                // Proc macro / plugin should not have `panic` set.
+                UnitFor::new_compiler()
+            } else {
+                UnitFor::new_normal()
+            };
+            // Custom build units are added in `build_unit_dependencies`.
+            assert!(!target.is_custom_build());
+            let target_mode = match target_mode {
+                CompileMode::Test => {
+                    if target.is_example() && !filter.is_specific() && !target.tested() {
+                        // Examples are included as regular binaries to verify
+                        // that they compile.
+                        CompileMode::Build
+                    } else {
+                        CompileMode::Test
+                    }
                 }
-            }
-            CompileMode::Build => match *target.kind() {
-                TargetKind::Test => CompileMode::Test,
-                TargetKind::Bench => CompileMode::Bench,
-                _ => CompileMode::Build,
-            },
-            // `CompileMode::Bench` is only used to inform `filter_default_targets`
-            // which command is being used (`cargo bench`). Afterwards, tests
-            // and benches are treated identically. Switching the mode allows
-            // de-duplication of units that are essentially identical. For
-            // example, `cargo build --all-targets --release` creates the units
-            // (lib profile:bench, mode:test) and (lib profile:bench, mode:bench)
-            // and since these are the same, we want them to be de-duplicated in
-            // `unit_dependencies`.
-            CompileMode::Bench => CompileMode::Test,
-            _ => target_mode,
-        };
-        let kind = default_arch_kind.for_target(target);
-        let profile =
-            bcx.profiles
-                .get_profile(pkg.package_id(), ws.is_member(pkg), unit_for, target_mode);
+                CompileMode::Build => match *target.kind() {
+                    TargetKind::Test => CompileMode::Test,
+                    TargetKind::Bench => CompileMode::Bench,
+                    _ => CompileMode::Build,
+                },
+                // `CompileMode::Bench` is only used to inform `filter_default_targets`
+                // which command is being used (`cargo bench`). Afterwards, tests
+                // and benches are treated identically. Switching the mode allows
+                // de-duplication of units that are essentially identical. For
+                // example, `cargo build --all-targets --release` creates the units
+                // (lib profile:bench, mode:test) and (lib profile:bench, mode:bench)
+                // and since these are the same, we want them to be de-duplicated in
+                // `unit_dependencies`.
+                CompileMode::Bench => CompileMode::Test,
+                _ => target_mode,
+            };
 
-        let features = Vec::from(resolved_features.activated_features(
-            pkg.package_id(),
-            FeaturesFor::NormalOrDev, // Root units are never build dependencies.
-        ));
-        bcx.units.intern(
-            pkg,
-            target,
-            profile,
-            kind,
-            target_mode,
-            features,
-            /*is_std*/ false,
-        )
-    };
+            let is_local = pkg.package_id().source_id().is_path();
+            let profile = profiles.get_profile(
+                pkg.package_id(),
+                ws.is_member(pkg),
+                is_local,
+                unit_for,
+                target_mode,
+            );
+
+            // No need to worry about build-dependencies, roots are never build dependencies.
+            let features_for = FeaturesFor::from_for_host(target.proc_macro());
+            let features = resolved_features.activated_features(pkg.package_id(), features_for);
+
+            for kind in requested_kinds {
+                let unit = interner.intern(
+                    pkg,
+                    target,
+                    profile,
+                    kind.for_target(target),
+                    target_mode,
+                    features.clone(),
+                    /*is_std*/ false,
+                );
+                units.insert(unit);
+            }
+        };
 
     // Create a list of proposed targets.
     let mut proposals: Vec<Proposal<'_>> = Vec::new();
@@ -772,14 +800,14 @@ fn generate_targets<'a>(
             required_features_filterable,
         } => {
             for pkg in packages {
-                let default = filter_default_targets(pkg.targets(), bcx.build_config.mode);
+                let default = filter_default_targets(pkg.targets(), mode);
                 proposals.extend(default.into_iter().map(|target| Proposal {
                     pkg,
                     target,
                     requires_features: !required_features_filterable,
-                    mode: bcx.build_config.mode,
+                    mode,
                 }));
-                if bcx.build_config.mode == CompileMode::Test {
+                if mode == CompileMode::Test {
                     if let Some(t) = pkg
                         .targets()
                         .iter()
@@ -805,14 +833,14 @@ fn generate_targets<'a>(
         } => {
             if *lib != LibRule::False {
                 let mut libs = Vec::new();
-                for proposal in
-                    filter_targets(packages, Target::is_lib, false, bcx.build_config.mode)
-                {
+                for proposal in filter_targets(packages, Target::is_lib, false, mode) {
                     let Proposal { target, pkg, .. } = proposal;
-                    if bcx.build_config.mode.is_doc_test() && !target.doctestable() {
+                    if mode.is_doc_test() && !target.doctestable() {
+                        let types = target.rustc_crate_types();
+                        let types_str: Vec<&str> = types.iter().map(|t| t.as_str()).collect();
                         ws.config().shell().warn(format!(
                             "doc tests are not supported for crate type(s) `{}` in package `{}`",
-                            target.rustc_crate_types().join(", "),
+                            types_str.join(", "),
                             pkg.name()
                         ))?;
                     } else {
@@ -836,10 +864,10 @@ fn generate_targets<'a>(
                 FilterRule::All => Target::tested,
                 FilterRule::Just(_) => Target::is_test,
             };
-            let test_mode = match bcx.build_config.mode {
+            let test_mode = match mode {
                 CompileMode::Build => CompileMode::Test,
                 CompileMode::Check { .. } => CompileMode::Check { test: true },
-                _ => bcx.build_config.mode,
+                _ => mode,
             };
             // If `--benches` was specified, add all targets that would be
             // generated by `cargo bench`.
@@ -847,10 +875,10 @@ fn generate_targets<'a>(
                 FilterRule::All => Target::benched,
                 FilterRule::Just(_) => Target::is_bench,
             };
-            let bench_mode = match bcx.build_config.mode {
+            let bench_mode = match mode {
                 CompileMode::Build => CompileMode::Bench,
                 CompileMode::Check { .. } => CompileMode::Check { test: true },
-                _ => bcx.build_config.mode,
+                _ => mode,
             };
 
             proposals.extend(list_rule_targets(
@@ -858,14 +886,14 @@ fn generate_targets<'a>(
                 bins,
                 "bin",
                 Target::is_bin,
-                bcx.build_config.mode,
+                mode,
             )?);
             proposals.extend(list_rule_targets(
                 packages,
                 examples,
                 "example",
                 Target::is_example,
-                bcx.build_config.mode,
+                mode,
             )?);
             proposals.extend(list_rule_targets(
                 packages,
@@ -902,15 +930,14 @@ fn generate_targets<'a>(
         let unavailable_features = match target.required_features() {
             Some(rf) => {
                 let features = features_map.entry(pkg).or_insert_with(|| {
-                    resolve_all_features(resolve, resolved_features, pkg.package_id())
+                    resolve_all_features(resolve, resolved_features, package_set, pkg.package_id())
                 });
                 rf.iter().filter(|f| !features.contains(*f)).collect()
             }
             None => Vec::new(),
         };
         if target.is_lib() || unavailable_features.is_empty() {
-            let unit = new_unit(pkg, target, mode);
-            units.insert(unit);
+            new_unit(&mut units, pkg, target, mode);
         } else if requires_features {
             let required_features = target.required_features().unwrap();
             let quoted_required_features: Vec<String> = required_features
@@ -936,9 +963,10 @@ fn generate_targets<'a>(
 ///
 /// Dependencies are added as `dep_name/feat_name` because `required-features`
 /// wants to support that syntax.
-fn resolve_all_features(
+pub fn resolve_all_features(
     resolve_with_overrides: &Resolve,
     resolved_features: &features::ResolvedFeatures,
+    package_set: &PackageSet<'_>,
     package_id: PackageId,
 ) -> HashSet<String> {
     let mut features: HashSet<String> = resolved_features
@@ -950,14 +978,17 @@ fn resolve_all_features(
     // Include features enabled for use by dependencies so targets can also use them with the
     // required-features field when deciding whether to be built or skipped.
     for (dep_id, deps) in resolve_with_overrides.deps(package_id) {
+        let is_proc_macro = package_set
+            .get_one(dep_id)
+            .expect("packages downloaded")
+            .proc_macro();
         for dep in deps {
-            let features_for = if dep.is_build() {
-                FeaturesFor::BuildDep
-            } else {
-                FeaturesFor::NormalOrDev
-            };
-            for feature in resolved_features.activated_features(dep_id, features_for) {
-                features.insert(dep.name_in_toml().to_string() + "/" + &feature);
+            let features_for = FeaturesFor::from_for_host(is_proc_macro || dep.is_build());
+            for feature in resolved_features
+                .activated_features_unverified(dep_id, features_for)
+                .unwrap_or_default()
+            {
+                features.insert(format!("{}/{}", dep.name_in_toml(), feature));
             }
         }
     }
@@ -1067,35 +1098,3 @@ fn filter_targets<'a>(
     }
     proposals
 }
-
-/// When using `-Zbuild-std` we're building the standard library, but a
-/// technical detail of the standard library right now is that it builds itself
-/// as both an `rlib` and a `dylib`. We don't actually want to really publicize
-/// the `dylib` and in general it's a pain to work with, so when building libstd
-/// we want to remove the `dylib` crate type.
-///
-/// Cargo doesn't have a fantastic way of doing that right now, so let's hack
-/// around it a bit and (ab)use the fact that we have mutable access to
-/// `PackageSet` here to rewrite downloaded packages. We iterate over all `path`
-/// packages (which should download immediately and not actually cause blocking
-/// here) and edit their manifests to only list one `LibKind` for an `Rlib`.
-fn remove_dylib_crate_type(set: &mut PackageSet<'_>) -> CargoResult<()> {
-    let ids = set
-        .package_ids()
-        .filter(|p| p.source_id().is_path())
-        .collect::<Vec<_>>();
-    set.get_many(ids.iter().cloned())?;
-
-    for id in ids {
-        let pkg = set.lookup_mut(id).expect("should be downloaded now");
-
-        for target in pkg.manifest_mut().targets_mut() {
-            if let TargetKind::Lib(crate_types) = target.kind_mut() {
-                crate_types.truncate(0);
-                crate_types.push(LibKind::Rlib);
-            }
-        }
-    }
-
-    Ok(())
-}
