@@ -1,25 +1,23 @@
-use crate::core::resolver::ResolveOpts;
+use crate::core::compiler::RustcTargetData;
+use crate::core::resolver::{HasDevUnits, ResolveOpts};
 use crate::core::{Shell, Workspace};
 use crate::ops;
 use crate::util::CargoResult;
-use failure::Fail;
-use opener;
 use std::collections::HashMap;
-use std::error::Error;
 use std::path::Path;
 use std::process::Command;
 
 /// Strongly typed options for the `cargo doc` command.
 #[derive(Debug)]
-pub struct DocOptions<'a> {
+pub struct DocOptions {
     /// Whether to attempt to open the browser after compiling the docs
     pub open_result: bool,
     /// Options to pass through to the compiler
-    pub compile_opts: ops::CompileOptions<'a>,
+    pub compile_opts: ops::CompileOptions,
 }
 
 /// Main method for `cargo doc`.
-pub fn doc(ws: &Workspace<'_>, options: &DocOptions<'_>) -> CargoResult<()> {
+pub fn doc(ws: &Workspace<'_>, options: &DocOptions) -> CargoResult<()> {
     let specs = options.compile_opts.spec.to_package_id_specs(ws)?;
     let opts = ResolveOpts::new(
         /*dev_deps*/ true,
@@ -27,12 +25,18 @@ pub fn doc(ws: &Workspace<'_>, options: &DocOptions<'_>) -> CargoResult<()> {
         options.compile_opts.all_features,
         !options.compile_opts.no_default_features,
     );
-    let ws_resolve = ops::resolve_ws_with_opts(ws, opts, &specs)?;
+    let target_data = RustcTargetData::new(ws, &options.compile_opts.build_config.requested_kinds)?;
+    let ws_resolve = ops::resolve_ws_with_opts(
+        ws,
+        &target_data,
+        &options.compile_opts.build_config.requested_kinds,
+        &opts,
+        &specs,
+        HasDevUnits::No,
+        crate::core::resolver::features::ForceAllTargets::No,
+    )?;
 
-    let ids = specs
-        .iter()
-        .map(|s| s.query(ws_resolve.targeted_resolve.iter()))
-        .collect::<CargoResult<Vec<_>>>()?;
+    let ids = ws_resolve.targeted_resolve.specs_to_ids(&specs)?;
     let pkgs = ws_resolve.pkg_set.get_many(ids)?;
 
     let mut lib_names = HashMap::new();
@@ -42,7 +46,7 @@ pub fn doc(ws: &Workspace<'_>, options: &DocOptions<'_>) -> CargoResult<()> {
         for target in package.targets().iter().filter(|t| t.documented()) {
             if target.is_lib() {
                 if let Some(prev) = lib_names.insert(target.crate_name(), package) {
-                    failure::bail!(
+                    anyhow::bail!(
                         "The library `{}` is specified by packages `{}` and \
                          `{}` but can only be documented once. Consider renaming \
                          or marking one of the targets as `doc = false`.",
@@ -52,7 +56,7 @@ pub fn doc(ws: &Workspace<'_>, options: &DocOptions<'_>) -> CargoResult<()> {
                     );
                 }
             } else if let Some(prev) = bin_names.insert(target.crate_name(), package) {
-                failure::bail!(
+                anyhow::bail!(
                     "The binary `{}` is specified by packages `{}` and \
                      `{}` but can be documented only once. Consider renaming \
                      or marking one of the targets as `doc = false`.",
@@ -65,20 +69,25 @@ pub fn doc(ws: &Workspace<'_>, options: &DocOptions<'_>) -> CargoResult<()> {
         }
     }
 
+    let open_kind = if options.open_result {
+        Some(options.compile_opts.build_config.single_requested_kind()?)
+    } else {
+        None
+    };
+
     let compilation = ops::compile(ws, &options.compile_opts)?;
 
-    if options.open_result {
+    if let Some(kind) = open_kind {
         let name = match names.first() {
             Some(s) => s.to_string(),
             None => return Ok(()),
         };
-        let path = compilation
-            .root_output
+        let path = compilation.root_output[&kind]
             .with_file_name("doc")
             .join(&name)
             .join("index.html");
         if path.exists() {
-            let mut shell = options.compile_opts.config.shell();
+            let mut shell = ws.config().shell();
             shell.status("Opening", path.display())?;
             open_docs(&path, &mut shell)?;
         }
@@ -94,16 +103,14 @@ fn open_docs(path: &Path, shell: &mut Shell) -> CargoResult<()> {
                 shell.warn(format!(
                     "Couldn't open docs with {}: {}",
                     browser.to_string_lossy(),
-                    e.description()
+                    e
                 ))?;
             }
         }
         None => {
             if let Err(e) = opener::open(&path) {
-                shell.warn(format!("Couldn't open docs: {}", e))?;
-                for cause in (&e as &dyn Fail).iter_chain() {
-                    shell.warn(format!("Caused by:\n {}", cause))?;
-                }
+                let e = e.into();
+                crate::display_warning_with_error("couldn't open docs", &e, shell);
             }
         }
     };
