@@ -60,7 +60,7 @@ pub fn read_packages(
             }
 
             // Don't automatically discover packages across git submodules
-            if fs::metadata(&dir.join(".git")).is_ok() {
+            if dir.join(".git").exists() {
                 return Ok(false);
             }
         }
@@ -112,7 +112,7 @@ fn walk(path: &Path, callback: &mut dyn FnMut(&Path) -> CargoResult<bool>) -> Ca
         Err(e) => {
             let cx = format!("failed to read directory `{}`", path.display());
             let e = anyhow::Error::from(e);
-            return Err(e.context(cx).into());
+            return Err(e.context(cx));
         }
     };
     for dir in dirs {
@@ -193,7 +193,26 @@ fn read_nested_packages(
     if !source_id.is_registry() {
         for p in nested.iter() {
             let path = util::normalize_path(&path.join(p));
-            read_nested_packages(&path, all_packages, source_id, config, visited, errors)?;
+            let result =
+                read_nested_packages(&path, all_packages, source_id, config, visited, errors);
+            // Ignore broken manifests found on git repositories.
+            //
+            // A well formed manifest might still fail to load due to reasons
+            // like referring to a "path" that requires an extra build step.
+            //
+            // See https://github.com/rust-lang/cargo/issues/6822.
+            if let Err(err) = result {
+                if source_id.is_git() {
+                    info!(
+                        "skipping nested package found at `{}`: {:?}",
+                        path.display(),
+                        &err,
+                    );
+                    errors.push(err);
+                } else {
+                    return Err(err);
+                }
+            }
         }
     }
 
