@@ -1,21 +1,21 @@
 use std::ffi::OsString;
 
-use crate::core::compiler::{Compilation, Doctest};
+use crate::core::compiler::{Compilation, CompileKind, Doctest};
 use crate::core::shell::Verbosity;
 use crate::core::Workspace;
 use crate::ops;
 use crate::util::errors::CargoResult;
-use crate::util::{CargoTestError, ProcessError, Test};
+use crate::util::{CargoTestError, Config, ProcessError, Test};
 
-pub struct TestOptions<'a> {
-    pub compile_opts: ops::CompileOptions<'a>,
+pub struct TestOptions {
+    pub compile_opts: ops::CompileOptions,
     pub no_run: bool,
     pub no_fail_fast: bool,
 }
 
 pub fn run_tests(
     ws: &Workspace<'_>,
-    options: &TestOptions<'_>,
+    options: &TestOptions,
     test_args: &[&str],
 ) -> CargoResult<Option<CargoTestError>> {
     let compilation = compile_tests(ws, options)?;
@@ -23,14 +23,14 @@ pub fn run_tests(
     if options.no_run {
         return Ok(None);
     }
-    let (test, mut errors) = run_unit_tests(options, test_args, &compilation)?;
+    let (test, mut errors) = run_unit_tests(ws.config(), options, test_args, &compilation)?;
 
     // If we have an error and want to fail fast, then return.
     if !errors.is_empty() && !options.no_fail_fast {
         return Ok(Some(CargoTestError::new(test, errors)));
     }
 
-    let (doctest, docerrors) = run_doc_tests(options, test_args, &compilation)?;
+    let (doctest, docerrors) = run_doc_tests(ws.config(), options, test_args, &compilation)?;
     let test = if docerrors.is_empty() { test } else { doctest };
     errors.extend(docerrors);
     if errors.is_empty() {
@@ -42,7 +42,7 @@ pub fn run_tests(
 
 pub fn run_benches(
     ws: &Workspace<'_>,
-    options: &TestOptions<'_>,
+    options: &TestOptions,
     args: &[&str],
 ) -> CargoResult<Option<CargoTestError>> {
     let compilation = compile_tests(ws, options)?;
@@ -54,7 +54,7 @@ pub fn run_benches(
     let mut args = args.to_vec();
     args.push("--bench");
 
-    let (test, errors) = run_unit_tests(options, &args, &compilation)?;
+    let (test, errors) = run_unit_tests(ws.config(), options, &args, &compilation)?;
 
     match errors.len() {
         0 => Ok(None),
@@ -62,35 +62,28 @@ pub fn run_benches(
     }
 }
 
-fn compile_tests<'a>(
-    ws: &Workspace<'a>,
-    options: &TestOptions<'a>,
-) -> CargoResult<Compilation<'a>> {
+fn compile_tests<'a>(ws: &Workspace<'a>, options: &TestOptions) -> CargoResult<Compilation<'a>> {
     let mut compilation = ops::compile(ws, &options.compile_opts)?;
-    compilation
-        .tests
-        .sort_by(|a, b| (a.0.package_id(), &a.1, &a.2).cmp(&(b.0.package_id(), &b.1, &b.2)));
+    compilation.tests.sort();
     Ok(compilation)
 }
 
 /// Runs the unit and integration tests of a package.
 fn run_unit_tests(
-    options: &TestOptions<'_>,
+    config: &Config,
+    options: &TestOptions,
     test_args: &[&str],
     compilation: &Compilation<'_>,
 ) -> CargoResult<(Test, Vec<ProcessError>)> {
-    let config = options.compile_opts.config;
-    let cwd = options.compile_opts.config.cwd();
-
+    let cwd = config.cwd();
     let mut errors = Vec::new();
 
-    for &(ref pkg, ref target, ref exe) in &compilation.tests {
-        let kind = target.kind();
-        let test = target.name().to_string();
+    for (unit, exe) in compilation.tests.iter() {
+        let test = unit.target.name().to_string();
         let exe_display = exe.strip_prefix(cwd).unwrap_or(exe).display();
-        let mut cmd = compilation.target_process(exe, pkg)?;
+        let mut cmd = compilation.target_process(exe, unit.kind, &unit.pkg)?;
         cmd.args(test_args);
-        if target.harness() && config.shell().verbosity() == Verbosity::Quiet {
+        if unit.target.harness() && config.shell().verbosity() == Verbosity::Quiet {
             cmd.arg("--quiet");
         }
         config
@@ -105,7 +98,12 @@ fn run_unit_tests(
         match result {
             Err(e) => {
                 let e = e.downcast::<ProcessError>()?;
-                errors.push((kind.clone(), test.clone(), pkg.name().to_string(), e));
+                errors.push((
+                    unit.target.kind().clone(),
+                    test.clone(),
+                    unit.pkg.name().to_string(),
+                    e,
+                ));
                 if !options.no_fail_fast {
                     break;
                 }
@@ -133,51 +131,65 @@ fn run_unit_tests(
 }
 
 fn run_doc_tests(
-    options: &TestOptions<'_>,
+    config: &Config,
+    options: &TestOptions,
     test_args: &[&str],
     compilation: &Compilation<'_>,
 ) -> CargoResult<(Test, Vec<ProcessError>)> {
     let mut errors = Vec::new();
-    let config = options.compile_opts.config;
-
-    // The unstable doctest-xcompile feature enables both per-target-ignores and
-    // cross-compiling doctests. As a side effect, this feature also gates running
-    // doctests with runtools when target == host.
     let doctest_xcompile = config.cli_unstable().doctest_xcompile;
-    let mut runtool: &Option<(std::path::PathBuf, Vec<String>)> = &None;
-    if doctest_xcompile {
-        runtool = compilation.target_runner();
-    } else if compilation.host != compilation.target {
-        return Ok((Test::Doc, errors));
-    }
 
     for doctest_info in &compilation.to_doc_test {
         let Doctest {
-            package,
-            target,
-            deps,
+            args,
+            unstable_opts,
+            unit,
+            linker,
         } = doctest_info;
-        config.shell().status("Doc-tests", target.name())?;
-        let mut p = compilation.rustdoc_process(package, target)?;
-        p.arg("--test")
-            .arg(target.src_path().path().unwrap())
-            .arg("--crate-name")
-            .arg(&target.crate_name());
 
-        if doctest_xcompile {
-            p.arg("--target").arg(&compilation.target);
-            p.arg("-Zunstable-options");
-            p.arg("--enable-per-target-ignores");
-        }
-
-        if let Some((runtool, runtool_args)) = runtool {
-            p.arg("--runtool").arg(runtool);
-            for arg in runtool_args {
-                p.arg("--runtool-arg").arg(arg);
+        if !doctest_xcompile {
+            match unit.kind {
+                CompileKind::Host => {}
+                CompileKind::Target(target) => {
+                    if target.short_name() != compilation.host {
+                        // Skip doctests, -Zdoctest-xcompile not enabled.
+                        continue;
+                    }
+                }
             }
         }
 
-        for &rust_dep in &[&compilation.deps_output] {
+        config.shell().status("Doc-tests", unit.target.name())?;
+        let mut p = compilation.rustdoc_process(unit)?;
+        p.arg("--test")
+            .arg(unit.target.src_path().path().unwrap())
+            .arg("--crate-name")
+            .arg(&unit.target.crate_name());
+
+        if doctest_xcompile {
+            if let CompileKind::Target(target) = unit.kind {
+                // use `rustc_target()` to properly handle JSON target paths
+                p.arg("--target").arg(target.rustc_target());
+            }
+            p.arg("-Zunstable-options");
+            p.arg("--enable-per-target-ignores");
+            if let Some((runtool, runtool_args)) = compilation.target_runner(unit.kind) {
+                p.arg("--runtool").arg(runtool);
+                for arg in runtool_args {
+                    p.arg("--runtool-arg").arg(arg);
+                }
+            }
+            if let Some(linker) = linker {
+                let mut joined = OsString::from("linker=");
+                joined.push(linker);
+                p.arg("-C").arg(joined);
+            }
+        }
+
+        for &rust_dep in &[
+            &compilation.deps_output[&unit.kind],
+            &compilation.deps_output[&CompileKind::Host],
+        ] {
             let mut arg = OsString::from("dependency=");
             arg.push(rust_dep);
             p.arg("-L").arg(arg);
@@ -187,30 +199,25 @@ fn run_doc_tests(
             p.arg("-L").arg(native_dep);
         }
 
-        for &host_rust_dep in &[&compilation.host_deps_output] {
-            let mut arg = OsString::from("dependency=");
-            arg.push(host_rust_dep);
-            p.arg("-L").arg(arg);
-        }
-
         for arg in test_args {
             p.arg("--test-args").arg(arg);
         }
 
-        if let Some(cfgs) = compilation.cfgs.get(&package.package_id()) {
+        if let Some(cfgs) = compilation.cfgs.get(&unit.pkg.package_id()) {
             for cfg in cfgs.iter() {
                 p.arg("--cfg").arg(cfg);
             }
         }
 
-        for &(ref extern_crate_name, ref lib) in deps.iter() {
-            let mut arg = OsString::from(extern_crate_name.as_str());
-            arg.push("=");
-            arg.push(lib);
-            p.arg("--extern").arg(&arg);
+        for arg in args {
+            p.arg(arg);
         }
 
-        if let Some(flags) = compilation.rustdocflags.get(&package.package_id()) {
+        if *unstable_opts {
+            p.arg("-Zunstable-options");
+        }
+
+        if let Some(flags) = compilation.rustdocflags.get(&unit.pkg.package_id()) {
             p.args(flags);
         }
         config
