@@ -1,7 +1,5 @@
 use std::io::prelude::*;
 
-use toml;
-
 use crate::core::{resolver, Resolve, ResolveVersion, Workspace};
 use crate::util::errors::{CargoResult, CargoResultExt};
 use crate::util::toml as cargo_toml;
@@ -47,7 +45,7 @@ pub fn write_pkg_lockfile(ws: &Workspace<'_>, resolve: &Resolve) -> CargoResult<
 
     if !ws.config().lock_update_allowed() {
         if ws.config().offline() {
-            failure::bail!("can't update in the offline mode");
+            anyhow::bail!("can't update in the offline mode");
         }
 
         let flag = if ws.config().network_allowed() {
@@ -55,9 +53,10 @@ pub fn write_pkg_lockfile(ws: &Workspace<'_>, resolve: &Resolve) -> CargoResult<
         } else {
             "--frozen"
         };
-        failure::bail!(
-            "the lock file {} needs to be updated but {} was passed to \
-             prevent this",
+        anyhow::bail!(
+            "the lock file {} needs to be updated but {} was passed to prevent this\n\
+             If you want to try to generate the lock file without accessing the network, \
+             use the --offline flag.",
             ws.root().to_path_buf().join("Cargo.lock").display(),
             flag
         );
