@@ -1,5 +1,7 @@
 pub use self::cargo_clean::{clean, CleanOptions};
-pub use self::cargo_compile::{compile, compile_with_exec, compile_ws, CompileOptions};
+pub use self::cargo_compile::{
+    compile, compile_with_exec, compile_ws, create_bcx, resolve_all_features, CompileOptions,
+};
 pub use self::cargo_compile::{CompileFilter, FilterRule, LibRule, Packages};
 pub use self::cargo_doc::{doc, DocOptions};
 pub use self::cargo_fetch::{fetch, FetchOptions};
@@ -46,4 +48,5 @@ mod fix;
 mod lockfile;
 mod registry;
 mod resolve;
+pub mod tree;
 mod vendor;
