@@ -4,35 +4,38 @@ use crate::ops;
 use crate::sources::path::PathSource;
 use crate::util::Sha256;
 use crate::util::{paths, CargoResult, CargoResultExt, Config};
-use failure::bail;
+use anyhow::bail;
 use serde::Serialize;
 use std::collections::HashSet;
 use std::collections::{BTreeMap, BTreeSet, HashMap};
-use std::fs::{self, File};
-use std::io::Write;
+use std::fs;
 use std::path::{Path, PathBuf};
 
 pub struct VendorOptions<'a> {
     pub no_delete: bool,
+    pub versioned_dirs: bool,
     pub destination: &'a Path,
     pub extra: Vec<PathBuf>,
 }
 
 pub fn vendor(ws: &Workspace<'_>, opts: &VendorOptions<'_>) -> CargoResult<()> {
+    let config = ws.config();
     let mut extra_workspaces = Vec::new();
     for extra in opts.extra.iter() {
-        let extra = ws.config().cwd().join(extra);
-        let ws = Workspace::new(&extra, ws.config())?;
+        let extra = config.cwd().join(extra);
+        let ws = Workspace::new(&extra, config)?;
         extra_workspaces.push(ws);
     }
     let workspaces = extra_workspaces.iter().chain(Some(ws)).collect::<Vec<_>>();
     let vendor_config =
-        sync(ws.config(), &workspaces, opts).chain_err(|| "failed to sync".to_string())?;
+        sync(config, &workspaces, opts).chain_err(|| "failed to sync".to_string())?;
 
-    let shell = ws.config().shell();
-    if shell.verbosity() != Verbosity::Quiet {
-        eprint!("To use vendored sources, add this to your .cargo/config for this project:\n\n");
-        print!("{}", &toml::to_string(&vendor_config).unwrap());
+    if config.shell().verbosity() != Verbosity::Quiet {
+        crate::drop_eprint!(
+            config,
+            "To use vendored sources, add this to your .cargo/config for this project:\n\n"
+        );
+        crate::drop_print!(config, "{}", &toml::to_string(&vendor_config).unwrap());
     }
 
     Ok(())
@@ -186,7 +189,7 @@ fn sync(
             .parent()
             .expect("manifest_path should point to a file");
         let max_version = *versions[&id.name()].iter().rev().next().unwrap().0;
-        let dir_has_version_suffix = id.version() != max_version;
+        let dir_has_version_suffix = opts.versioned_dirs || id.version() != max_version;
         let dst_name = if dir_has_version_suffix {
             // Eg vendor/futures-0.1.13
             format!("{}-{}", id.name(), id.version())
@@ -222,7 +225,7 @@ fn sync(
             "files": map,
         });
 
-        File::create(&cksum)?.write_all(json.to_string().as_bytes())?;
+        paths::write(&cksum, json.to_string())?;
     }
 
     for path in to_remove {
@@ -257,6 +260,12 @@ fn sync(
                 registry: None,
                 replace_with: merged_source_name.to_string(),
             }
+        } else if source_id.is_remote_registry() {
+            let registry = source_id.url().to_string();
+            VendorSource::Registry {
+                registry: Some(registry),
+                replace_with: merged_source_name.to_string(),
+            }
         } else if source_id.is_git() {
             let mut branch = None;
             let mut tag = None;
@@ -324,8 +333,7 @@ fn cp_sources(
 
         paths::create_dir_all(dst.parent().unwrap())?;
 
-        fs::copy(&p, &dst)
-            .chain_err(|| format!("failed to copy `{}` to `{}`", p.display(), dst.display()))?;
+        paths::copy(&p, &dst)?;
         let cksum = Sha256::new().update_path(dst)?.finish_hex();
         cksums.insert(relative.to_str().unwrap().replace("\\", "/"), cksum);
     }
