@@ -3,19 +3,19 @@ use crate::core::Workspace;
 use crate::ops::{CompileFilter, CompileOptions, NewOptions, Packages, VersionControl};
 use crate::sources::CRATES_IO_REGISTRY;
 use crate::util::important_paths::find_root_manifest_for_wd;
+use crate::util::interning::InternedString;
 use crate::util::{paths, toml::TomlProfile, validate_package_name};
 use crate::util::{
     print_available_benches, print_available_binaries, print_available_examples,
     print_available_tests,
 };
 use crate::CargoResult;
+use anyhow::bail;
 use clap::{self, SubCommand};
-use failure::bail;
 use std::ffi::{OsStr, OsString};
-use std::fs;
 use std::path::PathBuf;
 
-pub use crate::core::compiler::{CompileMode, ProfileKind};
+pub use crate::core::compiler::CompileMode;
 pub use crate::{CliError, CliResult, Config};
 pub use clap::{AppSettings, Arg, ArgMatches};
 
@@ -36,6 +36,20 @@ pub trait AppExt: Sized {
             ._arg(multi_opt("exclude", "SPEC", exclude))
     }
 
+    /// Variant of arg_package_spec that does not include the `--all` flag
+    /// (but does include `--workspace`). Used to avoid confusion with
+    /// historical uses of `--all`.
+    fn arg_package_spec_no_all(
+        self,
+        package: &'static str,
+        all: &'static str,
+        exclude: &'static str,
+    ) -> Self {
+        self.arg_package_spec_simple(package)
+            ._arg(opt("workspace", all))
+            ._arg(multi_opt("exclude", "SPEC", exclude))
+    }
+
     fn arg_package_spec_simple(self, package: &'static str) -> Self {
         self._arg(multi_opt("package", "SPEC", package).short("p"))
     }
@@ -103,7 +117,7 @@ pub trait AppExt: Sized {
         self._arg(multi_opt(
             "features",
             "FEATURES",
-            "Space-separated list of features to activate",
+            "Space or comma separated list of features to activate",
         ))
         ._arg(opt("all-features", "Activate all available features"))
         ._arg(opt(
@@ -125,7 +139,7 @@ pub trait AppExt: Sized {
     }
 
     fn arg_target_triple(self, target: &'static str) -> Self {
-        self._arg(opt("target", target).value_name("TRIPLE"))
+        self._arg(multi_opt("target", target, "TRIPLE"))
     }
 
     fn arg_target_dir(self) -> Self {
@@ -149,6 +163,10 @@ pub trait AppExt: Sized {
         ))
     }
 
+    fn arg_unit_graph(self) -> Self {
+        self._arg(opt("unit-graph", "Output build graph in JSON (unstable)").hidden(true))
+    }
+
     fn arg_new_opts(self) -> Self {
         self._arg(
             opt(
@@ -264,10 +282,10 @@ pub trait ArgMatchesExt {
             // but in this particular case we need it to fix #3586.
             let path = paths::normalize_path(&path);
             if !path.ends_with("Cargo.toml") {
-                failure::bail!("the manifest-path must be a path to a Cargo.toml file")
+                anyhow::bail!("the manifest-path must be a path to a Cargo.toml file")
             }
-            if fs::metadata(&path).is_err() {
-                failure::bail!(
+            if !path.exists() {
+                anyhow::bail!(
                     "manifest path `{}` does not exist",
                     self._value_of("manifest-path").unwrap()
                 )
@@ -283,13 +301,14 @@ pub trait ArgMatchesExt {
         if config.cli_unstable().avoid_dev_deps {
             ws.set_require_optional_deps(false);
         }
-        if ws.is_virtual() && !config.cli_unstable().package_features {
+        if ws.is_virtual() && !ws.allows_unstable_package_features() {
             // --all-features is actually honored. In general, workspaces and
             // feature flags are a bit of a mess right now.
             for flag in &["features", "no-default-features"] {
                 if self._is_present(flag) {
                     bail!(
-                        "--{} is not allowed in the root of a virtual workspace",
+                        "--{} is not allowed in the root of a virtual workspace\n\
+                         note: while this was previously accepted, it didn't actually do anything",
                         flag
                     );
                 }
@@ -302,23 +321,21 @@ pub trait ArgMatchesExt {
         self.value_of_u32("jobs")
     }
 
-    fn target(&self) -> Option<String> {
-        self._value_of("target").map(|s| s.to_string())
+    fn targets(&self) -> Vec<String> {
+        self._values_of("target")
     }
 
-    fn get_profile_kind(
+    fn get_profile_name(
         &self,
         config: &Config,
-        default: ProfileKind,
+        default: &str,
         profile_checking: ProfileChecking,
-    ) -> CargoResult<ProfileKind> {
+    ) -> CargoResult<InternedString> {
         let specified_profile = match self._value_of("profile") {
             None => None,
-            Some("dev") => Some(ProfileKind::Dev),
-            Some("release") => Some(ProfileKind::Release),
             Some(name) => {
                 TomlProfile::validate_name(name, "profile name")?;
-                Some(ProfileKind::Custom(name.to_string()))
+                Some(InternedString::new(name))
             }
         };
 
@@ -326,48 +343,55 @@ pub trait ArgMatchesExt {
             ProfileChecking::Unchecked => {}
             ProfileChecking::Checked => {
                 if specified_profile.is_some() && !config.cli_unstable().unstable_options {
-                    failure::bail!("Usage of `--profile` requires `-Z unstable-options`")
+                    anyhow::bail!("Usage of `--profile` requires `-Z unstable-options`")
                 }
             }
         }
 
         if self._is_present("release") {
             if !config.cli_unstable().unstable_options {
-                Ok(ProfileKind::Release)
+                Ok(InternedString::new("release"))
             } else {
                 match specified_profile {
-                    None | Some(ProfileKind::Release) => Ok(ProfileKind::Release),
-                    _ => failure::bail!("Conflicting usage of --profile and --release"),
+                    Some(name) if name != "release" => {
+                        anyhow::bail!("Conflicting usage of --profile and --release")
+                    }
+                    _ => Ok(InternedString::new("release")),
                 }
             }
         } else if self._is_present("debug") {
             if !config.cli_unstable().unstable_options {
-                Ok(ProfileKind::Dev)
+                Ok(InternedString::new("dev"))
             } else {
                 match specified_profile {
-                    None | Some(ProfileKind::Dev) => Ok(ProfileKind::Dev),
-                    _ => failure::bail!("Conflicting usage of --profile and --debug"),
+                    Some(name) if name != "dev" => {
+                        anyhow::bail!("Conflicting usage of --profile and --debug")
+                    }
+                    _ => Ok(InternedString::new("dev")),
                 }
             }
         } else {
-            Ok(specified_profile.unwrap_or(default))
+            Ok(specified_profile.unwrap_or_else(|| InternedString::new(default)))
         }
     }
 
-    fn compile_options<'a>(
-        &self,
-        config: &'a Config,
-        mode: CompileMode,
-        workspace: Option<&Workspace<'a>>,
-        profile_checking: ProfileChecking,
-    ) -> CargoResult<CompileOptions<'a>> {
-        let spec = Packages::from_flags(
+    fn packages_from_flags(&self) -> CargoResult<Packages> {
+        Packages::from_flags(
             // TODO Integrate into 'workspace'
             self._is_present("workspace") || self._is_present("all"),
             self._values_of("exclude"),
             self._values_of("package"),
-        )?;
+        )
+    }
 
+    fn compile_options(
+        &self,
+        config: &Config,
+        mode: CompileMode,
+        workspace: Option<&Workspace<'_>>,
+        profile_checking: ProfileChecking,
+    ) -> CargoResult<CompileOptions> {
+        let spec = self.packages_from_flags()?;
         let mut message_format = None;
         let default_json = MessageFormat::Json {
             short: false,
@@ -430,19 +454,23 @@ pub trait ArgMatchesExt {
             }
         }
 
-        let mut build_config = BuildConfig::new(config, self.jobs()?, &self.target(), mode)?;
+        let mut build_config = BuildConfig::new(config, self.jobs()?, &self.targets(), mode)?;
         build_config.message_format = message_format.unwrap_or(MessageFormat::Human);
-        build_config.profile_kind =
-            self.get_profile_kind(config, ProfileKind::Dev, profile_checking)?;
+        build_config.requested_profile = self.get_profile_name(config, "dev", profile_checking)?;
         build_config.build_plan = self._is_present("build-plan");
+        build_config.unit_graph = self._is_present("unit-graph");
         if build_config.build_plan {
             config
                 .cli_unstable()
                 .fail_if_stable_opt("--build-plan", 5579)?;
         };
+        if build_config.unit_graph {
+            config
+                .cli_unstable()
+                .fail_if_stable_opt("--unit-graph", 8002)?;
+        }
 
         let opts = CompileOptions {
-            config,
             build_config,
             features: self._values_of("features"),
             all_features: self._is_present("all-features"),
@@ -464,7 +492,6 @@ pub trait ArgMatchesExt {
             target_rustc_args: None,
             local_rustdoc_args: None,
             rustdoc_document_private_items: false,
-            export_dir: None,
         };
 
         if let Some(ws) = workspace {
@@ -474,13 +501,13 @@ pub trait ArgMatchesExt {
         Ok(opts)
     }
 
-    fn compile_options_for_single_package<'a>(
+    fn compile_options_for_single_package(
         &self,
-        config: &'a Config,
+        config: &Config,
         mode: CompileMode,
-        workspace: Option<&Workspace<'a>>,
+        workspace: Option<&Workspace<'_>>,
         profile_checking: ProfileChecking,
-    ) -> CargoResult<CompileOptions<'a>> {
+    ) -> CargoResult<CompileOptions> {
         let mut compile_opts = self.compile_options(config, mode, workspace, profile_checking)?;
         compile_opts.spec = Packages::Packages(self._values_of("package"));
         Ok(compile_opts)
@@ -553,7 +580,7 @@ about this warning.";
     fn check_optional_opts(
         &self,
         workspace: &Workspace<'_>,
-        compile_opts: &CompileOptions<'_>,
+        compile_opts: &CompileOptions,
     ) -> CargoResult<()> {
         if self.is_present_with_zero_values("example") {
             print_available_examples(workspace, compile_opts)?;
