@@ -1,21 +1,18 @@
 #![allow(unknown_lints)]
 
+use crate::core::{TargetKind, Workspace};
+use crate::ops::CompileOptions;
+use anyhow::Error;
 use std::fmt;
 use std::path::PathBuf;
 use std::process::{ExitStatus, Output};
 use std::str;
 
-use clap;
-use failure::{Context, Error, Fail};
-use log::trace;
-
-use crate::core::{TargetKind, Workspace};
-use crate::ops::CompileOptions;
-
-pub type CargoResult<T> = failure::Fallible<T>; // Alex's body isn't quite ready to give up "Result"
+pub type CargoResult<T> = anyhow::Result<T>;
 
+// TODO: should delete this trait and just use `with_context` instead
 pub trait CargoResultExt<T, E> {
-    fn chain_err<F, D>(self, f: F) -> Result<T, Context<D>>
+    fn chain_err<F, D>(self, f: F) -> CargoResult<T>
     where
         F: FnOnce() -> D,
         D: fmt::Display + Send + Sync + 'static;
@@ -25,56 +22,108 @@ impl<T, E> CargoResultExt<T, E> for Result<T, E>
 where
     E: Into<Error>,
 {
-    fn chain_err<F, D>(self, f: F) -> Result<T, Context<D>>
+    fn chain_err<F, D>(self, f: F) -> CargoResult<T>
     where
         F: FnOnce() -> D,
         D: fmt::Display + Send + Sync + 'static,
     {
-        self.map_err(|failure| {
-            let err = failure.into();
-            let context = f();
-            trace!("error: {}", err);
-            trace!("\tcontext: {}", context);
-            err.context(context)
-        })
+        self.map_err(|e| e.into().context(f()))
     }
 }
 
-#[derive(Debug, Fail)]
-#[fail(display = "failed to get 200 response from `{}`, got {}", url, code)]
+#[derive(Debug)]
 pub struct HttpNot200 {
     pub code: u32,
     pub url: String,
 }
 
-pub struct Internal {
+impl fmt::Display for HttpNot200 {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(
+            f,
+            "failed to get 200 response from `{}`, got {}",
+            self.url, self.code
+        )
+    }
+}
+
+impl std::error::Error for HttpNot200 {}
+
+// =============================================================================
+// Verbose error
+
+/// An error wrapper for errors that should only be displayed with `--verbose`.
+///
+/// This should only be used in rare cases. When emitting this error, you
+/// should have a normal error higher up the error-cause chain (like "could
+/// not compile `foo`"), so at least *something* gets printed without
+/// `--verbose`.
+pub struct VerboseError {
     inner: Error,
 }
 
-impl Internal {
-    pub fn new(inner: Error) -> Internal {
-        Internal { inner }
+impl VerboseError {
+    pub fn new(inner: Error) -> VerboseError {
+        VerboseError { inner }
     }
 }
 
-impl Fail for Internal {
-    fn cause(&self) -> Option<&dyn Fail> {
-        self.inner.as_fail().cause()
+impl std::error::Error for VerboseError {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+        self.inner.source()
     }
 }
 
-impl fmt::Debug for Internal {
+impl fmt::Debug for VerboseError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         self.inner.fmt(f)
     }
 }
 
-impl fmt::Display for Internal {
+impl fmt::Display for VerboseError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         self.inner.fmt(f)
     }
 }
 
+// =============================================================================
+// Internal error
+
+/// An unexpected, internal error.
+///
+/// This should only be used for unexpected errors. It prints a message asking
+/// the user to file a bug report.
+pub struct InternalError {
+    inner: Error,
+}
+
+impl InternalError {
+    pub fn new(inner: Error) -> InternalError {
+        InternalError { inner }
+    }
+}
+
+impl std::error::Error for InternalError {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+        self.inner.source()
+    }
+}
+
+impl fmt::Debug for InternalError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.inner.fmt(f)
+    }
+}
+
+impl fmt::Display for InternalError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.inner.fmt(f)
+    }
+}
+
+// =============================================================================
+// Manifest error
+
 /// Error wrapper related to a particular manifest and providing it's path.
 ///
 /// This error adds no displayable info of it's own.
@@ -103,9 +152,9 @@ impl ManifestError {
     }
 }
 
-impl Fail for ManifestError {
-    fn cause(&self) -> Option<&dyn Fail> {
-        self.cause.as_fail().cause()
+impl std::error::Error for ManifestError {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+        self.cause.source()
     }
 }
 
@@ -139,20 +188,33 @@ impl<'a> ::std::iter::FusedIterator for ManifestCauses<'a> {}
 
 // =============================================================================
 // Process errors
-#[derive(Debug, Fail)]
-#[fail(display = "{}", desc)]
+#[derive(Debug)]
 pub struct ProcessError {
+    /// A detailed description to show to the user why the process failed.
     pub desc: String,
+    /// The exit status of the process.
+    ///
+    /// This can be `None` if the process failed to launch (like process not found).
     pub exit: Option<ExitStatus>,
+    /// The output from the process.
+    ///
+    /// This can be `None` if the process failed to launch, or the output was not captured.
     pub output: Option<Output>,
 }
 
+impl fmt::Display for ProcessError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.desc.fmt(f)
+    }
+}
+
+impl std::error::Error for ProcessError {}
+
 // =============================================================================
 // Cargo test errors.
 
 /// Error when testcases fail
-#[derive(Debug, Fail)]
-#[fail(display = "{}", desc)]
+#[derive(Debug)]
 pub struct CargoTestError {
     pub test: Test,
     pub desc: String,
@@ -160,6 +222,14 @@ pub struct CargoTestError {
     pub causes: Vec<ProcessError>,
 }
 
+impl fmt::Display for CargoTestError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.desc.fmt(f)
+    }
+}
+
+impl std::error::Error for CargoTestError {}
+
 #[derive(Debug)]
 pub enum Test {
     Multiple,
@@ -189,7 +259,7 @@ impl CargoTestError {
         }
     }
 
-    pub fn hint(&self, ws: &Workspace<'_>, opts: &CompileOptions<'_>) -> String {
+    pub fn hint(&self, ws: &Workspace<'_>, opts: &CompileOptions) -> String {
         match self.test {
             Test::UnitTest {
                 ref kind,
@@ -231,19 +301,27 @@ impl CargoTestError {
 pub type CliResult = Result<(), CliError>;
 
 #[derive(Debug)]
+/// The CLI error is the error type used at Cargo's CLI-layer.
+///
+/// All errors from the lib side of Cargo will get wrapped with this error.
+/// Other errors (such as command-line argument validation) will create this
+/// directly.
 pub struct CliError {
-    pub error: Option<failure::Error>,
-    pub unknown: bool,
+    /// The error to display. This can be `None` in rare cases to exit with a
+    /// code without displaying a message. For example `cargo run -q` where
+    /// the resulting process exits with a nonzero code (on Windows), or an
+    /// external subcommand that exits nonzero (we assume it printed its own
+    /// message).
+    pub error: Option<anyhow::Error>,
+    /// The process exit code.
     pub exit_code: i32,
 }
 
 impl CliError {
-    pub fn new(error: failure::Error, code: i32) -> CliError {
-        let unknown = error.downcast_ref::<Internal>().is_some();
+    pub fn new(error: anyhow::Error, code: i32) -> CliError {
         CliError {
             error: Some(error),
             exit_code: code,
-            unknown,
         }
     }
 
@@ -251,13 +329,12 @@ impl CliError {
         CliError {
             error: None,
             exit_code: code,
-            unknown: false,
         }
     }
 }
 
-impl From<failure::Error> for CliError {
-    fn from(err: failure::Error) -> CliError {
+impl From<anyhow::Error> for CliError {
+    fn from(err: anyhow::Error) -> CliError {
         CliError::new(err, 101)
     }
 }
@@ -272,6 +349,10 @@ impl From<clap::Error> for CliError {
 // =============================================================================
 // Construction helpers
 
+/// Creates a new process error.
+///
+/// `status` can be `None` if the process did not launch.
+/// `output` can be `None` if the process did not launch, or output was not captured.
 pub fn process_error(
     msg: &str,
     status: Option<ExitStatus>,
@@ -392,18 +473,6 @@ pub fn is_simple_exit_code(code: i32) -> bool {
     code >= 0 && code <= 127
 }
 
-pub fn internal<S: fmt::Display>(error: S) -> failure::Error {
-    _internal(&error)
-}
-
-fn _internal(error: &dyn fmt::Display) -> failure::Error {
-    Internal::new(failure::format_err!("{}", error)).into()
-}
-
-pub fn display_causes(error: &Error) -> String {
-    error
-        .iter_chain()
-        .map(|e| e.to_string())
-        .collect::<Vec<_>>()
-        .join("\n\nCaused by:\n  ")
+pub fn internal<S: fmt::Display>(error: S) -> anyhow::Error {
+    InternalError::new(anyhow::format_err!("{}", error)).into()
 }
