@@ -1,6 +1,5 @@
 use crate::util::errors::CargoResult;
 use crate::util::paths;
-use std::fs;
 use std::path::{Path, PathBuf};
 
 /// Finds the root `Cargo.toml`.
@@ -8,12 +7,12 @@ pub fn find_root_manifest_for_wd(cwd: &Path) -> CargoResult<PathBuf> {
     let file = "Cargo.toml";
     for current in paths::ancestors(cwd) {
         let manifest = current.join(file);
-        if fs::metadata(&manifest).is_ok() {
+        if manifest.exists() {
             return Ok(manifest);
         }
     }
 
-    failure::bail!(
+    anyhow::bail!(
         "could not find `{}` in `{}` or any parent directory",
         file,
         cwd.display()
@@ -27,6 +26,6 @@ pub fn find_project_manifest_exact(pwd: &Path, file: &str) -> CargoResult<PathBu
     if manifest.exists() {
         Ok(manifest)
     } else {
-        failure::bail!("Could not find `{}` in `{}`", file, pwd.display())
+        anyhow::bail!("Could not find `{}` in `{}`", file, pwd.display())
     }
 }
