@@ -1,20 +1,20 @@
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 
 use serde::ser;
 use serde::Serialize;
 use serde_json::{self, json, value::RawValue};
 
-use crate::core::{PackageId, Target};
+use crate::core::{compiler::CompileMode, PackageId, Target};
 
 pub trait Message: ser::Serialize {
     fn reason(&self) -> &str;
-}
 
-pub fn emit<T: Message>(t: &T) {
-    let json = serde_json::to_string(t).unwrap();
-    assert!(json.starts_with("{\""));
-    let reason = json!(t.reason());
-    println!("{{\"reason\":{},{}", reason, &json[1..]);
+    fn to_json_string(&self) -> String {
+        let json = serde_json::to_string(self).unwrap();
+        assert!(json.starts_with("{\""));
+        let reason = json!(self.reason());
+        format!("{{\"reason\":{},{}", reason, &json[1..])
+    }
 }
 
 #[derive(Serialize)]
@@ -66,6 +66,7 @@ pub struct BuildScript<'a> {
     pub linked_paths: &'a [String],
     pub cfgs: &'a [String],
     pub env: &'a [(String, String)],
+    pub out_dir: &'a Path,
 }
 
 impl<'a> Message for BuildScript<'a> {
@@ -73,3 +74,30 @@ impl<'a> Message for BuildScript<'a> {
         "build-script-executed"
     }
 }
+
+#[derive(Serialize)]
+pub struct TimingInfo<'a> {
+    pub package_id: PackageId,
+    pub target: &'a Target,
+    pub mode: CompileMode,
+    pub duration: f64,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub rmeta_time: Option<f64>,
+}
+
+impl<'a> Message for TimingInfo<'a> {
+    fn reason(&self) -> &str {
+        "timing-info"
+    }
+}
+
+#[derive(Serialize)]
+pub struct BuildFinished {
+    pub success: bool,
+}
+
+impl Message for BuildFinished {
+    fn reason(&self) -> &str {
+        "build-finished"
+    }
+}
