@@ -7,27 +7,23 @@ use std::iter;
 use std::path::{Component, Path, PathBuf};
 
 use filetime::FileTime;
+use tempfile::Builder as TempFileBuilder;
 
-use crate::util::errors::{CargoResult, CargoResultExt, Internal};
+use crate::util::errors::{CargoResult, CargoResultExt};
 
 pub fn join_paths<T: AsRef<OsStr>>(paths: &[T], env: &str) -> CargoResult<OsString> {
-    let err = match env::join_paths(paths.iter()) {
-        Ok(paths) => return Ok(paths),
-        Err(e) => e,
-    };
-    let paths = paths.iter().map(Path::new).collect::<Vec<_>>();
-    let err = failure::Error::from(err);
-    let explain = Internal::new(failure::format_err!(
-        "failed to join path array: {:?}",
-        paths
-    ));
-    let err = failure::Error::from(err.context(explain));
-    let more_explain = format!(
-        "failed to join search paths together\n\
-         Does ${} have an unterminated quote character?",
-        env
-    );
-    Err(err.context(more_explain).into())
+    env::join_paths(paths.iter())
+        .chain_err(|| {
+            let paths = paths.iter().map(Path::new).collect::<Vec<_>>();
+            format!("failed to join path array: {:?}", paths)
+        })
+        .chain_err(|| {
+            format!(
+                "failed to join search paths together\n\
+                     Does ${} have an unterminated quote character?",
+                env
+            )
+        })
 }
 
 pub fn dylib_path_envvar() -> &'static str {
@@ -91,7 +87,7 @@ pub fn normalize_path(path: &Path) -> PathBuf {
 
 pub fn resolve_executable(exec: &Path) -> CargoResult<PathBuf> {
     if exec.components().count() == 1 {
-        let paths = env::var_os("PATH").ok_or_else(|| failure::format_err!("no PATH"))?;
+        let paths = env::var_os("PATH").ok_or_else(|| anyhow::format_err!("no PATH"))?;
         let candidates = env::split_paths(&paths).flat_map(|path| {
             let candidate = path.join(&exec);
             let with_exe = if env::consts::EXE_EXTENSION == "" {
@@ -109,7 +105,7 @@ pub fn resolve_executable(exec: &Path) -> CargoResult<PathBuf> {
             }
         }
 
-        failure::bail!("no executable for `{}` found in PATH", exec.display())
+        anyhow::bail!("no executable for `{}` found in PATH", exec.display())
     } else {
         Ok(exec.canonicalize()?)
     }
@@ -118,32 +114,17 @@ pub fn resolve_executable(exec: &Path) -> CargoResult<PathBuf> {
 pub fn read(path: &Path) -> CargoResult<String> {
     match String::from_utf8(read_bytes(path)?) {
         Ok(s) => Ok(s),
-        Err(_) => failure::bail!("path at `{}` was not valid utf-8", path.display()),
+        Err(_) => anyhow::bail!("path at `{}` was not valid utf-8", path.display()),
     }
 }
 
 pub fn read_bytes(path: &Path) -> CargoResult<Vec<u8>> {
-    let res = (|| -> CargoResult<_> {
-        let mut ret = Vec::new();
-        let mut f = File::open(path)?;
-        if let Ok(m) = f.metadata() {
-            ret.reserve(m.len() as usize + 1);
-        }
-        f.read_to_end(&mut ret)?;
-        Ok(ret)
-    })()
-    .chain_err(|| format!("failed to read `{}`", path.display()))?;
-    Ok(res)
+    fs::read(path).chain_err(|| format!("failed to read `{}`", path.display()))
 }
 
-pub fn write(path: &Path, contents: &[u8]) -> CargoResult<()> {
-    (|| -> CargoResult<()> {
-        let mut f = File::create(path)?;
-        f.write_all(contents)?;
-        Ok(())
-    })()
-    .chain_err(|| format!("failed to write `{}`", path.display()))?;
-    Ok(())
+pub fn write<P: AsRef<Path>, C: AsRef<[u8]>>(path: P, contents: C) -> CargoResult<()> {
+    let path = path.as_ref();
+    fs::write(path, contents.as_ref()).chain_err(|| format!("failed to write `{}`", path.display()))
 }
 
 pub fn write_if_changed<P: AsRef<Path>, C: AsRef<[u8]>>(path: P, contents: C) -> CargoResult<()> {
@@ -182,6 +163,18 @@ pub fn append(path: &Path, contents: &[u8]) -> CargoResult<()> {
     Ok(())
 }
 
+/// Creates a new file.
+pub fn create<P: AsRef<Path>>(path: P) -> CargoResult<File> {
+    let path = path.as_ref();
+    File::create(path).chain_err(|| format!("failed to create file `{}`", path.display()))
+}
+
+/// Opens an existing file.
+pub fn open<P: AsRef<Path>>(path: P) -> CargoResult<File> {
+    let path = path.as_ref();
+    File::open(path).chain_err(|| format!("failed to open file `{}`", path.display()))
+}
+
 pub fn mtime(path: &Path) -> CargoResult<FileTime> {
     let meta = fs::metadata(path).chain_err(|| format!("failed to stat `{}`", path.display()))?;
     Ok(FileTime::from_last_modification_time(&meta))
@@ -195,9 +188,11 @@ pub fn set_invocation_time(path: &Path) -> CargoResult<FileTime> {
     let timestamp = path.join("invoked.timestamp");
     write(
         &timestamp,
-        b"This file has an mtime of when this was started.",
+        "This file has an mtime of when this was started.",
     )?;
-    mtime(&timestamp)
+    let ft = mtime(&timestamp)?;
+    log::debug!("invocation time for {:?} is {}", path, ft);
+    Ok(ft)
 }
 
 #[cfg(unix)]
@@ -209,7 +204,7 @@ pub fn path2bytes(path: &Path) -> CargoResult<&[u8]> {
 pub fn path2bytes(path: &Path) -> CargoResult<&[u8]> {
     match path.as_os_str().to_str() {
         Some(s) => Ok(s.as_bytes()),
-        None => Err(failure::format_err!(
+        None => Err(anyhow::format_err!(
             "invalid non-unicode path: {}",
             path.display()
         )),
@@ -226,7 +221,7 @@ pub fn bytes2path(bytes: &[u8]) -> CargoResult<PathBuf> {
     use std::str;
     match str::from_utf8(bytes) {
         Ok(s) => Ok(PathBuf::from(s)),
-        Err(..) => Err(failure::format_err!("invalid non-unicode path")),
+        Err(..) => Err(anyhow::format_err!("invalid non-unicode path")),
     }
 }
 
@@ -283,7 +278,11 @@ pub fn remove_dir_all<P: AsRef<Path>>(p: P) -> CargoResult<()> {
 }
 
 fn _remove_dir_all(p: &Path) -> CargoResult<()> {
-    if p.symlink_metadata()?.file_type().is_symlink() {
+    if p.symlink_metadata()
+        .chain_err(|| format!("could not get metadata for `{}` to remove", p.display()))?
+        .file_type()
+        .is_symlink()
+    {
         return remove_file(p);
     }
     let entries = p
@@ -383,6 +382,17 @@ fn _link_or_copy(src: &Path, dst: &Path) -> CargoResult<()> {
             src
         };
         symlink(src, dst)
+    } else if env::var_os("__CARGO_COPY_DONT_LINK_DO_NOT_USE_THIS").is_some() {
+        // This is a work-around for a bug in macOS 10.15. When running on
+        // APFS, there seems to be a strange race condition with
+        // Gatekeeper where it will forcefully kill a process launched via
+        // `cargo run` with SIGKILL. Copying seems to avoid the problem.
+        // This shouldn't affect anyone except Cargo's test suite because
+        // it is very rare, and only seems to happen under heavy load and
+        // rapidly creating lots of executables and running them.
+        // See https://github.com/rust-lang/cargo/issues/7821 for the
+        // gory details.
+        fs::copy(src, dst).map(|_| ())
     } else {
         fs::hard_link(src, dst)
     };
@@ -400,3 +410,139 @@ fn _link_or_copy(src: &Path, dst: &Path) -> CargoResult<()> {
         })?;
     Ok(())
 }
+
+/// Copies a file from one location to another.
+pub fn copy<P: AsRef<Path>, Q: AsRef<Path>>(from: P, to: Q) -> CargoResult<u64> {
+    let from = from.as_ref();
+    let to = to.as_ref();
+    fs::copy(from, to)
+        .chain_err(|| format!("failed to copy `{}` to `{}`", from.display(), to.display()))
+}
+
+/// Changes the filesystem mtime (and atime if possible) for the given file.
+///
+/// This intentionally does not return an error, as this is sometimes not
+/// supported on network filesystems. For the current uses in Cargo, this is a
+/// "best effort" approach, and errors shouldn't be propagated.
+pub fn set_file_time_no_err<P: AsRef<Path>>(path: P, time: FileTime) {
+    let path = path.as_ref();
+    match filetime::set_file_times(path, time, time) {
+        Ok(()) => log::debug!("set file mtime {} to {}", path.display(), time),
+        Err(e) => log::warn!(
+            "could not set mtime of {} to {}: {:?}",
+            path.display(),
+            time,
+            e
+        ),
+    }
+}
+
+/// Strips `base` from `path`.
+///
+/// This canonicalizes both paths before stripping. This is useful if the
+/// paths are obtained in different ways, and one or the other may or may not
+/// have been normalized in some way.
+pub fn strip_prefix_canonical<P: AsRef<Path>>(
+    path: P,
+    base: P,
+) -> Result<PathBuf, std::path::StripPrefixError> {
+    // Not all filesystems support canonicalize. Just ignore if it doesn't work.
+    let safe_canonicalize = |path: &Path| match path.canonicalize() {
+        Ok(p) => p,
+        Err(e) => {
+            log::warn!("cannot canonicalize {:?}: {:?}", path, e);
+            path.to_path_buf()
+        }
+    };
+    let canon_path = safe_canonicalize(path.as_ref());
+    let canon_base = safe_canonicalize(base.as_ref());
+    canon_path.strip_prefix(canon_base).map(|p| p.to_path_buf())
+}
+
+/// Creates an excluded from cache directory atomically with its parents as needed.
+///
+/// The atomicity only covers creating the leaf directory and exclusion from cache. Any missing
+/// parent directories will not be created in an atomic manner.
+///
+/// This function is idempotent and in addition to that it won't exclude ``p`` from cache if it
+/// already exists.
+pub fn create_dir_all_excluded_from_backups_atomic(p: impl AsRef<Path>) -> CargoResult<()> {
+    let path = p.as_ref();
+    if path.is_dir() {
+        return Ok(());
+    }
+
+    let parent = path.parent().unwrap();
+    let base = path.file_name().unwrap();
+    create_dir_all(parent)?;
+    // We do this in two steps (first create a temporary directory and exlucde
+    // it from backups, then rename it to the desired name. If we created the
+    // directory directly where it should be and then excluded it from backups
+    // we would risk a situation where cargo is interrupted right after the directory
+    // creation but before the exclusion the the directory would remain non-excluded from
+    // backups because we only perform exclusion right after we created the directory
+    // ourselves.
+    //
+    // We need the tempdir created in parent instead of $TMP, because only then we can be
+    // easily sure that rename() will succeed (the new name needs to be on the same mount
+    // point as the old one).
+    let tempdir = TempFileBuilder::new().prefix(base).tempdir_in(parent)?;
+    exclude_from_backups(&tempdir.path());
+    // Previously std::fs::create_dir_all() (through paths::create_dir_all()) was used
+    // here to create the directory directly and fs::create_dir_all() explicitly treats
+    // the directory being created concurrently by another thread or process as success,
+    // hence the check below to follow the existing behavior. If we get an error at
+    // rename() and suddently the directory (which didn't exist a moment earlier) exists
+    // we can infer from it it's another cargo process doing work.
+    if let Err(e) = fs::rename(tempdir.path(), path) {
+        if !path.exists() {
+            return Err(anyhow::Error::from(e));
+        }
+    }
+    Ok(())
+}
+
+/// Marks the directory as excluded from archives/backups.
+///
+/// This is recommended to prevent derived/temporary files from bloating backups. There are two
+/// mechanisms used to achieve this right now:
+///
+/// * A dedicated resource property excluding from Time Machine backups on macOS
+/// * CACHEDIR.TAG files supported by various tools in a platform-independent way
+fn exclude_from_backups(path: &Path) {
+    exclude_from_time_machine(path);
+    let _ = std::fs::write(
+        path.join("CACHEDIR.TAG"),
+        "Signature: 8a477f597d28d172789f06886806bc55
+# This file is a cache directory tag created by cargo.
+# For information about cache directory tags see https://bford.info/cachedir/",
+    );
+    // Similarly to exclude_from_time_machine() we ignore errors here as it's an optional feature.
+}
+
+#[cfg(not(target_os = "macos"))]
+fn exclude_from_time_machine(_: &Path) {}
+
+#[cfg(target_os = "macos")]
+/// Marks files or directories as excluded from Time Machine on macOS
+fn exclude_from_time_machine(path: &Path) {
+    use core_foundation::base::TCFType;
+    use core_foundation::{number, string, url};
+    use std::ptr;
+
+    // For compatibility with 10.7 a string is used instead of global kCFURLIsExcludedFromBackupKey
+    let is_excluded_key: Result<string::CFString, _> = "NSURLIsExcludedFromBackupKey".parse();
+    let path = url::CFURL::from_path(path, false);
+    if let (Some(path), Ok(is_excluded_key)) = (path, is_excluded_key) {
+        unsafe {
+            url::CFURLSetResourcePropertyForKey(
+                path.as_concrete_TypeRef(),
+                is_excluded_key.as_concrete_TypeRef(),
+                number::kCFBooleanTrue as *const _,
+                ptr::null_mut(),
+            );
+        }
+    }
+    // Errors are ignored, since it's an optional feature and failure
+    // doesn't prevent Cargo from working
+}
