@@ -2,10 +2,11 @@ use crate::util::{process_error, read2, CargoResult, CargoResultExt};
 use anyhow::bail;
 use jobserver::Client;
 use shell_escape::escape;
-use std::collections::HashMap;
+use std::collections::BTreeMap;
 use std::env;
 use std::ffi::{OsStr, OsString};
 use std::fmt;
+use std::iter::once;
 use std::path::Path;
 use std::process::{Command, Output, Stdio};
 
@@ -17,7 +18,7 @@ pub struct ProcessBuilder {
     /// A list of arguments to pass to the program.
     args: Vec<OsString>,
     /// Any environment variables that should be set for the program.
-    env: HashMap<String, Option<OsString>>,
+    env: BTreeMap<String, Option<OsString>>,
     /// The directory to run the program from.
     cwd: Option<OsString>,
     /// The `make` jobserver. See the [jobserver crate][jobserver_docs] for
@@ -128,7 +129,7 @@ impl ProcessBuilder {
 
     /// Gets all environment variables explicitly set or unset for the process (not inherited
     /// vars).
-    pub fn get_envs(&self) -> &HashMap<String, Option<OsString>> {
+    pub fn get_envs(&self) -> &BTreeMap<String, Option<OsString>> {
         &self.env
     }
 
@@ -326,6 +327,37 @@ impl ProcessBuilder {
         }
         command
     }
+
+    /// Wraps an existing command with the provided wrapper, if it is present and valid.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use cargo::util::{ProcessBuilder, process};
+    /// // Running this would execute `rustc`
+    /// let cmd: ProcessBuilder = process("rustc");
+    ///
+    /// // Running this will execute `sccache rustc`
+    /// let cmd = cmd.wrapped(Some("sccache"));
+    /// ```
+    pub fn wrapped(mut self, wrapper: Option<impl AsRef<OsStr>>) -> Self {
+        let wrapper = if let Some(wrapper) = wrapper.as_ref() {
+            wrapper.as_ref()
+        } else {
+            return self;
+        };
+
+        if wrapper.is_empty() {
+            return self;
+        }
+
+        let args = once(self.program).chain(self.args.into_iter()).collect();
+
+        self.program = wrapper.to_os_string();
+        self.args = args;
+
+        self
+    }
 }
 
 /// A helper function to create a `ProcessBuilder`.
@@ -334,7 +366,7 @@ pub fn process<T: AsRef<OsStr>>(cmd: T) -> ProcessBuilder {
         program: cmd.as_ref().to_os_string(),
         args: Vec::new(),
         cwd: None,
-        env: HashMap::new(),
+        env: BTreeMap::new(),
         jobserver: None,
         display_env_vars: false,
     }
@@ -349,13 +381,11 @@ mod imp {
     pub fn exec_replace(process_builder: &ProcessBuilder) -> CargoResult<()> {
         let mut command = process_builder.build_command();
         let error = command.exec();
-        Err(anyhow::Error::from(error)
-            .context(process_error(
-                &format!("could not execute process {}", process_builder),
-                None,
-                None,
-            ))
-            .into())
+        Err(anyhow::Error::from(error).context(process_error(
+            &format!("could not execute process {}", process_builder),
+            None,
+            None,
+        )))
     }
 }
 
