@@ -1,4 +1,4 @@
-use crate::util::{CargoResult, CargoResultExt};
+use crate::util::{paths, CargoResult, CargoResultExt};
 use crypto_hash::{Algorithm, Hasher};
 use std::fs::File;
 use std::io::{self, Read, Write};
@@ -30,7 +30,7 @@ impl Sha256 {
 
     pub fn update_path<P: AsRef<Path>>(&mut self, path: P) -> CargoResult<&mut Sha256> {
         let path = path.as_ref();
-        let file = File::open(path)?;
+        let file = paths::open(path)?;
         self.update_file(&file)
             .chain_err(|| format!("failed to read `{}`", path.display()))?;
         Ok(self)
