@@ -1,6 +1,5 @@
 use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
 use std::fmt;
-use std::fs;
 use std::path::{Path, PathBuf};
 use std::rc::Rc;
 use std::str;
@@ -15,12 +14,15 @@ use serde::{Deserialize, Serialize};
 use url::Url;
 
 use crate::core::dependency::DepKind;
-use crate::core::manifest::{LibKind, ManifestMetadata, TargetSourcePath, Warnings};
-use crate::core::{Dependency, InternedString, Manifest, PackageId, Summary, Target};
-use crate::core::{Edition, EitherManifest, Feature, Features, VirtualManifest};
+use crate::core::manifest::{ManifestMetadata, TargetSourcePath, Warnings};
+use crate::core::profiles::Strip;
+use crate::core::resolver::ResolveBehavior;
+use crate::core::{Dependency, Manifest, PackageId, Summary, Target};
+use crate::core::{Edition, EitherManifest, Feature, Features, VirtualManifest, Workspace};
 use crate::core::{GitReference, PackageIdSpec, SourceId, WorkspaceConfig, WorkspaceRootConfig};
 use crate::sources::{CRATES_IO_INDEX, CRATES_IO_REGISTRY};
 use crate::util::errors::{CargoResult, CargoResultExt, ManifestError};
+use crate::util::interning::InternedString;
 use crate::util::{self, paths, validate_package_name, Config, IntoUrl};
 
 mod targets;
@@ -78,9 +80,9 @@ fn do_read_manifest(
         let (mut manifest, paths) =
             TomlManifest::to_real_manifest(&manifest, source_id, package_root, config)?;
         add_unused(manifest.warnings_mut());
-        if !manifest.targets().iter().any(|t| !t.is_custom_build()) {
+        if manifest.targets().iter().all(|t| t.is_custom_build()) {
             bail!(
-                "no targets specified in the manifest\n  \
+                "no targets specified in the manifest\n\
                  either src/lib.rs, src/main.rs, a [lib] section, or \
                  [[bin]] section must be present"
             )
@@ -162,7 +164,7 @@ and this will become a hard error in the future.",
     }
 
     let first_error = anyhow::Error::from(first_error);
-    Err(first_error.context("could not parse input as TOML").into())
+    Err(first_error.context("could not parse input as TOML"))
 }
 
 type TomlLibTarget = TomlTarget;
@@ -321,7 +323,7 @@ impl<'de> de::Deserialize<'de> for TomlOptLevel {
                 } else {
                     Err(E::custom(format!(
                         "must be an integer, `z`, or `s`, \
-                         but found: {}",
+                         but found the string: \"{}\"",
                         value
                     )))
                 }
@@ -407,6 +409,7 @@ pub struct TomlProfile {
     pub build_override: Option<Box<TomlProfile>>,
     pub dir_name: Option<InternedString>,
     pub inherits: Option<InternedString>,
+    pub strip: Option<Strip>,
 }
 
 #[derive(Clone, Debug, PartialEq, Eq, Ord, PartialOrd, Hash)]
@@ -522,6 +525,10 @@ impl TomlProfile {
                 );
             }
         }
+
+        if self.strip.is_some() {
+            features.require(Feature::strip())?;
+        }
         Ok(())
     }
 
@@ -641,6 +648,10 @@ impl TomlProfile {
         if let Some(v) = &profile.dir_name {
             self.dir_name = Some(*v);
         }
+
+        if let Some(v) = profile.strip {
+            self.strip = Some(v);
+        }
     }
 }
 
@@ -798,7 +809,7 @@ pub struct TomlProject {
     description: Option<String>,
     homepage: Option<String>,
     documentation: Option<String>,
-    readme: Option<String>,
+    readme: Option<StringOrBool>,
     keywords: Option<Vec<String>>,
     categories: Option<Vec<String>>,
     license: Option<String>,
@@ -806,6 +817,7 @@ pub struct TomlProject {
     license_file: Option<String>,
     repository: Option<String>,
     metadata: Option<toml::Value>,
+    resolver: Option<String>,
 }
 
 #[derive(Debug, Deserialize, Serialize)]
@@ -814,6 +826,8 @@ pub struct TomlWorkspace {
     #[serde(rename = "default-members")]
     default_members: Option<Vec<String>>,
     exclude: Option<Vec<String>>,
+    metadata: Option<toml::Value>,
+    resolver: Option<String>,
 }
 
 impl TomlProject {
@@ -835,7 +849,12 @@ struct Context<'a, 'b> {
 }
 
 impl TomlManifest {
-    pub fn prepare_for_publish(&self, config: &Config) -> CargoResult<TomlManifest> {
+    pub fn prepare_for_publish(
+        &self,
+        ws: &Workspace<'_>,
+        package_root: &Path,
+    ) -> CargoResult<TomlManifest> {
+        let config = ws.config();
         let mut package = self
             .package
             .as_ref()
@@ -843,6 +862,35 @@ impl TomlManifest {
             .unwrap()
             .clone();
         package.workspace = None;
+        let mut cargo_features = self.cargo_features.clone();
+        package.resolver = ws.resolve_behavior().to_manifest();
+        if package.resolver.is_some() {
+            // This should be removed when stabilizing.
+            match &mut cargo_features {
+                None => cargo_features = Some(vec!["resolver".to_string()]),
+                Some(feats) => {
+                    if !feats.iter().any(|feat| feat == "resolver") {
+                        feats.push("resolver".to_string());
+                    }
+                }
+            }
+        }
+        if let Some(license_file) = &package.license_file {
+            let license_path = Path::new(&license_file);
+            let abs_license_path = paths::normalize_path(&package_root.join(license_path));
+            if abs_license_path.strip_prefix(package_root).is_err() {
+                // This path points outside of the package root. `cargo package`
+                // will copy it into the root, so adjust the path to this location.
+                package.license_file = Some(
+                    license_path
+                        .file_name()
+                        .unwrap()
+                        .to_str()
+                        .unwrap()
+                        .to_string(),
+                );
+            }
+        }
         let all = |_d: &TomlDependency| true;
         return Ok(TomlManifest {
             package: Some(package),
@@ -908,7 +956,7 @@ impl TomlManifest {
             patch: None,
             workspace: None,
             badges: self.badges.clone(),
-            cargo_features: self.cargo_features.clone(),
+            cargo_features,
         });
 
         fn map_deps(
@@ -996,6 +1044,25 @@ impl TomlManifest {
             features.require(Feature::metabuild())?;
         }
 
+        if project.resolver.is_some()
+            || me
+                .workspace
+                .as_ref()
+                .map_or(false, |ws| ws.resolver.is_some())
+        {
+            features.require(Feature::resolver())?;
+        }
+        let resolve_behavior = match (
+            project.resolver.as_ref(),
+            me.workspace.as_ref().and_then(|ws| ws.resolver.as_ref()),
+        ) {
+            (None, None) => None,
+            (Some(s), None) | (None, Some(s)) => Some(ResolveBehavior::from_manifest(s)?),
+            (Some(_), Some(_)) => {
+                bail!("cannot specify `resolver` field in both `[workspace]` and `[package]`")
+            }
+        };
+
         // If we have no lib at all, use the inferred lib, if available.
         // If we have a lib with a path, we're done.
         // If we have a lib with no path, use the inferred lib or else the package name.
@@ -1062,6 +1129,7 @@ impl TomlManifest {
                 };
                 for (n, v) in dependencies.iter() {
                     let dep = v.to_dependency(n, cx, kind)?;
+                    validate_package_name(dep.name_in_toml().as_str(), "dependency name", "")?;
                     cx.deps.push(dep);
                 }
 
@@ -1126,25 +1194,28 @@ impl TomlManifest {
             features.require(Feature::namespaced_features())?;
         }
 
+        let summary_features = me
+            .features
+            .as_ref()
+            .map(|x| {
+                x.iter()
+                    .map(|(k, v)| (k.as_str(), v.iter().collect()))
+                    .collect()
+            })
+            .unwrap_or_else(BTreeMap::new);
         let summary = Summary::new(
             pkgid,
             deps,
-            &me.features
-                .as_ref()
-                .map(|x| {
-                    x.iter()
-                        .map(|(k, v)| (k.as_str(), v.iter().collect()))
-                        .collect()
-                })
-                .unwrap_or_else(BTreeMap::new),
-            project.links.as_ref().map(|x| x.as_str()),
+            &summary_features,
+            project.links.as_deref(),
             project.namespaced_features.unwrap_or(false),
         )?;
+
         let metadata = ManifestMetadata {
             description: project.description.clone(),
             homepage: project.homepage.clone(),
             documentation: project.documentation.clone(),
-            readme: project.readme.clone(),
+            readme: readme_for_project(package_root, project),
             authors: project.authors.clone().unwrap_or_default(),
             license: project.license.clone(),
             license_file: project.license_file.clone(),
@@ -1161,6 +1232,7 @@ impl TomlManifest {
                 &config.members,
                 &config.default_members,
                 &config.exclude,
+                &config.metadata,
             )),
             (None, root) => WorkspaceConfig::Member {
                 root: root.cloned(),
@@ -1234,6 +1306,7 @@ impl TomlManifest {
             project.default_run.clone(),
             Rc::clone(me),
             project.metabuild.clone().map(|sov| sov.0),
+            resolve_behavior,
         );
         if project.license_file.is_some() && project.license.is_some() {
             manifest.warnings_mut().add_warning(
@@ -1261,43 +1334,43 @@ impl TomlManifest {
         config: &Config,
     ) -> CargoResult<(VirtualManifest, Vec<PathBuf>)> {
         if me.project.is_some() {
-            bail!("virtual manifests do not define [project]");
+            bail!("this virtual manifest specifies a [project] section, which is not allowed");
         }
         if me.package.is_some() {
-            bail!("virtual manifests do not define [package]");
+            bail!("this virtual manifest specifies a [package] section, which is not allowed");
         }
         if me.lib.is_some() {
-            bail!("virtual manifests do not specify [lib]");
+            bail!("this virtual manifest specifies a [lib] section, which is not allowed");
         }
         if me.bin.is_some() {
-            bail!("virtual manifests do not specify [[bin]]");
+            bail!("this virtual manifest specifies a [[bin]] section, which is not allowed");
         }
         if me.example.is_some() {
-            bail!("virtual manifests do not specify [[example]]");
+            bail!("this virtual manifest specifies a [[example]] section, which is not allowed");
         }
         if me.test.is_some() {
-            bail!("virtual manifests do not specify [[test]]");
+            bail!("this virtual manifest specifies a [[test]] section, which is not allowed");
         }
         if me.bench.is_some() {
-            bail!("virtual manifests do not specify [[bench]]");
+            bail!("this virtual manifest specifies a [[bench]] section, which is not allowed");
         }
         if me.dependencies.is_some() {
-            bail!("virtual manifests do not specify [dependencies]");
+            bail!("this virtual manifest specifies a [dependencies] section, which is not allowed");
         }
         if me.dev_dependencies.is_some() || me.dev_dependencies2.is_some() {
-            bail!("virtual manifests do not specify [dev-dependencies]");
+            bail!("this virtual manifest specifies a [dev-dependencies] section, which is not allowed");
         }
         if me.build_dependencies.is_some() || me.build_dependencies2.is_some() {
-            bail!("virtual manifests do not specify [build-dependencies]");
+            bail!("this virtual manifest specifies a [build-dependencies] section, which is not allowed");
         }
         if me.features.is_some() {
-            bail!("virtual manifests do not specify [features]");
+            bail!("this virtual manifest specifies a [features] section, which is not allowed");
         }
         if me.target.is_some() {
-            bail!("virtual manifests do not specify [target]");
+            bail!("this virtual manifest specifies a [target] section, which is not allowed");
         }
         if me.badges.is_some() {
-            bail!("virtual manifests do not specify [badges]");
+            bail!("this virtual manifest specifies a [badges] section, which is not allowed");
         }
 
         let mut nested_paths = Vec::new();
@@ -1325,19 +1398,40 @@ impl TomlManifest {
         if let Some(profiles) = &profiles {
             profiles.validate(&features, &mut warnings)?;
         }
+        if me
+            .workspace
+            .as_ref()
+            .map_or(false, |ws| ws.resolver.is_some())
+        {
+            features.require(Feature::resolver())?;
+        }
+        let resolve_behavior = me
+            .workspace
+            .as_ref()
+            .and_then(|ws| ws.resolver.as_deref())
+            .map(|r| ResolveBehavior::from_manifest(r))
+            .transpose()?;
         let workspace_config = match me.workspace {
             Some(ref config) => WorkspaceConfig::Root(WorkspaceRootConfig::new(
                 root,
                 &config.members,
                 &config.default_members,
                 &config.exclude,
+                &config.metadata,
             )),
             None => {
                 bail!("virtual manifests must be configured with [workspace]");
             }
         };
         Ok((
-            VirtualManifest::new(replace, patch, workspace_config, profiles, features),
+            VirtualManifest::new(
+                replace,
+                patch,
+                workspace_config,
+                profiles,
+                features,
+                resolve_behavior,
+            ),
             nested_paths,
         ))
     }
@@ -1418,11 +1512,12 @@ impl TomlManifest {
             Some(StringOrBool::Bool(true)) => Some(build_rs),
             Some(StringOrBool::String(ref s)) => Some(PathBuf::from(s)),
             None => {
-                match fs::metadata(&build_rs) {
-                    // If there is a `build.rs` file next to the `Cargo.toml`, assume it is
-                    // a build script.
-                    Ok(ref e) if e.is_file() => Some(build_rs),
-                    Ok(_) | Err(_) => None,
+                // If there is a `build.rs` file next to the `Cargo.toml`, assume it is
+                // a build script.
+                if build_rs.is_file() {
+                    Some(build_rs)
+                } else {
+                    None
                 }
             }
         }
@@ -1433,6 +1528,32 @@ impl TomlManifest {
     }
 }
 
+/// Returns the name of the README file for a `TomlProject`.
+fn readme_for_project(package_root: &Path, project: &TomlProject) -> Option<String> {
+    match &project.readme {
+        None => default_readme_from_package_root(package_root),
+        Some(value) => match value {
+            StringOrBool::Bool(false) => None,
+            StringOrBool::Bool(true) => Some("README.md".to_string()),
+            StringOrBool::String(v) => Some(v.clone()),
+        },
+    }
+}
+
+const DEFAULT_README_FILES: [&str; 3] = ["README.md", "README.txt", "README"];
+
+/// Checks if a file with any of the default README file names exists in the package root.
+/// If so, returns a `String` representing that name.
+fn default_readme_from_package_root(package_root: &Path) -> Option<String> {
+    for &readme_filename in DEFAULT_README_FILES.iter() {
+        if package_root.join(readme_filename).is_file() {
+            return Some(readme_filename.to_string());
+        }
+    }
+
+    None
+}
+
 /// Checks a list of build targets, and ensures the target names are unique within a vector.
 /// If not, the name of the offending build target is returned.
 fn unique_build_targets(targets: &[Target], package_root: &Path) -> Result<(), String> {
@@ -1571,6 +1692,17 @@ impl DetailedTomlDependency {
                     .or_else(|| self.rev.clone().map(GitReference::Rev))
                     .unwrap_or_else(|| GitReference::Branch("master".to_string()));
                 let loc = git.into_url()?;
+
+                if let Some(fragment) = loc.fragment() {
+                    let msg = format!(
+                        "URL fragment `#{}` in git URL is ignored for dependency ({}). \
+                        If you were trying to specify a specific git revision, \
+                        use `rev = \"{}\"` in the dependency declaration.",
+                        fragment, name_in_toml, fragment
+                    );
+                    cx.warnings.push(msg)
+                }
+
                 SourceId::for_git(&loc, reference)?
             }
             (None, Some(path), _, _) => {
@@ -1604,7 +1736,7 @@ impl DetailedTomlDependency {
             None => (name_in_toml, None),
         };
 
-        let version = self.version.as_ref().map(|v| &v[..]);
+        let version = self.version.as_deref();
         let mut dep = match cx.pkgid {
             Some(id) => Dependency::parse(pkg_name, version, new_source_id, id, cx.config)?,
             None => Dependency::parse_no_deprecated(pkg_name, version, new_source_id)?,
@@ -1666,9 +1798,9 @@ struct TomlTarget {
     doc: Option<bool>,
     plugin: Option<bool>,
     #[serde(rename = "proc-macro")]
-    proc_macro: Option<bool>,
+    proc_macro_raw: Option<bool>,
     #[serde(rename = "proc_macro")]
-    proc_macro2: Option<bool>,
+    proc_macro_raw2: Option<bool>,
     harness: Option<bool>,
     #[serde(rename = "required-features")]
     required_features: Option<Vec<String>>,
@@ -1723,7 +1855,7 @@ impl TomlTarget {
     }
 
     fn proc_macro(&self) -> Option<bool> {
-        self.proc_macro.or(self.proc_macro2).or_else(|| {
+        self.proc_macro_raw.or(self.proc_macro_raw2).or_else(|| {
             if let Some(types) = self.crate_types() {
                 if types.contains(&"proc-macro".to_string()) {
                     return Some(true);
