@@ -15,11 +15,13 @@ use std::fs::{self, DirEntry};
 use std::path::{Path, PathBuf};
 
 use super::{
-    LibKind, PathValue, StringOrBool, StringOrVec, TomlBenchTarget, TomlBinTarget,
-    TomlExampleTarget, TomlLibTarget, TomlManifest, TomlTarget, TomlTestTarget,
+    PathValue, StringOrBool, StringOrVec, TomlBenchTarget, TomlBinTarget, TomlExampleTarget,
+    TomlLibTarget, TomlManifest, TomlTarget, TomlTestTarget,
 };
-use crate::core::{compiler, Edition, Feature, Features, Target};
+use crate::core::compiler::CrateType;
+use crate::core::{Edition, Feature, Features, Target};
 use crate::util::errors::{CargoResult, CargoResultExt};
+use crate::util::restricted_names;
 
 pub fn targets(
     features: &Features,
@@ -54,7 +56,7 @@ pub fn targets(
         .package
         .as_ref()
         .or_else(|| manifest.project.as_ref())
-        .ok_or_else(|| failure::format_err!("manifest has no `package` (or `project`)"))?;
+        .ok_or_else(|| anyhow::format_err!("manifest has no `package` (or `project`)"))?;
 
     targets.extend(clean_bins(
         features,
@@ -101,7 +103,7 @@ pub fn targets(
     // processing the custom build script
     if let Some(custom_build) = manifest.maybe_custom_build(custom_build, package_root) {
         if metabuild.is_some() {
-            failure::bail!("cannot specify both `metabuild` and `build`");
+            anyhow::bail!("cannot specify both `metabuild` and `build`");
         }
         let name = format!(
             "build-script-{}",
@@ -121,7 +123,7 @@ pub fn targets(
         let bdeps = manifest.build_dependencies.as_ref();
         for name in &metabuild.0 {
             if !bdeps.map_or(false, |bd| bd.contains_key(name)) {
-                failure::bail!(
+                anyhow::bail!(
                     "metabuild package `{}` must be specified in `build-dependencies`",
                     name
                 );
@@ -151,7 +153,7 @@ fn clean_lib(
             if let Some(ref name) = lib.name {
                 // XXX: other code paths dodge this validation
                 if name.contains('-') {
-                    failure::bail!("library target names cannot contain hyphens: {}", name)
+                    anyhow::bail!("library target names cannot contain hyphens: {}", name)
                 }
             }
             Some(TomlTarget {
@@ -171,7 +173,7 @@ fn clean_lib(
         None => return Ok(None),
     };
 
-    validate_has_name(lib, "library", "lib")?;
+    validate_target_name(lib, "library", "lib", warnings)?;
 
     let path = match (lib.path.as_ref(), inferred) {
         (Some(path), _) => package_root.join(&path.0),
@@ -187,7 +189,7 @@ fn clean_lib(
                 ));
                 legacy_path
             } else {
-                failure::bail!(
+                anyhow::bail!(
                     "can't find library `{}`, \
                      rename file to `src/lib.rs` or specify lib.path",
                     lib.name()
@@ -198,7 +200,7 @@ fn clean_lib(
 
     // Per the Macros 1.1 RFC:
     //
-    // > Initially if a crate is compiled with the proc-macro crate type
+    // > Initially if a crate is compiled with the `proc-macro` crate type
     // > (and possibly others) it will forbid exporting any items in the
     // > crate other than those functions tagged #[proc_macro_derive] and
     // > those functions must also be placed at the crate root.
@@ -219,17 +221,17 @@ fn clean_lib(
                 lib.name()
             ));
             if kinds.len() > 1 {
-                failure::bail!("cannot mix `proc-macro` crate type with others");
+                anyhow::bail!("cannot mix `proc-macro` crate type with others");
             }
-            vec![LibKind::ProcMacro]
+            vec![CrateType::ProcMacro]
         }
         (_, Some(true), Some(true)) => {
-            failure::bail!("lib.plugin and lib.proc-macro cannot both be true")
+            anyhow::bail!("`lib.plugin` and `lib.proc-macro` cannot both be `true`")
         }
         (Some(kinds), _, _) => kinds.iter().map(|s| s.into()).collect(),
-        (None, Some(true), _) => vec![LibKind::Dylib],
-        (None, _, Some(true)) => vec![LibKind::ProcMacro],
-        (None, _, _) => vec![LibKind::Lib],
+        (None, Some(true), _) => vec![CrateType::Dylib],
+        (None, _, Some(true)) => vec![CrateType::ProcMacro],
+        (None, _, _) => vec![CrateType::Lib],
     };
 
     let mut target = Target::lib_target(&lib.name(), crate_types, path, edition);
@@ -263,7 +265,7 @@ fn clean_bins(
     );
 
     for bin in &bins {
-        validate_has_name(bin, "binary", "bin")?;
+        validate_target_name(bin, "binary", "bin", warnings)?;
 
         let name = bin.name();
 
@@ -286,8 +288,8 @@ fn clean_bins(
             ));
         }
 
-        if compiler::is_bad_artifact_name(&name) {
-            failure::bail!("the binary target name `{}` is forbidden", name)
+        if restricted_names::is_conflicting_artifact_name(&name) {
+            anyhow::bail!("the binary target name `{}` is forbidden", name)
         }
     }
 
@@ -310,7 +312,7 @@ fn clean_bins(
         });
         let path = match path {
             Ok(path) => path,
-            Err(e) => failure::bail!("{}", e),
+            Err(e) => anyhow::bail!("{}", e),
         };
 
         let mut target =
@@ -528,7 +530,7 @@ fn clean_targets_with_legacy_path(
     );
 
     for target in &toml_targets {
-        validate_has_name(target, target_kind_human, target_kind)?;
+        validate_target_name(target, target_kind_human, target_kind, warnings)?;
     }
 
     validate_unique_names(&toml_targets, target_kind)?;
@@ -556,7 +558,7 @@ fn clean_targets_with_legacy_path(
 
 fn inferred_lib(package_root: &Path) -> Option<PathBuf> {
     let lib = package_root.join("src").join("lib.rs");
-    if fs::metadata(&lib).is_ok() {
+    if lib.exists() {
         Some(lib)
     } else {
         None
@@ -661,7 +663,7 @@ fn toml_targets_and_inferred(
 
             let autodiscover = match autodiscover {
                 Some(autodiscover) => autodiscover,
-                None =>
+                None => {
                     if edition == Edition::Edition2015 {
                         if !rem_targets.is_empty() {
                             let mut rem_targets_str = String::new();
@@ -696,6 +698,7 @@ https://github.com/rust-lang/cargo/issues/5330",
                     } else {
                         true
                     }
+                }
             };
 
             if autodiscover {
@@ -718,18 +721,26 @@ fn inferred_to_toml_targets(inferred: &[(String, PathBuf)]) -> Vec<TomlTarget> {
         .collect()
 }
 
-fn validate_has_name(
+fn validate_target_name(
     target: &TomlTarget,
     target_kind_human: &str,
     target_kind: &str,
+    warnings: &mut Vec<String>,
 ) -> CargoResult<()> {
     match target.name {
         Some(ref name) => {
             if name.trim().is_empty() {
-                failure::bail!("{} target names cannot be empty", target_kind_human)
+                anyhow::bail!("{} target names cannot be empty", target_kind_human)
+            }
+            if cfg!(windows) && restricted_names::is_windows_reserved(name) {
+                warnings.push(format!(
+                    "{} target `{}` is a reserved Windows filename, \
+                        this target will not work on Windows platforms",
+                    target_kind_human, name
+                ));
             }
         }
-        None => failure::bail!(
+        None => anyhow::bail!(
             "{} target {}.name is required",
             target_kind_human,
             target_kind
@@ -744,7 +755,7 @@ fn validate_unique_names(targets: &[TomlTarget], target_kind: &str) -> CargoResu
     let mut seen = HashSet::new();
     for name in targets.iter().map(|e| e.name()) {
         if !seen.insert(name.clone()) {
-            failure::bail!(
+            anyhow::bail!(
                 "found duplicate {target_kind} name {name}, \
                  but all {target_kind} targets must have a unique name",
                 target_kind = target_kind,
@@ -763,6 +774,7 @@ fn configure(features: &Features, toml: &TomlTarget, target: &mut Target) -> Car
         .set_doctest(toml.doctest.unwrap_or_else(|| t2.doctested()))
         .set_benched(toml.bench.unwrap_or_else(|| t2.benched()))
         .set_harness(toml.harness.unwrap_or_else(|| t2.harness()))
+        .set_proc_macro(toml.proc_macro().unwrap_or_else(|| t2.proc_macro()))
         .set_for_host(match (toml.plugin, toml.proc_macro()) {
             (None, None) => t2.for_host(),
             (Some(true), _) | (_, Some(true)) => true,
