@@ -5,7 +5,7 @@ use std::fmt::{self, Write};
 use cargo_test_support::install::exe;
 use cargo_test_support::paths::CargoPathExt;
 use cargo_test_support::registry::Package;
-use cargo_test_support::{basic_manifest, is_nightly, project};
+use cargo_test_support::{basic_manifest, project};
 
 #[cargo_test]
 fn check_success() {
@@ -64,7 +64,7 @@ fn check_fail() {
 
     foo.cargo("check")
         .with_status(101)
-        .with_stderr_contains("[..]this function takes 0 parameters but 1 parameter was supplied")
+        .with_stderr_contains("[..]this function takes 0[..]")
         .run();
 }
 
@@ -683,11 +683,6 @@ fn check_artifacts() {
 
 #[cargo_test]
 fn short_message_format() {
-    if !is_nightly() {
-        // This relies on a bug fix https://github.com/rust-lang/rust/pull/64753.
-        // This check may be removed once 1.40 is stable.
-        return;
-    }
     let foo = project()
         .file("src/lib.rs", "fn foo() { let _x: bool = 'a'; }")
         .build();
@@ -752,6 +747,15 @@ fn does_not_use_empty_rustc_wrapper() {
     p.cargo("check").env("RUSTC_WRAPPER", "").run();
 }
 
+#[cargo_test]
+fn does_not_use_empty_rustc_workspace_wrapper() {
+    let p = project().file("src/lib.rs", "").build();
+    p.cargo("check -Zunstable-options")
+        .masquerade_as_nightly_cargo()
+        .env("RUSTC_WORKSPACE_WRAPPER", "")
+        .run();
+}
+
 #[cargo_test]
 fn error_from_deep_recursion() -> Result<(), fmt::Error> {
     let mut big_macro = String::new();
@@ -766,9 +770,130 @@ fn error_from_deep_recursion() -> Result<(), fmt::Error> {
     p.cargo("check --message-format=json")
         .with_status(101)
         .with_stdout_contains(
-            "[..]\"message\":\"recursion limit reached while expanding the macro `m`\"[..]",
+            "[..]\"message\":\"recursion limit reached while expanding [..]`m[..]`\"[..]",
         )
         .run();
 
     Ok(())
 }
+
+#[cargo_test]
+fn rustc_workspace_wrapper_affects_all_workspace_members() {
+    use cargo_test_support::paths;
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [workspace]
+            members = ["bar", "baz"]
+        "#,
+        )
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("bar/src/lib.rs", "pub fn bar() {}")
+        .file("baz/Cargo.toml", &basic_manifest("baz", "0.1.0"))
+        .file("baz/src/lib.rs", "pub fn baz() {}")
+        .build();
+
+    p.cargo("check -Zunstable-options")
+        .env("RUSTC_WORKSPACE_WRAPPER", paths::echo_wrapper())
+        .masquerade_as_nightly_cargo()
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name bar [..]")
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name baz [..]")
+        .run();
+}
+
+#[cargo_test]
+fn rustc_workspace_wrapper_includes_path_deps() {
+    use cargo_test_support::paths;
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+
+            [workspace]
+            members = ["bar"]
+
+            [dependencies]
+            baz = { path = "baz" }
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("bar/src/lib.rs", "pub fn bar() {}")
+        .file("baz/Cargo.toml", &basic_manifest("baz", "0.1.0"))
+        .file("baz/src/lib.rs", "pub fn baz() {}")
+        .build();
+
+    p.cargo("check --workspace -Zunstable-options")
+        .env("RUSTC_WORKSPACE_WRAPPER", paths::echo_wrapper())
+        .masquerade_as_nightly_cargo()
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name foo [..]")
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name bar [..]")
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name baz [..]")
+        .run();
+}
+
+#[cargo_test]
+fn rustc_workspace_wrapper_respects_primary_units() {
+    use cargo_test_support::paths;
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [workspace]
+            members = ["bar", "baz"]
+        "#,
+        )
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("bar/src/lib.rs", "pub fn bar() {}")
+        .file("baz/Cargo.toml", &basic_manifest("baz", "0.1.0"))
+        .file("baz/src/lib.rs", "pub fn baz() {}")
+        .build();
+
+    p.cargo("check -p bar -Zunstable-options")
+        .env("RUSTC_WORKSPACE_WRAPPER", paths::echo_wrapper())
+        .masquerade_as_nightly_cargo()
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name bar [..]")
+        .with_stdout_does_not_contain("WRAPPER CALLED: rustc --crate-name baz [..]")
+        .run();
+}
+
+#[cargo_test]
+fn rustc_workspace_wrapper_excludes_published_deps() {
+    use cargo_test_support::paths;
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+            authors = []
+
+            [workspace]
+            members = ["bar"]
+
+            [dependencies]
+            baz = "1.0.0"
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("bar/src/lib.rs", "pub fn bar() {}")
+        .build();
+
+    Package::new("baz", "1.0.0").publish();
+
+    p.cargo("check --workspace -v -Zunstable-options")
+        .env("RUSTC_WORKSPACE_WRAPPER", paths::echo_wrapper())
+        .masquerade_as_nightly_cargo()
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name foo [..]")
+        .with_stderr_contains("WRAPPER CALLED: rustc --crate-name bar [..]")
+        .with_stderr_contains("[CHECKING] baz [..]")
+        .with_stdout_does_not_contain("WRAPPER CALLED: rustc --crate-name baz [..]")
+        .run();
+}
