@@ -1,13 +1,11 @@
 //! Tests for the `cargo doc` command.
 
-use std::fs::{self, File};
-use std::io::Read;
-use std::str;
-
 use cargo_test_support::paths::CargoPathExt;
 use cargo_test_support::registry::Package;
 use cargo_test_support::{basic_lib_manifest, basic_manifest, git, project};
 use cargo_test_support::{is_nightly, rustc_host};
+use std::fs;
+use std::str;
 
 #[cargo_test]
 fn simple() {
@@ -385,14 +383,7 @@ fn doc_lib_bin_same_name_documents_lib() {
 ",
         )
         .run();
-    assert!(p.root().join("target/doc").is_dir());
-    let doc_file = p.root().join("target/doc/foo/index.html");
-    assert!(doc_file.is_file());
-    let mut doc_html = String::new();
-    File::open(&doc_file)
-        .unwrap()
-        .read_to_string(&mut doc_html)
-        .unwrap();
+    let doc_html = p.read_file("target/doc/foo/index.html");
     assert!(doc_html.contains("Library"));
     assert!(!doc_html.contains("Binary"));
 }
@@ -427,14 +418,7 @@ fn doc_lib_bin_same_name_documents_lib_when_requested() {
 ",
         )
         .run();
-    assert!(p.root().join("target/doc").is_dir());
-    let doc_file = p.root().join("target/doc/foo/index.html");
-    assert!(doc_file.is_file());
-    let mut doc_html = String::new();
-    File::open(&doc_file)
-        .unwrap()
-        .read_to_string(&mut doc_html)
-        .unwrap();
+    let doc_html = p.read_file("target/doc/foo/index.html");
     assert!(doc_html.contains("Library"));
     assert!(!doc_html.contains("Binary"));
 }
@@ -470,14 +454,7 @@ fn doc_lib_bin_same_name_documents_named_bin_when_requested() {
 ",
         )
         .run();
-    assert!(p.root().join("target/doc").is_dir());
-    let doc_file = p.root().join("target/doc/foo/index.html");
-    assert!(doc_file.is_file());
-    let mut doc_html = String::new();
-    File::open(&doc_file)
-        .unwrap()
-        .read_to_string(&mut doc_html)
-        .unwrap();
+    let doc_html = p.read_file("target/doc/foo/index.html");
     assert!(!doc_html.contains("Library"));
     assert!(doc_html.contains("Binary"));
 }
@@ -513,14 +490,7 @@ fn doc_lib_bin_same_name_documents_bins_when_requested() {
 ",
         )
         .run();
-    assert!(p.root().join("target/doc").is_dir());
-    let doc_file = p.root().join("target/doc/foo/index.html");
-    assert!(doc_file.is_file());
-    let mut doc_html = String::new();
-    File::open(&doc_file)
-        .unwrap()
-        .read_to_string(&mut doc_html)
-        .unwrap();
+    let doc_html = p.read_file("target/doc/foo/index.html");
     assert!(!doc_html.contains("Library"));
     assert!(doc_html.contains("Binary"));
 }
@@ -1513,7 +1483,7 @@ fn bin_private_items_deps() {
 
 #[cargo_test]
 fn crate_versions() {
-    // Testing unstable flag
+    // Testing flag that will reach stable on 1.44
     if !is_nightly() {
         return;
     }
@@ -1534,12 +1504,50 @@ fn crate_versions() {
         .masquerade_as_nightly_cargo()
         .run();
 
-    let doc_file = p.root().join("target/doc/foo/index.html");
-    let mut doc_html = String::new();
-    File::open(&doc_file)
-        .unwrap()
-        .read_to_string(&mut doc_html)
-        .unwrap();
+    let output_path = p.root().join("target/doc/foo/index.html");
+    let output_documentation = fs::read_to_string(&output_path).unwrap();
 
-    assert!(doc_html.contains("Version 1.2.4"));
+    assert!(output_documentation.contains("Version 1.2.4"));
+}
+
+#[cargo_test]
+fn crate_versions_flag_is_overridden() {
+    // Testing flag that will reach stable on 1.44
+    if !is_nightly() {
+        return;
+    }
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "1.2.4"
+            authors = []
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    let output_documentation = || {
+        let output_path = p.root().join("target/doc/foo/index.html");
+        fs::read_to_string(&output_path).unwrap()
+    };
+    let asserts = |html: String| {
+        assert!(!html.contains("1.2.4"));
+        assert!(html.contains("Version 2.0.3"));
+    };
+
+    p.cargo("-Z crate-versions doc")
+        .masquerade_as_nightly_cargo()
+        .env("RUSTDOCFLAGS", "--crate-version 2.0.3")
+        .run();
+    asserts(output_documentation());
+
+    p.build_dir().rm_rf();
+
+    p.cargo("-Z crate-versions rustdoc -- --crate-version 2.0.3")
+        .masquerade_as_nightly_cargo()
+        .run();
+    asserts(output_documentation());
 }
