@@ -1,11 +1,8 @@
 //! Tests for `[features]` table.
 
-use std::fs::File;
-use std::io::prelude::*;
-
 use cargo_test_support::paths::CargoPathExt;
-use cargo_test_support::registry::Package;
-use cargo_test_support::{basic_manifest, project, t};
+use cargo_test_support::registry::{Dependency, Package};
+use cargo_test_support::{basic_manifest, project};
 
 #[cargo_test]
 fn invalid1() {
@@ -101,7 +98,7 @@ fn invalid3() {
 
 Caused by:
   Feature `bar` depends on `baz` which is not an optional dependency.
-Consider adding `optional = true` to the dependency
+  Consider adding `optional = true` to the dependency
 ",
         )
         .run();
@@ -920,9 +917,7 @@ fn everything_in_the_lockfile() {
         .build();
 
     p.cargo("fetch").run();
-    let loc = p.root().join("Cargo.lock");
-    let mut lockfile = String::new();
-    t!(t!(File::open(&loc)).read_to_string(&mut lockfile));
+    let lockfile = p.read_lockfile();
     assert!(
         lockfile.contains(r#"name = "d1""#),
         "d1 not found\n{}",
@@ -1358,80 +1353,6 @@ fn many_cli_features_comma_and_space_delimited() {
         .run();
 }
 
-#[cargo_test]
-fn combining_features_and_package() {
-    Package::new("dep", "1.0.0").publish();
-
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-
-            [workspace]
-            members = ["bar"]
-
-            [dependencies]
-            dep = "1"
-        "#,
-        )
-        .file("src/lib.rs", "")
-        .file(
-            "bar/Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.0.1"
-            authors = []
-            [features]
-            main = []
-        "#,
-        )
-        .file(
-            "bar/src/main.rs",
-            r#"
-            #[cfg(feature = "main")]
-            fn main() {}
-        "#,
-        )
-        .build();
-
-    p.cargo("build -Z package-features --workspace --features main")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for more than one package")
-        .run();
-
-    p.cargo("build -Z package-features --package dep --features main")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for packages outside of workspace")
-        .run();
-    p.cargo("build -Z package-features --package dep --all-features")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for packages outside of workspace")
-        .run();
-    p.cargo("build -Z package-features --package dep --no-default-features")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for packages outside of workspace")
-        .run();
-
-    p.cargo("build -Z package-features --workspace --all-features")
-        .masquerade_as_nightly_cargo()
-        .run();
-    p.cargo("run -Z package-features --package bar --features main")
-        .masquerade_as_nightly_cargo()
-        .run();
-    p.cargo("build -Z package-features --package dep")
-        .masquerade_as_nightly_cargo()
-        .run();
-}
-
 #[cargo_test]
 fn namespaced_invalid_feature() {
     let p = project()
@@ -1535,7 +1456,7 @@ fn namespaced_non_optional_dependency() {
 
 Caused by:
   Feature `bar` includes `crate:baz` which is not an optional dependency.
-Consider adding `optional = true` to the dependency
+  Consider adding `optional = true` to the dependency
 ",
         )
         .run();
@@ -1598,7 +1519,7 @@ fn namespaced_shadowed_dep() {
 
 Caused by:
   Feature `baz` includes the optional dependency of the same name, but this is left implicit in the features included by this feature.
-Consider adding `crate:baz` to this feature's requirements.
+  Consider adding `crate:baz` to this feature's requirements.
 ",
         )
         .run();
@@ -1634,8 +1555,8 @@ fn namespaced_shadowed_non_optional() {
 
 Caused by:
   Feature `baz` includes the dependency of the same name, but this is left implicit in the features included by this feature.
-Additionally, the dependency must be marked as optional to be included in the feature definition.
-Consider adding `crate:baz` to this feature's requirements and marking the dependency as `optional = true`
+  Additionally, the dependency must be marked as optional to be included in the feature definition.
+  Consider adding `crate:baz` to this feature's requirements and marking the dependency as `optional = true`
 ",
         )
         .run();
@@ -1671,7 +1592,7 @@ fn namespaced_implicit_non_optional() {
 
 Caused by:
   Feature `bar` includes `baz` which is not defined as a feature.
-A non-optional dependency of the same name is defined; consider adding `optional = true` to its definition
+  A non-optional dependency of the same name is defined; consider adding `optional = true` to its definition
 ",
         ).run(
     );
@@ -2192,3 +2113,80 @@ fn slash_optional_enables() {
 
     p.cargo("check --features dep/feat").run();
 }
+
+#[cargo_test]
+fn registry_summary_order_doesnt_matter() {
+    // Checks for an issue where the resolver depended on the order of entries
+    // in the registry summary. If there was a non-optional dev-dependency
+    // that appeared before an optional normal dependency, then the resolver
+    // would not activate the optional dependency with a pkg/featname feature
+    // syntax.
+    Package::new("dep", "0.1.0")
+        .feature("feat1", &[])
+        .file(
+            "src/lib.rs",
+            r#"
+                #[cfg(feature="feat1")]
+                pub fn work() {
+                    println!("it works");
+                }
+            "#,
+        )
+        .publish();
+    Package::new("bar", "0.1.0")
+        .feature("bar_feat", &["dep/feat1"])
+        .add_dep(Dependency::new("dep", "0.1.0").dev())
+        .add_dep(Dependency::new("dep", "0.1.0").optional(true))
+        .file(
+            "src/lib.rs",
+            r#"
+                // This will fail to compile without `dep` optional dep activated.
+                extern crate dep;
+
+                pub fn doit() {
+                    dep::work();
+                }
+            "#,
+        )
+        .publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                edition = "2018"
+
+                [dependencies]
+                bar = { version="0.1", features = ["bar_feat"] }
+            "#,
+        )
+        .file(
+            "src/main.rs",
+            r#"
+                fn main() {
+                    bar::doit();
+                }
+            "#,
+        )
+        .build();
+
+    p.cargo("run")
+        .with_stderr(
+            "\
+[UPDATING] [..]
+[DOWNLOADING] crates ...
+[DOWNLOADED] [..]
+[DOWNLOADED] [..]
+[COMPILING] dep v0.1.0
+[COMPILING] bar v0.1.0
+[COMPILING] foo v0.1.0 [..]
+[FINISHED] [..]
+[RUNNING] `target/debug/foo[EXE]`
+",
+        )
+        .with_stdout("it works")
+        .run();
+}
