@@ -1,8 +1,7 @@
 //! Tests for git support.
 
-use git2;
 use std::env;
-use std::fs::{self, File};
+use std::fs;
 use std::io::prelude::*;
 use std::net::{TcpListener, TcpStream};
 use std::path::Path;
@@ -324,6 +323,16 @@ fn cargo_compile_with_malformed_nested_paths() {
             "#,
             )
             .file("vendor/dep2/Cargo.toml", "!INVALID!")
+            .file(
+                "vendor/dep3/Cargo.toml",
+                r#"
+                [project]
+                name = "dep3"
+                version = "0.5.0"
+                [dependencies]
+                subdep1 = { path = "../require-extra-build-step" }"#,
+            )
+            .file("vendor/dep3/src/lib.rs", "")
     });
 
     let p = project()
@@ -492,10 +501,7 @@ fn two_revs_same_deps() {
     let rev1 = repo.revparse_single("HEAD").unwrap().id();
 
     // Commit the changes and make sure we trigger a recompile
-    File::create(&bar.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 2 }"#)
-        .unwrap();
+    bar.change_file("src/lib.rs", "pub fn bar() -> i32 { 2 }");
     git::add(&repo);
     let rev2 = git::commit(&repo);
 
@@ -614,10 +620,7 @@ fn recompilation() {
     p.cargo("build").with_stdout("").run();
 
     // Modify a file manually, shouldn't trigger a recompile
-    File::create(&git_project.root().join("src/bar.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() { println!("hello!"); }"#)
-        .unwrap();
+    git_project.change_file("src/bar.rs", r#"pub fn bar() { println!("hello!"); }"#);
 
     p.cargo("build").with_stdout("").run();
 
@@ -755,10 +758,7 @@ fn update_with_shared_deps() {
         .run();
 
     // Modify a file manually, and commit it
-    File::create(&git_project.root().join("src/bar.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() { println!("hello!"); }"#)
-        .unwrap();
+    git_project.change_file("src/bar.rs", r#"pub fn bar() { println!("hello!"); }"#);
     let repo = git2::Repository::open(&git_project.root()).unwrap();
     let old_head = repo.head().unwrap().target().unwrap();
     git::add(&repo);
@@ -776,11 +776,13 @@ fn update_with_shared_deps() {
         .with_status(101)
         .with_stderr(
             "\
-[UPDATING] git repository [..]
 [ERROR] Unable to update [..]
 
 Caused by:
-  revspec '0.1.2' not found; [..]
+  precise value for git is not a git revision: 0.1.2
+
+Caused by:
+  unable to parse OID - contains invalid characters; class=Invalid (3)
 ",
         )
         .run();
@@ -869,6 +871,7 @@ fn dep_with_submodule() {
         .with_stderr(
             "\
 [UPDATING] git repository [..]
+[UPDATING] git submodule `file://[..]/dep2`
 [COMPILING] dep1 [..]
 [COMPILING] foo [..]
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]\n",
@@ -932,7 +935,11 @@ fn dep_with_bad_submodule() {
     let expected = format!(
         "\
 [UPDATING] git repository [..]
-[ERROR] failed to load source for a dependency on `dep1`
+[UPDATING] git submodule `file://[..]/dep2`
+[ERROR] failed to get `dep1` as a dependency of package `foo v0.5.0 [..]`
+
+Caused by:
+  failed to load source for dependency `dep1`
 
 Caused by:
   Unable to update {}
@@ -1012,10 +1019,7 @@ fn two_deps_only_update_one() {
         )
         .run();
 
-    File::create(&git1.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn foo() {}"#)
-        .unwrap();
+    git1.change_file("src/lib.rs", "pub fn foo() {}");
     let repo = git2::Repository::open(&git1.root()).unwrap();
     git::add(&repo);
     let oid = git::commit(&repo);
@@ -1072,10 +1076,7 @@ fn stale_cached_version() {
 
     // Update the repo, and simulate someone else updating the lock file and then
     // us pulling it down.
-    File::create(&bar.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 1 + 0 }"#)
-        .unwrap();
+    bar.change_file("src/lib.rs", "pub fn bar() -> i32 { 1 + 0 }");
     let repo = git2::Repository::open(&bar.root()).unwrap();
     git::add(&repo);
     git::commit(&repo);
@@ -1084,29 +1085,26 @@ fn stale_cached_version() {
 
     let rev = repo.revparse_single("HEAD").unwrap().id();
 
-    File::create(&foo.root().join("Cargo.lock"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-        [[package]]
-        name = "foo"
-        version = "0.0.0"
-        dependencies = [
-         'bar 0.0.0 (git+{url}#{hash})'
-        ]
+    foo.change_file(
+        "Cargo.lock",
+        &format!(
+            r#"
+                [[package]]
+                name = "foo"
+                version = "0.0.0"
+                dependencies = [
+                 'bar 0.0.0 (git+{url}#{hash})'
+                ]
 
-        [[package]]
-        name = "bar"
-        version = "0.0.0"
-        source = 'git+{url}#{hash}'
-    "#,
-                url = bar.url(),
-                hash = rev
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+                [[package]]
+                name = "bar"
+                version = "0.0.0"
+                source = 'git+{url}#{hash}'
+            "#,
+            url = bar.url(),
+            hash = rev
+        ),
+    );
 
     // Now build!
     foo.cargo("build")
@@ -1170,6 +1168,7 @@ fn dep_with_changed_submodule() {
     p.cargo("run")
         .with_stderr(
             "[UPDATING] git repository `[..]`\n\
+             [UPDATING] git submodule `file://[..]/dep2`\n\
              [COMPILING] dep1 v0.5.0 ([..])\n\
              [COMPILING] foo v0.5.0 ([..])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) in \
@@ -1179,16 +1178,13 @@ fn dep_with_changed_submodule() {
         .with_stdout("project2\n")
         .run();
 
-    File::create(&git_project.root().join(".gitmodules"))
-        .unwrap()
-        .write_all(
-            format!(
-                "[submodule \"src\"]\n\tpath = src\n\turl={}",
-                git_project3.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+    git_project.change_file(
+        ".gitmodules",
+        &format!(
+            "[submodule \"src\"]\n\tpath = src\n\turl={}",
+            git_project3.url()
+        ),
+    );
 
     // Sync the submodule and reset it to the new remote.
     sub.sync().unwrap();
@@ -1217,6 +1213,7 @@ fn dep_with_changed_submodule() {
         .with_stderr("")
         .with_stderr(&format!(
             "[UPDATING] git repository `{}`\n\
+             [UPDATING] git submodule `file://[..]/dep3`\n\
              [UPDATING] dep1 v0.5.0 ([..]) -> #[..]\n\
              ",
             git_project.url()
@@ -1345,7 +1342,7 @@ fn git_build_cmd_freshness() {
 
     // Modify an ignored file and make sure we don't rebuild
     println!("second pass");
-    File::create(&foo.root().join("src/bar.rs")).unwrap();
+    foo.change_file("src/bar.rs", "");
     foo.cargo("build").with_stdout("").run();
 }
 
@@ -1443,10 +1440,7 @@ fn git_repo_changing_no_rebuild() {
         .run();
 
     // Make a commit to lock p2 to a different rev
-    File::create(&bar.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 2 }"#)
-        .unwrap();
+    bar.change_file("src/lib.rs", "pub fn bar() -> i32 { 2 }");
     let repo = git2::Repository::open(&bar.root()).unwrap();
     git::add(&repo);
     git::commit(&repo);
@@ -1550,10 +1544,7 @@ fn git_dep_build_cmd() {
     p.process(&p.bin("foo")).with_stdout("0\n").run();
 
     // Touching bar.rs.in should cause the `build` command to run again.
-    fs::File::create(&p.root().join("bar/src/bar.rs.in"))
-        .unwrap()
-        .write_all(b"pub fn gimme() -> i32 { 1 }")
-        .unwrap();
+    p.change_file("bar/src/bar.rs.in", "pub fn gimme() -> i32 { 1 }");
 
     p.cargo("build").run();
 
@@ -1820,23 +1811,20 @@ fn switch_deps_does_not_update_transitive() {
 
     // Update the dependency to point to the second repository, but this
     // shouldn't update the transitive dependency which is the same.
-    File::create(&p.root().join("Cargo.toml"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-            [project]
-            name = "foo"
-            version = "0.5.0"
-            authors = []
-            [dependencies.dep]
-            git = '{}'
-    "#,
-                dep2.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+    p.change_file(
+        "Cargo.toml",
+        &format!(
+            r#"
+                [project]
+                name = "foo"
+                version = "0.5.0"
+                authors = []
+                [dependencies.dep]
+                git = '{}'
+            "#,
+            dep2.url()
+        ),
+    );
 
     p.cargo("build")
         .with_stderr(&format!(
@@ -1896,19 +1884,12 @@ fn update_one_source_updates_all_packages_in_that_git_source() {
     let rev1 = repo.revparse_single("HEAD").unwrap().id();
 
     // Just be sure to change a file
-    File::create(&dep.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 2 }"#)
-        .unwrap();
+    dep.change_file("src/lib.rs", "pub fn bar() -> i32 { 2 }");
     git::add(&repo);
     git::commit(&repo);
 
     p.cargo("update -p dep").run();
-    let mut lockfile = String::new();
-    File::open(&p.root().join("Cargo.lock"))
-        .unwrap()
-        .read_to_string(&mut lockfile)
-        .unwrap();
+    let lockfile = p.read_lockfile();
     assert!(
         !lockfile.contains(&rev1.to_string()),
         "{} in {}",
@@ -1972,23 +1953,20 @@ fn switch_sources() {
         )
         .run();
 
-    File::create(&p.root().join("b/Cargo.toml"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-        [project]
-        name = "b"
-        version = "0.5.0"
-        authors = []
-        [dependencies.a]
-        git = '{}'
-    "#,
-                a2.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+    p.change_file(
+        "b/Cargo.toml",
+        &format!(
+            r#"
+                [project]
+                name = "b"
+                version = "0.5.0"
+                authors = []
+                [dependencies.a]
+                git = '{}'
+            "#,
+            a2.url()
+        ),
+    );
 
     p.cargo("build")
         .with_stderr(
@@ -2205,24 +2183,21 @@ fn add_a_git_dep() {
 
     p.cargo("build").run();
 
-    File::create(p.root().join("a/Cargo.toml"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-        [package]
-        name = "a"
-        version = "0.0.1"
-        authors = []
+    p.change_file(
+        "a/Cargo.toml",
+        &format!(
+            r#"
+                [package]
+                name = "a"
+                version = "0.0.1"
+                authors = []
 
-        [dependencies]
-        git = {{ git = '{}' }}
-    "#,
-                git.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+                [dependencies]
+                git = {{ git = '{}' }}
+            "#,
+            git.url()
+        ),
+    );
 
     p.cargo("build").run();
 }
@@ -2314,6 +2289,7 @@ fn include_overrides_gitignore() {
         .with_stdout(
             "\
 Cargo.toml
+Cargo.toml.orig
 ignored.txt
 src/lib.rs
 ",
@@ -2382,20 +2358,24 @@ fn invalid_git_dependency_manifest() {
         .cargo("build")
         .with_status(101)
         .with_stderr(&format!(
-            "[UPDATING] git repository `{}`\n\
-             error: failed to load source for a dependency on `dep1`\n\
-             \n\
-             Caused by:\n  \
-             Unable to update {}\n\
-             \n\
-             Caused by:\n  \
-             failed to parse manifest at `[..]`\n\
-             \n\
-             Caused by:\n  \
-             could not parse input as TOML\n\
-             \n\
-             Caused by:\n  \
-             duplicate key: `categories` for key `project` at line 10 column 17",
+            "\
+[UPDATING] git repository `{}`
+[ERROR] failed to get `dep1` as a dependency of package `foo v0.5.0 ([..])`
+
+Caused by:
+  failed to load source for dependency `dep1`
+
+Caused by:
+  Unable to update {}
+
+Caused by:
+  failed to parse manifest at `[..]`
+
+Caused by:
+  could not parse input as TOML
+
+Caused by:
+  duplicate key: `categories` for key `project` at line 10 column 17",
             path2url(&git_root),
             path2url(&git_root),
         ))
@@ -2563,25 +2543,23 @@ fn templatedir_doesnt_cause_problems() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    File::create(paths::home().join(".gitconfig"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
+    fs::write(
+        paths::home().join(".gitconfig"),
+        format!(
+            r#"
                 [init]
                 templatedir = {}
             "#,
-                git_project2
-                    .url()
-                    .to_file_path()
-                    .unwrap()
-                    .to_str()
-                    .unwrap()
-                    .replace("\\", "/")
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+            git_project2
+                .url()
+                .to_file_path()
+                .unwrap()
+                .to_str()
+                .unwrap()
+                .replace("\\", "/")
+        ),
+    )
+    .unwrap();
 
     p.cargo("build").run();
 }
