@@ -3,7 +3,7 @@
 #![allow(clippy::blacklisted_name)]
 #![allow(clippy::explicit_iter_loop)]
 #![allow(clippy::redundant_closure)]
-#![allow(clippy::block_in_if_condition_stmt)] // clippy doesn't agree with rustfmt 😂
+#![allow(clippy::blocks_in_if_conditions)] // clippy doesn't agree with rustfmt 😂
 #![allow(clippy::inefficient_to_string)] // this causes suggestions that result in `(*s).to_string()`
 #![warn(clippy::needless_borrow)]
 #![warn(clippy::redundant_clone)]
@@ -11,6 +11,7 @@
 #[macro_use]
 extern crate cargo_test_macro;
 
+mod advanced_env;
 mod alt_registry;
 mod bad_config;
 mod bad_manifest_path;
@@ -23,13 +24,15 @@ mod cache_messages;
 mod cargo_alias_config;
 mod cargo_command;
 mod cargo_features;
+mod cargo_targets;
 mod cfg;
 mod check;
 mod clean;
-mod clippy;
 mod collisions;
 mod concurrent;
 mod config;
+mod config_cli;
+mod config_include;
 mod corrupt_git;
 mod cross_compile;
 mod cross_publish;
@@ -39,7 +42,9 @@ mod dep_info;
 mod directory;
 mod doc;
 mod edition;
+mod error;
 mod features;
+mod features2;
 mod fetch;
 mod fix;
 mod freshness;
@@ -53,21 +58,27 @@ mod install_upgrade;
 mod jobserver;
 mod list_targets;
 mod local_registry;
+mod locate_project;
 mod lockfile_compat;
 mod login;
+mod lto;
 mod member_errors;
 mod message_format;
 mod metabuild;
 mod metadata;
 mod minimal_versions;
+mod multitarget;
 mod net_config;
 mod new;
 mod offline;
 mod out_dir;
+mod owner;
 mod package;
+mod package_features;
 mod patch;
 mod path;
 mod paths;
+mod pkgid;
 mod plugins;
 mod proc_macro;
 mod profile_config;
@@ -87,6 +98,7 @@ mod run;
 mod rustc;
 mod rustc_info_cache;
 mod rustdoc;
+mod rustdoc_extern_html;
 mod rustdocflags;
 mod rustflags;
 mod search;
@@ -95,12 +107,16 @@ mod standard_lib;
 mod test;
 mod timings;
 mod tool_paths;
+mod tree;
+mod tree_graph_features;
+mod unit_graph;
 mod update;
 mod vendor;
 mod verify_project;
 mod version;
 mod warn_on_failure;
 mod workspaces;
+mod yank;
 
 #[cargo_test]
 fn aaa_trigger_cross_compile_disabled_check() {
