@@ -32,7 +32,7 @@ error: failed to parse manifest at `[..]`
 Caused by:
   feature `metabuild` is required
 
-consider adding `cargo-features = [\"metabuild\"]` to the manifest
+  consider adding `cargo-features = [\"metabuild\"]` to the manifest
 ",
         )
         .run();
@@ -461,7 +461,7 @@ fn metabuild_build_plan() {
             "package_version": "0.5.0",
             "target_kind": ["lib"],
             "compile_mode": "build",
-            "kind": "Host",
+            "kind": null,
             "deps": [],
             "outputs": [
                 "[..]/target/debug/deps/libmb-[..].rlib",
@@ -478,7 +478,7 @@ fn metabuild_build_plan() {
             "package_version": "0.0.1",
             "target_kind": ["lib"],
             "compile_mode": "build",
-            "kind": "Host",
+            "kind": null,
             "deps": [],
             "outputs": [
                 "[..]/target/debug/deps/libmb_other-[..].rlib",
@@ -495,9 +495,9 @@ fn metabuild_build_plan() {
             "package_version": "0.0.1",
             "target_kind": ["custom-build"],
             "compile_mode": "build",
-            "kind": "Host",
+            "kind": null,
             "deps": [0, 1],
-            "outputs": ["[..]/target/debug/build/foo-[..]/metabuild_foo-[..][EXE]"],
+            "outputs": "{...}",
             "links": "{...}",
             "program": "rustc",
             "args": "{...}",
@@ -509,7 +509,7 @@ fn metabuild_build_plan() {
             "package_version": "0.0.1",
             "target_kind": ["custom-build"],
             "compile_mode": "run-custom-build",
-            "kind": "Host",
+            "kind": null,
             "deps": [2],
             "outputs": [],
             "links": {},
@@ -523,7 +523,7 @@ fn metabuild_build_plan() {
             "package_version": "0.0.1",
             "target_kind": ["lib"],
             "compile_mode": "build",
-            "kind": "Host",
+            "kind": null,
             "deps": [3],
             "outputs": [
                 "[..]/foo/target/debug/deps/libfoo-[..].rlib",
@@ -686,9 +686,7 @@ fn metabuild_json_artifact() {
 {
   "executable": null,
   "features": [],
-  "filenames": [
-    "[..]/foo/target/debug/build/foo-[..]/metabuild-foo[EXE]"
-  ],
+  "filenames": "{...}",
   "fresh": false,
   "package_id": "foo [..]",
   "profile": "{...}",
