@@ -1,11 +1,9 @@
 //! Tests for the `cargo new` command.
 
-use std::env;
-use std::fs::{self, File};
-use std::io::prelude::*;
-
 use cargo_test_support::paths;
 use cargo_test_support::{cargo_process, git_process};
+use std::env;
+use std::fs::{self, File};
 
 fn create_empty_gitconfig() {
     // This helps on Windows where libgit2 is very aggressive in attempting to
@@ -27,11 +25,7 @@ fn simple_lib() {
     assert!(!paths::root().join("foo/.gitignore").is_file());
 
     let lib = paths::root().join("foo/src/lib.rs");
-    let mut contents = String::new();
-    File::open(&lib)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&lib).unwrap();
     assert_eq!(
         contents,
         r#"#[cfg(test)]
@@ -86,11 +80,7 @@ fn simple_git() {
     assert!(paths::root().join("foo/.gitignore").is_file());
 
     let fp = paths::root().join("foo/.gitignore");
-    let mut contents = String::new();
-    File::open(&fp)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&fp).unwrap();
     assert_eq!(contents, "/target\nCargo.lock\n",);
 
     cargo_process("build").cwd(&paths::root().join("foo")).run();
@@ -126,11 +116,7 @@ fn existing() {
 fn invalid_characters() {
     cargo_process("new foo.rs")
         .with_status(101)
-        .with_stderr(
-            "\
-[ERROR] Invalid character `.` in crate name: `foo.rs`
-use --name to override crate name",
-        )
+        .with_stderr("[ERROR] invalid character `.` in crate name: `foo.rs`, [..]")
         .run();
 }
 
@@ -138,10 +124,7 @@ use --name to override crate name",
 fn reserved_name() {
     cargo_process("new test")
         .with_status(101)
-        .with_stderr(
-            "[ERROR] The name `test` cannot be used as a crate name\n\
-             use --name to override crate name",
-        )
+        .with_stderr("[ERROR] the name `test` cannot be used as a crate name, it conflicts [..]")
         .run();
 }
 
@@ -150,8 +133,18 @@ fn reserved_binary_name() {
     cargo_process("new --bin incremental")
         .with_status(101)
         .with_stderr(
-            "[ERROR] The name `incremental` cannot be used as a crate name\n\
-             use --name to override crate name",
+            "[ERROR] the name `incremental` cannot be used as a crate name, it conflicts [..]",
+        )
+        .run();
+
+    cargo_process("new --lib incremental")
+        .env("USER", "foo")
+        .with_stderr(
+            "\
+[WARNING] the name `incremental` will not support binary executables with that name, \
+it conflicts with cargo's build directory names
+[CREATED] library `incremental` package
+",
         )
         .run();
 }
@@ -160,9 +153,20 @@ fn reserved_binary_name() {
 fn keyword_name() {
     cargo_process("new pub")
         .with_status(101)
+        .with_stderr("[ERROR] the name `pub` cannot be used as a crate name, it is a Rust keyword")
+        .run();
+}
+
+#[cargo_test]
+fn std_name() {
+    cargo_process("new core")
+        .env("USER", "foo")
         .with_stderr(
-            "[ERROR] The name `pub` cannot be used as a crate name\n\
-             use --name to override crate name",
+            "\
+[WARNING] the name `core` is part of Rust's standard library
+It is recommended to use a different name to avoid problems.
+[CREATED] binary (application) `core` package
+",
         )
         .run();
 }
@@ -173,11 +177,7 @@ fn finds_author_user() {
     cargo_process("new foo").env("USER", "foo").run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["foo"]"#));
 }
 
@@ -187,11 +187,7 @@ fn finds_author_user_escaped() {
     cargo_process("new foo").env("USER", "foo \"bar\"").run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["foo \"bar\""]"#));
 }
 
@@ -204,11 +200,20 @@ fn finds_author_username() {
         .run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
+    assert!(contents.contains(r#"authors = ["foo"]"#));
+}
+
+#[cargo_test]
+fn finds_author_name() {
+    create_empty_gitconfig();
+    cargo_process("new foo")
+        .env_remove("USERNAME")
+        .env("NAME", "foo")
+        .run();
+
+    let toml = paths::root().join("foo/Cargo.toml");
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["foo"]"#));
 }
 
@@ -222,11 +227,7 @@ fn finds_author_priority() {
         .run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["bar <baz>"]"#));
 }
 
@@ -239,11 +240,7 @@ fn finds_author_email() {
         .run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["bar <baz>"]"#));
 }
 
@@ -256,11 +253,7 @@ fn finds_author_git() {
     cargo_process("new foo").env("USER", "foo").run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["bar <baz>"]"#));
 }
 
@@ -278,45 +271,34 @@ fn finds_local_author_git() {
     cargo_process("init").env("USER", "foo").run();
 
     let toml = paths::root().join("Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["bar <baz>"]"#));
 }
 
 #[cargo_test]
-fn finds_git_email() {
+fn finds_git_author() {
     cargo_process("new foo")
         .env("GIT_AUTHOR_NAME", "foo")
         .env("GIT_AUTHOR_EMAIL", "gitfoo")
         .run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["foo <gitfoo>"]"#), contents);
 }
 
 #[cargo_test]
-fn finds_git_author() {
+fn finds_git_committer() {
     create_empty_gitconfig();
     cargo_process("new foo")
         .env_remove("USER")
-        .env("GIT_COMMITTER_NAME", "gitfoo")
+        .env("GIT_COMMITTER_NAME", "foo")
+        .env("GIT_COMMITTER_EMAIL", "gitfoo")
         .run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
-    assert!(contents.contains(r#"authors = ["gitfoo"]"#));
+    let contents = fs::read_to_string(&toml).unwrap();
+    assert!(contents.contains(r#"authors = ["foo <gitfoo>"]"#));
 }
 
 #[cargo_test]
@@ -327,26 +309,21 @@ fn author_prefers_cargo() {
         .unwrap();
     let root = paths::root();
     fs::create_dir(&root.join(".cargo")).unwrap();
-    File::create(&root.join(".cargo/config"))
-        .unwrap()
-        .write_all(
-            br#"
-        [cargo-new]
-        name = "new-foo"
-        email = "new-bar"
-        vcs = "none"
-    "#,
-        )
-        .unwrap();
+    fs::write(
+        &root.join(".cargo/config"),
+        r#"
+            [cargo-new]
+            name = "new-foo"
+            email = "new-bar"
+            vcs = "none"
+        "#,
+    )
+    .unwrap();
 
     cargo_process("new foo").env("USER", "foo").run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["new-foo <new-bar>"]"#));
     assert!(!root.join("foo/.gitignore").exists());
 }
@@ -360,11 +337,7 @@ fn strip_angle_bracket_author_email() {
         .run();
 
     let toml = paths::root().join("foo/Cargo.toml");
-    let mut contents = String::new();
-    File::open(&toml)
-        .unwrap()
-        .read_to_string(&mut contents)
-        .unwrap();
+    let contents = fs::read_to_string(&toml).unwrap();
     assert!(contents.contains(r#"authors = ["bar <baz>"]"#));
 }
 
@@ -372,17 +345,16 @@ fn strip_angle_bracket_author_email() {
 fn git_prefers_command_line() {
     let root = paths::root();
     fs::create_dir(&root.join(".cargo")).unwrap();
-    File::create(&root.join(".cargo/config"))
-        .unwrap()
-        .write_all(
-            br#"
-        [cargo-new]
-        vcs = "none"
-        name = "foo"
-        email = "bar"
-    "#,
-        )
-        .unwrap();
+    fs::write(
+        &root.join(".cargo/config"),
+        r#"
+            [cargo-new]
+            vcs = "none"
+            name = "foo"
+            email = "bar"
+        "#,
+    )
+    .unwrap();
 
     cargo_process("new foo --vcs git").env("USER", "foo").run();
     assert!(paths::root().join("foo/.gitignore").exists());
@@ -465,7 +437,10 @@ fn unknown_flags() {
 fn explicit_invalid_name_not_suggested() {
     cargo_process("new --name 10-invalid a")
         .with_status(101)
-        .with_stderr("[ERROR] Package names starting with a digit cannot be used as a crate name")
+        .with_stderr(
+            "[ERROR] the name `10-invalid` cannot be used as a crate name, \
+            the name cannot start with a digit",
+        )
         .run();
 }
 
@@ -540,3 +515,61 @@ fn lockfile_constant_during_new() {
     let after = fs::read_to_string(paths::root().join("foo/Cargo.lock")).unwrap();
     assert_eq!(before, after);
 }
+
+#[cargo_test]
+fn restricted_windows_name() {
+    if cfg!(windows) {
+        cargo_process("new nul")
+            .env("USER", "foo")
+            .with_status(101)
+            .with_stderr("[ERROR] cannot use name `nul`, it is a reserved Windows filename")
+            .run();
+    } else {
+        cargo_process("new nul")
+            .env("USER", "foo")
+            .with_stderr(
+                "\
+[WARNING] the name `nul` is a reserved Windows filename
+This package will not work on Windows platforms.
+[CREATED] binary (application) `nul` package
+",
+            )
+            .run();
+    }
+}
+
+#[cargo_test]
+fn non_ascii_name() {
+    cargo_process("new Привет")
+        .env("USER", "foo")
+        .with_stderr(
+            "\
+[WARNING] the name `Привет` contains non-ASCII characters
+Support for non-ASCII crate names is experimental and only valid on the nightly toolchain.
+[CREATED] binary (application) `Привет` package
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn non_ascii_name_invalid() {
+    // These are alphanumeric characters, but not Unicode XID.
+    cargo_process("new ⒶⒷⒸ")
+        .env("USER", "foo")
+        .with_status(101)
+        .with_stderr(
+            "[ERROR] invalid character `Ⓐ` in crate name: `ⒶⒷⒸ`, \
+            the first character must be a Unicode XID start character (most letters or `_`)",
+        )
+        .run();
+
+    cargo_process("new a¼")
+        .env("USER", "foo")
+        .with_status(101)
+        .with_stderr(
+            "[ERROR] invalid character `¼` in crate name: `a¼`, \
+            characters must be Unicode XID characters (numbers, `-`, `_`, or most letters)",
+        )
+        .run();
+}
