@@ -1,6 +1,8 @@
+//! Tests for profiles.
+
 use std::env;
 
-use cargo_test_support::project;
+use cargo_test_support::{is_nightly, project};
 
 #[cargo_test]
 fn profile_overrides() {
@@ -26,9 +28,9 @@ fn profile_overrides() {
         .with_stderr(
             "\
 [COMPILING] test v0.0.0 ([CWD])
-[RUNNING] `rustc --crate-name test src/lib.rs --color never --crate-type lib \
-        --emit=[..]link \
-        -C opt-level=1 \
+[RUNNING] `rustc --crate-name test src/lib.rs [..]--crate-type lib \
+        --emit=[..]link[..]\
+        -C opt-level=1[..]\
         -C debug-assertions=on \
         -C metadata=[..] \
         -C rpath \
@@ -62,8 +64,8 @@ fn opt_level_override_0() {
         .with_stderr(
             "\
 [COMPILING] test v0.0.0 ([CWD])
-[RUNNING] `rustc --crate-name test src/lib.rs --color never --crate-type lib \
-        --emit=[..]link \
+[RUNNING] `rustc --crate-name test src/lib.rs [..]--crate-type lib \
+        --emit=[..]link[..]\
         -C debuginfo=2 \
         -C metadata=[..] \
         --out-dir [..] \
@@ -95,8 +97,8 @@ fn debug_override_1() {
         .with_stderr(
             "\
 [COMPILING] test v0.0.0 ([CWD])
-[RUNNING] `rustc --crate-name test src/lib.rs --color never --crate-type lib \
-        --emit=[..]link \
+[RUNNING] `rustc --crate-name test src/lib.rs [..]--crate-type lib \
+        --emit=[..]link[..]\
         -C debuginfo=1 \
         -C metadata=[..] \
         --out-dir [..] \
@@ -131,9 +133,9 @@ fn check_opt_level_override(profile_level: &str, rustc_level: &str) {
         .with_stderr(&format!(
             "\
 [COMPILING] test v0.0.0 ([CWD])
-[RUNNING] `rustc --crate-name test src/lib.rs --color never --crate-type lib \
+[RUNNING] `rustc --crate-name test src/lib.rs [..]--crate-type lib \
         --emit=[..]link \
-        -C opt-level={level} \
+        -C opt-level={level}[..]\
         -C debuginfo=2 \
         -C debug-assertions=on \
         -C metadata=[..] \
@@ -204,19 +206,19 @@ fn top_level_overrides_deps() {
         .with_stderr(&format!(
             "\
 [COMPILING] foo v0.0.0 ([CWD]/foo)
-[RUNNING] `rustc --crate-name foo foo/src/lib.rs --color never \
+[RUNNING] `rustc --crate-name foo foo/src/lib.rs [..]\
         --crate-type dylib --crate-type rlib \
         --emit=[..]link \
         -C prefer-dynamic \
-        -C opt-level=1 \
+        -C opt-level=1[..]\
         -C debuginfo=2 \
         -C metadata=[..] \
         --out-dir [CWD]/target/release/deps \
         -L dependency=[CWD]/target/release/deps`
 [COMPILING] test v0.0.0 ([CWD])
-[RUNNING] `rustc --crate-name test src/lib.rs --color never --crate-type lib \
+[RUNNING] `rustc --crate-name test src/lib.rs [..]--crate-type lib \
         --emit=[..]link \
-        -C opt-level=1 \
+        -C opt-level=1[..]\
         -C debuginfo=2 \
         -C metadata=[..] \
         --out-dir [..] \
@@ -343,8 +345,8 @@ fn profile_panic_test_bench() {
     p.cargo("build")
         .with_stderr_contains(
             "\
-[WARNING] `panic` setting is ignored for `test` profile
 [WARNING] `panic` setting is ignored for `bench` profile
+[WARNING] `panic` setting is ignored for `test` profile
 ",
         )
         .run();
@@ -437,3 +439,142 @@ fn debug_0_report() {
         )
         .run();
 }
+
+#[cargo_test]
+fn thin_lto_works() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "top"
+            version = "0.5.0"
+            authors = []
+
+            [profile.release]
+            lto = 'thin'
+        "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build --release -v")
+        .with_stderr(
+            "\
+[COMPILING] top [..]
+[RUNNING] `rustc [..] -C lto=thin [..]`
+[FINISHED] [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+// Strip doesn't work on macos.
+#[cfg_attr(target_os = "macos", ignore)]
+fn strip_works() {
+    if !is_nightly() {
+        return;
+    }
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            cargo-features = ["strip"]
+
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [profile.release]
+            strip = 'symbols'
+        "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build --release -v")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(
+            "\
+[COMPILING] foo [..]
+[RUNNING] `rustc [..] -Z strip=symbols [..]`
+[FINISHED] [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn strip_requires_cargo_feature() {
+    if !is_nightly() {
+        return;
+    }
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [profile.release]
+            strip = 'symbols'
+        "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build --release -v")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] failed to parse manifest at `[CWD]/Cargo.toml`
+
+Caused by:
+  feature `strip` is required
+
+  consider adding `cargo-features = [\"strip\"]` to the manifest
+",
+        )
+        .run();
+}
+#[cargo_test]
+fn strip_rejects_invalid_option() {
+    if !is_nightly() {
+        return;
+    }
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            cargo-features = ["strip"]
+
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [profile.release]
+            strip = 'wrong'
+        "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build --release -v")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] failed to parse manifest at `[CWD]/Cargo.toml`
+
+Caused by:
+  unknown variant `wrong`, expected one of `debuginfo`, `none`, `symbols` for key [..]
+",
+        )
+        .run();
+}
