@@ -1,6 +1,5 @@
 //! Tests for including `Cargo.lock` when publishing/packaging.
 
-use std;
 use std::fs::File;
 
 use cargo_test_support::registry::Package;
@@ -85,6 +84,7 @@ fn package_lockfile() {
             "\
 Cargo.lock
 Cargo.toml
+Cargo.toml.orig
 src/main.rs
 ",
         )
@@ -114,6 +114,7 @@ fn package_lockfile_git_repo() {
 .cargo_vcs_info.json
 Cargo.lock
 Cargo.toml
+Cargo.toml.orig
 src/main.rs
 ",
         )
@@ -123,10 +124,11 @@ src/main.rs
         .with_stderr(
             "\
 [PACKAGING] foo v0.0.1 ([..])
-[ARCHIVING] Cargo.toml
-[ARCHIVING] src/main.rs
 [ARCHIVING] .cargo_vcs_info.json
 [ARCHIVING] Cargo.lock
+[ARCHIVING] Cargo.toml
+[ARCHIVING] Cargo.toml.orig
+[ARCHIVING] src/main.rs
 [VERIFYING] foo v0.0.1 ([..])
 [COMPILING] foo v0.0.1 ([..])
 [RUNNING] `rustc --crate-name foo src/main.rs [..]
@@ -221,9 +223,10 @@ fn note_resolve_changes() {
         .with_stderr_unordered(
             "\
 [PACKAGING] foo v0.0.1 ([..])
-[ARCHIVING] Cargo.toml
-[ARCHIVING] src/main.rs
 [ARCHIVING] Cargo.lock
+[ARCHIVING] Cargo.toml
+[ARCHIVING] Cargo.toml.orig
+[ARCHIVING] src/main.rs
 [UPDATING] `[..]` index
 [NOTE] package `mutli v0.1.0` added to the packaged Cargo.lock file, was originally sourced from `[..]/foo/mutli`
 [NOTE] package `patched v1.0.0` added to the packaged Cargo.lock file, was originally sourced from `[..]/foo/patched`
@@ -404,24 +407,18 @@ fn ignore_lockfile() {
     // With an explicit `include` list, but Cargo.lock in .gitignore, don't
     // complain about `Cargo.lock` being ignored. Note that it is still
     // included in the packaged regardless.
-    let (p, _r) = git::new_repo("foo", |p| {
+    let p = git::new("foo", |p| {
         p.file(
             "Cargo.toml",
-            r#"
-                [package]
-                name = "foo"
-                version = "0.0.1"
-                authors = []
-                license = "MIT"
-                description = "foo"
-                documentation = "foo"
-                homepage = "foo"
-                repository = "foo"
-
+            &pl_manifest(
+                "foo",
+                "0.0.1",
+                r#"
                 include = [
                     "src/main.rs"
                 ]
-            "#,
+                "#,
+            ),
         )
         .file("src/main.rs", "fn main() {}")
         .file(".gitignore", "Cargo.lock")
@@ -432,6 +429,7 @@ fn ignore_lockfile() {
 .cargo_vcs_info.json
 Cargo.lock
 Cargo.toml
+Cargo.toml.orig
 src/main.rs
 ",
         )
@@ -441,10 +439,11 @@ src/main.rs
         .with_stderr(
             "\
 [PACKAGING] foo v0.0.1 ([..])
-[ARCHIVING] Cargo.toml
-[ARCHIVING] src/main.rs
 [ARCHIVING] .cargo_vcs_info.json
 [ARCHIVING] Cargo.lock
+[ARCHIVING] Cargo.toml
+[ARCHIVING] Cargo.toml.orig
+[ARCHIVING] src/main.rs
 [VERIFYING] foo v0.0.1 ([..])
 [COMPILING] foo v0.0.1 ([..])
 [RUNNING] `rustc --crate-name foo src/main.rs [..]
@@ -453,3 +452,29 @@ src/main.rs
         )
         .run();
 }
+
+#[cargo_test]
+fn ignore_lockfile_inner() {
+    // Ignore `Cargo.lock` if in .gitignore in a git subdirectory.
+    let p = git::new("foo", |p| {
+        p.no_manifest()
+            .file("bar/Cargo.toml", &pl_manifest("bar", "0.0.1", ""))
+            .file("bar/src/main.rs", "fn main() {}")
+            .file("bar/.gitignore", "Cargo.lock")
+    });
+    p.cargo("generate-lockfile").cwd("bar").run();
+    p.cargo("package -v --no-verify")
+        .cwd("bar")
+        .with_stderr(
+            "\
+[PACKAGING] bar v0.0.1 ([..])
+[ARCHIVING] .cargo_vcs_info.json
+[ARCHIVING] .gitignore
+[ARCHIVING] Cargo.lock
+[ARCHIVING] Cargo.toml
+[ARCHIVING] Cargo.toml.orig
+[ARCHIVING] src/main.rs
+",
+        )
+        .run();
+}
