@@ -1,15 +1,15 @@
+//! Tests for normal registry dependencies.
+
+use cargo::util::paths::remove_dir_all;
+use cargo_test_support::cargo_process;
+use cargo_test_support::git;
+use cargo_test_support::paths::{self, CargoPathExt};
+use cargo_test_support::registry::{self, registry_path, Dependency, Package};
+use cargo_test_support::{basic_manifest, project, t};
 use std::fs::{self, File};
-use std::io::prelude::*;
 use std::path::Path;
 
-use crate::support::cargo_process;
-use crate::support::git;
-use crate::support::paths::{self, CargoPathExt};
-use crate::support::registry::{self, registry_path, registry_url, Dependency, Package};
-use crate::support::{basic_manifest, project};
-use cargo::util::paths::remove_dir_all;
-
-#[test]
+#[cargo_test]
 fn simple() {
     let p = project()
         .file(
@@ -57,7 +57,7 @@ fn simple() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn deps() {
     let p = project()
         .file(
@@ -95,7 +95,7 @@ fn deps() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn nonexistent() {
     Package::new("init", "0.0.1").publish();
 
@@ -128,7 +128,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn wrong_case() {
     Package::new("init", "0.0.1").publish();
 
@@ -163,7 +163,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn mis_hyphenated() {
     Package::new("mis-hyphenated", "0.0.1").publish();
 
@@ -198,7 +198,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn wrong_version() {
     let p = project()
         .file(
@@ -223,9 +223,9 @@ fn wrong_version() {
         .with_status(101)
         .with_stderr_contains(
             "\
-error: failed to select a version for the requirement `foo = \">= 1.0.0\"`
-  candidate versions found which didn't match: 0.0.2, 0.0.1
-  location searched: `[..]` index (which is replacing registry `[..]`)
+error: failed to select a version for the requirement `foo = \">=1.0.0\"`
+candidate versions found which didn't match: 0.0.2, 0.0.1
+location searched: `[..]` index (which is replacing registry `[..]`)
 required by package `foo v0.0.1 ([..])`
 ",
         )
@@ -238,16 +238,16 @@ required by package `foo v0.0.1 ([..])`
         .with_status(101)
         .with_stderr_contains(
             "\
-error: failed to select a version for the requirement `foo = \">= 1.0.0\"`
-  candidate versions found which didn't match: 0.0.4, 0.0.3, 0.0.2, ...
-  location searched: `[..]` index (which is replacing registry `[..]`)
+error: failed to select a version for the requirement `foo = \">=1.0.0\"`
+candidate versions found which didn't match: 0.0.4, 0.0.3, 0.0.2, ...
+location searched: `[..]` index (which is replacing registry `[..]`)
 required by package `foo v0.0.1 ([..])`
 ",
         )
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_cksum() {
     let p = project()
         .file(
@@ -285,7 +285,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_registry() {
     Package::new("init", "0.0.1").publish();
 
@@ -333,7 +333,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn package_with_path_deps() {
     Package::new("init", "0.0.1").publish();
 
@@ -359,16 +359,18 @@ fn package_with_path_deps() {
         .file("notyet/src/lib.rs", "")
         .build();
 
-    p.cargo("package -v")
+    p.cargo("package")
         .with_status(101)
         .with_stderr_contains(
             "\
-[ERROR] failed to verify package tarball
+[PACKAGING] foo [..]
+[UPDATING] [..]
+[ERROR] failed to prepare local package for uploading
 
 Caused by:
   no matching package named `notyet` found
-location searched: registry [..]
-required by package `foo v0.0.1 ([..])`
+  location searched: registry `https://github.com/rust-lang/crates.io-index`
+  required by package `foo v0.0.1 [..]`
 ",
         )
         .run();
@@ -379,8 +381,8 @@ required by package `foo v0.0.1 ([..])`
         .with_stderr(
             "\
 [PACKAGING] foo v0.0.1 ([CWD])
-[VERIFYING] foo v0.0.1 ([CWD])
 [UPDATING] `[..]` index
+[VERIFYING] foo v0.0.1 ([CWD])
 [DOWNLOADING] crates ...
 [DOWNLOADED] notyet v0.0.1 (registry `[ROOT][..]`)
 [COMPILING] notyet v0.0.1
@@ -391,7 +393,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn lockfile_locks() {
     let p = project()
         .file(
@@ -430,7 +432,7 @@ fn lockfile_locks() {
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn lockfile_locks_transitively() {
     let p = project()
         .file(
@@ -473,7 +475,7 @@ fn lockfile_locks_transitively() {
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_are_not_used() {
     let p = project()
         .file(
@@ -515,7 +517,7 @@ fn yanks_are_not_used() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn relying_on_a_yank_is_bad() {
     let p = project()
         .file(
@@ -541,9 +543,9 @@ fn relying_on_a_yank_is_bad() {
         .with_status(101)
         .with_stderr_contains(
             "\
-error: failed to select a version for the requirement `baz = \"= 0.0.2\"`
-  candidate versions found which didn't match: 0.0.1
-  location searched: `[..]` index (which is replacing registry `[..]`)
+error: failed to select a version for the requirement `baz = \"=0.0.2\"`
+candidate versions found which didn't match: 0.0.1
+location searched: `[..]` index (which is replacing registry `[..]`)
 required by package `bar v0.0.1`
     ... which is depended on by `foo [..]`
 ",
@@ -551,7 +553,7 @@ required by package `bar v0.0.1`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_in_lockfiles_are_ok() {
     let p = project()
         .file(
@@ -591,7 +593,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_in_lockfiles_are_ok_for_other_update() {
     let p = project()
         .file(
@@ -645,7 +647,7 @@ required by package `foo v0.0.1 ([..])`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn yanks_in_lockfiles_are_ok_with_new_dep() {
     let p = project()
         .file(
@@ -672,8 +674,9 @@ fn yanks_in_lockfiles_are_ok_with_new_dep() {
     Package::new("bar", "0.0.1").yanked(true).publish();
     Package::new("baz", "0.0.1").publish();
 
-    t!(t!(File::create(p.root().join("Cargo.toml"))).write_all(
-        br#"
+    p.change_file(
+        "Cargo.toml",
+        r#"
             [project]
             name = "foo"
             version = "0.0.1"
@@ -682,13 +685,13 @@ fn yanks_in_lockfiles_are_ok_with_new_dep() {
             [dependencies]
             bar = "*"
             baz = "*"
-    "#
-    ));
+        "#,
+    );
 
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn update_with_lockfile_if_packages_missing() {
     let p = project()
         .file(
@@ -723,7 +726,7 @@ fn update_with_lockfile_if_packages_missing() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_lockfile() {
     let p = project()
         .file(
@@ -820,7 +823,7 @@ fn update_lockfile() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn dev_dependency_not_used() {
     let p = project()
         .file(
@@ -855,7 +858,7 @@ fn dev_dependency_not_used() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn login_with_no_cargo_dir() {
     // Create a config in the root directory because `login` requires the
     // index to be updated, and we don't want to hit crates.io.
@@ -867,7 +870,7 @@ fn login_with_no_cargo_dir() {
     assert_eq!(credentials, "[registry]\ntoken = \"foo\"\n");
 }
 
-#[test]
+#[cargo_test]
 fn login_with_differently_sized_token() {
     // Verify that the configuration file gets properly truncated.
     registry::init();
@@ -880,7 +883,7 @@ fn login_with_differently_sized_token() {
     assert_eq!(credentials, "[registry]\ntoken = \"lmaolmaolmao\"\n");
 }
 
-#[test]
+#[cargo_test]
 fn bad_license_file() {
     Package::new("foo", "1.0.0").publish();
     let p = project()
@@ -898,14 +901,13 @@ fn bad_license_file() {
         )
         .file("src/main.rs", "fn main() {}")
         .build();
-    p.cargo("publish -v --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish -v --token sekrit")
         .with_status(101)
         .with_stderr_contains("[ERROR] the license file `foo` does not exist")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn updating_a_dep() {
     let p = project()
         .file(
@@ -952,8 +954,9 @@ fn updating_a_dep() {
         )
         .run();
 
-    t!(t!(File::create(&p.root().join("a/Cargo.toml"))).write_all(
-        br#"
+    p.change_file(
+        "a/Cargo.toml",
+        r#"
         [project]
         name = "a"
         version = "0.0.1"
@@ -961,8 +964,8 @@ fn updating_a_dep() {
 
         [dependencies]
         bar = "0.1.0"
-    "#
-    ));
+        "#,
+    );
     Package::new("bar", "0.1.0").publish();
 
     println!("second");
@@ -981,7 +984,7 @@ fn updating_a_dep() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn git_and_registry_dep() {
     let b = git::repo(&paths::root().join("b"))
         .file(
@@ -1043,7 +1046,7 @@ fn git_and_registry_dep() {
     p.cargo("build").with_stdout("").run();
 }
 
-#[test]
+#[cargo_test]
 fn update_publish_then_update() {
     // First generate a Cargo.lock and a clone of the registry index at the
     // "head" of the current registry.
@@ -1115,7 +1118,7 @@ fn update_publish_then_update() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn fetch_downloads() {
     let p = project()
         .file(
@@ -1146,7 +1149,7 @@ fn fetch_downloads() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_transitive_dependency() {
     let p = project()
         .file(
@@ -1194,7 +1197,7 @@ fn update_transitive_dependency() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_backtracking_ok() {
     let p = project()
         .file(
@@ -1243,7 +1246,7 @@ fn update_backtracking_ok() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn update_multiple_packages() {
     let p = project()
         .file(
@@ -1303,7 +1306,7 @@ fn update_multiple_packages() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bundled_crate_in_registry() {
     let p = project()
         .file(
@@ -1345,7 +1348,7 @@ fn bundled_crate_in_registry() {
     p.cargo("run").run();
 }
 
-#[test]
+#[cargo_test]
 fn update_same_prefix_oh_my_how_was_this_a_bug() {
     let p = project()
         .file(
@@ -1372,7 +1375,7 @@ fn update_same_prefix_oh_my_how_was_this_a_bug() {
     p.cargo("update -pfoobar --precise=0.2.0").run();
 }
 
-#[test]
+#[cargo_test]
 fn use_semver() {
     let p = project()
         .file(
@@ -1395,7 +1398,56 @@ fn use_semver() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
+fn use_semver_package_incorrectly() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [workspace]
+            members = ["a", "b"]
+            "#,
+        )
+        .file(
+            "a/Cargo.toml",
+            r#"
+            [project]
+            name = "a"
+            version = "0.1.1-alpha.0"
+            authors = []
+            "#,
+        )
+        .file(
+            "b/Cargo.toml",
+            r#"
+            [project]
+            name = "b"
+            version = "0.1.0"
+            authors = []
+
+            [dependencies]
+            a = { version = "^0.1", path = "../a" }
+            "#,
+        )
+        .file("a/src/main.rs", "fn main() {}")
+        .file("b/src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
+            "\
+error: no matching package named `a` found
+location searched: [..]
+prerelease package needs to be specified explicitly
+a = { version = \"0.1.1-alpha.0\" }
+required by package `b v0.1.0 ([..])`
+",
+        )
+        .run();
+}
+
+#[cargo_test]
 fn only_download_relevant() {
     let p = project()
         .file(
@@ -1435,7 +1487,7 @@ fn only_download_relevant() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn resolve_and_backtracking() {
     let p = project()
         .file(
@@ -1461,7 +1513,7 @@ fn resolve_and_backtracking() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn upstream_warnings_on_extra_verbose() {
     let p = project()
         .file(
@@ -1488,7 +1540,7 @@ fn upstream_warnings_on_extra_verbose() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn disallow_network() {
     let p = project()
         .file(
@@ -1510,7 +1562,10 @@ fn disallow_network() {
         .with_status(101)
         .with_stderr(
             "\
-error: failed to load source for a dependency on `foo`
+[ERROR] failed to get `foo` as a dependency of package `bar v0.5.0 [..]`
+
+Caused by:
+  failed to load source for dependency `foo`
 
 Caused by:
   Unable to update registry [..]
@@ -1522,7 +1577,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn add_dep_dont_update_registry() {
     let p = project()
         .file(
@@ -1557,8 +1612,9 @@ fn add_dep_dont_update_registry() {
 
     p.cargo("build").run();
 
-    t!(t!(File::create(p.root().join("Cargo.toml"))).write_all(
-        br#"
+    p.change_file(
+        "Cargo.toml",
+        r#"
         [project]
         name = "bar"
         version = "0.5.0"
@@ -1567,8 +1623,8 @@ fn add_dep_dont_update_registry() {
         [dependencies]
         baz = { path = "baz" }
         remote = "0.3"
-    "#
-    ));
+        "#,
+    );
 
     p.cargo("build")
         .with_stderr(
@@ -1580,7 +1636,7 @@ fn add_dep_dont_update_registry() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bump_version_dont_update_registry() {
     let p = project()
         .file(
@@ -1615,8 +1671,9 @@ fn bump_version_dont_update_registry() {
 
     p.cargo("build").run();
 
-    t!(t!(File::create(p.root().join("Cargo.toml"))).write_all(
-        br#"
+    p.change_file(
+        "Cargo.toml",
+        r#"
         [project]
         name = "bar"
         version = "0.6.0"
@@ -1624,8 +1681,8 @@ fn bump_version_dont_update_registry() {
 
         [dependencies]
         baz = { path = "baz" }
-    "#
-    ));
+        "#,
+    );
 
     p.cargo("build")
         .with_stderr(
@@ -1637,7 +1694,7 @@ fn bump_version_dont_update_registry() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn old_version_req() {
     let p = project()
         .file(
@@ -1691,7 +1748,7 @@ this warning.
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn old_version_req_upstream() {
     let p = project()
         .file(
@@ -1750,7 +1807,7 @@ this warning.
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn toml_lies_but_index_is_truth() {
     Package::new("foo", "0.2.0").publish();
     Package::new("bar", "0.3.0")
@@ -1789,7 +1846,7 @@ fn toml_lies_but_index_is_truth() {
     p.cargo("build -v").run();
 }
 
-#[test]
+#[cargo_test]
 fn vv_prints_warnings() {
     Package::new("foo", "0.2.0")
         .file(
@@ -1817,7 +1874,7 @@ fn vv_prints_warnings() {
     p.cargo("build -vv").run();
 }
 
-#[test]
+#[cargo_test]
 fn bad_and_or_malicious_packages_rejected() {
     Package::new("foo", "0.2.0")
         .extra_file("foo-0.1.0/src/lib.rs", "")
@@ -1858,7 +1915,7 @@ Caused by:
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn git_init_templatedir_missing() {
     Package::new("foo", "0.2.0").dep("bar", "*").publish();
     Package::new("bar", "0.2.0").publish();
@@ -1882,21 +1939,20 @@ fn git_init_templatedir_missing() {
     p.cargo("build").run();
 
     remove_dir_all(paths::home().join(".cargo/registry")).unwrap();
-    File::create(paths::home().join(".gitconfig"))
-        .unwrap()
-        .write_all(
-            br#"
+    fs::write(
+        paths::home().join(".gitconfig"),
+        r#"
             [init]
             templatedir = nowhere
         "#,
-        )
-        .unwrap();
+    )
+    .unwrap();
 
     p.cargo("build").run();
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn rename_deps_and_features() {
     Package::new("foo", "0.1.0")
         .file("src/lib.rs", "pub fn f1() {}")
@@ -1955,7 +2011,7 @@ fn rename_deps_and_features() {
     p.cargo("build --features bar/another").run();
 }
 
-#[test]
+#[cargo_test]
 fn ignore_invalid_json_lines() {
     Package::new("foo", "0.1.0").publish();
     Package::new("foo", "0.1.1").invalid_json(true).publish();
@@ -1981,7 +2037,7 @@ fn ignore_invalid_json_lines() {
     p.cargo("build").run();
 }
 
-#[test]
+#[cargo_test]
 fn readonly_registry_still_works() {
     Package::new("foo", "0.1.0").publish();
 
@@ -2008,7 +2064,6 @@ fn readonly_registry_still_works() {
     // make sure we un-readonly the files afterwards so "cargo clean" can remove them (#6934)
     chmod_readonly(&paths::home(), false);
 
-
     fn chmod_readonly(path: &Path, readonly: bool) {
         for entry in t!(path.read_dir()) {
             let entry = t!(entry);
@@ -2028,3 +2083,53 @@ fn readonly_registry_still_works() {
         t!(fs::set_permissions(path, perms));
     }
 }
+
+#[cargo_test]
+fn registry_index_rejected() {
+    Package::new("dep", "0.1.0").publish();
+
+    let p = project()
+        .file(
+            ".cargo/config",
+            r#"
+            [registry]
+            index = "https://example.com/"
+            "#,
+        )
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [dependencies]
+            dep = "0.1"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("check")
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] failed to parse manifest at `[..]/foo/Cargo.toml`
+
+Caused by:
+  the `registry.index` config value is no longer supported
+  Use `[source]` replacement to alter the default index for crates.io.
+",
+        )
+        .run();
+
+    p.cargo("login")
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] the `registry.index` config value is no longer supported
+Use `[source]` replacement to alter the default index for crates.io.
+",
+        )
+        .run();
+}
