@@ -1,8 +1,13 @@
-use crate::support::install::{assert_has_installed_exe, assert_has_not_installed_exe, cargo_home};
-use crate::support::is_nightly;
-use crate::support::project;
+//! Tests for targets with `required-features`.
 
-#[test]
+use cargo_test_support::install::{
+    assert_has_installed_exe, assert_has_not_installed_exe, cargo_home,
+};
+use cargo_test_support::is_nightly;
+use cargo_test_support::paths::CargoPathExt;
+use cargo_test_support::project;
+
+#[cargo_test]
 fn build_bin_default_features() {
     let p = project()
         .file(
@@ -57,7 +62,7 @@ Consider enabling them by passing, e.g., `--features=\"a\"`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_bin_arg_features() {
     let p = project()
         .file(
@@ -83,7 +88,7 @@ fn build_bin_arg_features() {
     assert!(p.bin("foo").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn build_bin_multiple_required_features() {
     let p = project()
         .file(
@@ -128,7 +133,7 @@ fn build_bin_multiple_required_features() {
     p.cargo("build --no-default-features").run();
 }
 
-#[test]
+#[cargo_test]
 fn build_example_default_features() {
     let p = project()
         .file(
@@ -165,7 +170,7 @@ Consider enabling them by passing, e.g., `--features=\"a\"`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn build_example_arg_features() {
     let p = project()
         .file(
@@ -191,7 +196,7 @@ fn build_example_arg_features() {
     assert!(p.bin("examples/foo").is_file());
 }
 
-#[test]
+#[cargo_test]
 fn build_example_multiple_required_features() {
     let p = project()
         .file(
@@ -261,7 +266,7 @@ Consider enabling them by passing, e.g., `--features=\"a\"`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn test_default_features() {
     let p = project()
         .file(
@@ -288,21 +293,21 @@ fn test_default_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("test test ... ok")
         .run();
 
     p.cargo("test --no-default-features")
-        .with_stderr("[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]")
+        .with_stderr("[FINISHED] test [unoptimized + debuginfo] target(s) in [..]")
         .with_stdout("")
         .run();
 
     p.cargo("test --test=foo")
         .with_stderr(
             "\
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("test test ... ok")
@@ -319,7 +324,7 @@ Consider enabling them by passing, e.g., `--features=\"a\"`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn test_arg_features() {
     let p = project()
         .file(
@@ -345,14 +350,14 @@ fn test_arg_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("test test ... ok")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn test_multiple_required_features() {
     let p = project()
         .file(
@@ -386,7 +391,7 @@ fn test_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo_2-[..][EXE]",
         )
         .with_stdout_contains("test test ... ok")
@@ -396,7 +401,7 @@ fn test_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo_1-[..][EXE]
 [RUNNING] target/debug/deps/foo_2-[..][EXE]",
         )
@@ -404,12 +409,12 @@ fn test_multiple_required_features() {
         .run();
 
     p.cargo("test --no-default-features")
-        .with_stderr("[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]")
+        .with_stderr("[FINISHED] test [unoptimized + debuginfo] target(s) in [..]")
         .with_stdout("")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bench_default_features() {
     if !is_nightly() {
         return;
@@ -449,21 +454,21 @@ fn bench_default_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("test bench ... bench: [..]")
         .run();
 
     p.cargo("bench --no-default-features")
-        .with_stderr("[FINISHED] release [optimized] target(s) in [..]".to_string())
+        .with_stderr("[FINISHED] bench [optimized] target(s) in [..]".to_string())
         .with_stdout("")
         .run();
 
     p.cargo("bench --bench=foo")
         .with_stderr(
             "\
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("test bench ... bench: [..]")
@@ -480,7 +485,7 @@ Consider enabling them by passing, e.g., `--features=\"a\"`
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bench_arg_features() {
     if !is_nightly() {
         return;
@@ -519,14 +524,14 @@ fn bench_arg_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("test bench ... bench: [..]")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn bench_multiple_required_features() {
     if !is_nightly() {
         return;
@@ -582,7 +587,7 @@ fn bench_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo_2-[..][EXE]",
         )
         .with_stdout_contains("test bench ... bench: [..]")
@@ -592,7 +597,7 @@ fn bench_multiple_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo_1-[..][EXE]
 [RUNNING] target/release/deps/foo_2-[..][EXE]",
         )
@@ -600,12 +605,12 @@ fn bench_multiple_required_features() {
         .run();
 
     p.cargo("bench --no-default-features")
-        .with_stderr("[FINISHED] release [optimized] target(s) in [..]")
+        .with_stderr("[FINISHED] bench [optimized] target(s) in [..]")
         .with_stdout("")
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn install_default_features() {
     let p = project()
         .file(
@@ -663,7 +668,7 @@ fn install_default_features() {
 
 Caused by:
   target `foo` in package `foo` requires the features: `a`
-Consider enabling them by passing, e.g., `--features=\"a\"`
+  Consider enabling them by passing, e.g., `--features=\"a\"`
 ",
         )
         .run();
@@ -683,14 +688,14 @@ Consider enabling them by passing, e.g., `--features=\"a\"`
 
 Caused by:
   target `foo` in package `foo` requires the features: `a`
-Consider enabling them by passing, e.g., `--features=\"a\"`
+  Consider enabling them by passing, e.g., `--features=\"a\"`
 ",
         )
         .run();
     assert_has_not_installed_exe(cargo_home(), "foo");
 }
 
-#[test]
+#[cargo_test]
 fn install_arg_features() {
     let p = project()
         .file(
@@ -717,7 +722,7 @@ fn install_arg_features() {
     p.cargo("uninstall foo").run();
 }
 
-#[test]
+#[cargo_test]
 fn install_multiple_required_features() {
     let p = project()
         .file(
@@ -773,7 +778,7 @@ fn install_multiple_required_features() {
     assert_has_not_installed_exe(cargo_home(), "foo_2");
 }
 
-#[test]
+#[cargo_test]
 fn dep_feature_in_toml() {
     let p = project()
         .file(
@@ -847,7 +852,7 @@ fn dep_feature_in_toml() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("test test ... ok")
@@ -860,7 +865,7 @@ fn dep_feature_in_toml() {
                 "\
 [COMPILING] bar v0.0.1 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo-[..][EXE]",
             )
             .with_stdout_contains("test bench ... bench: [..]")
@@ -873,7 +878,7 @@ fn dep_feature_in_toml() {
     p.cargo("uninstall foo").run();
 }
 
-#[test]
+#[cargo_test]
 fn dep_feature_in_cmd_line() {
     let p = project()
         .file(
@@ -906,7 +911,17 @@ fn dep_feature_in_cmd_line() {
         )
         .file("src/main.rs", "fn main() {}")
         .file("examples/foo.rs", "fn main() {}")
-        .file("tests/foo.rs", "#[test]\nfn test() {}")
+        .file(
+            "tests/foo.rs",
+            r#"
+            #[test]
+            fn bin_is_built() {
+                let s = format!("target/debug/foo{}", std::env::consts::EXE_SUFFIX);
+                let p = std::path::Path::new(&s);
+                assert!(p.exists(), "foo does not exist");
+            }
+            "#,
+        )
         .file(
             "benches/foo.rs",
             r#"
@@ -932,7 +947,9 @@ fn dep_feature_in_cmd_line() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("build").run();
+    // This is a no-op
+    p.cargo("build").with_stderr("[FINISHED] dev [..]").run();
+    assert!(!p.bin("foo").is_file());
 
     // bin
     p.cargo("build --bin=foo")
@@ -963,25 +980,29 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
     assert!(p.bin("examples/foo").is_file());
 
     // test
+    // This is a no-op, since no tests are enabled
     p.cargo("test")
-        .with_stderr("[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]")
+        .with_stderr("[FINISHED] test [unoptimized + debuginfo] target(s) in [..]")
         .with_stdout("")
         .run();
 
+    // Delete the target directory so this can check if the main.rs gets built.
+    p.build_dir().rm_rf();
     p.cargo("test --test=foo --features bar/a")
         .with_stderr(
             "\
+[COMPILING] bar v0.0.1 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo-[..][EXE]",
         )
-        .with_stdout_contains("test test ... ok")
+        .with_stdout_contains("test bin_is_built ... ok")
         .run();
 
     // bench
     if is_nightly() {
         p.cargo("bench")
-            .with_stderr("[FINISHED] release [optimized] target(s) in [..]")
+            .with_stderr("[FINISHED] bench [optimized] target(s) in [..]")
             .with_stdout("")
             .run();
 
@@ -990,7 +1011,7 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
                 "\
 [COMPILING] bar v0.0.1 ([CWD]/bar)
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo-[..][EXE]",
             )
             .with_stdout_contains("test bench ... bench: [..]")
@@ -1015,7 +1036,7 @@ Consider enabling them by passing, e.g., `--features=\"bar/a\"`
     p.cargo("uninstall foo").run();
 }
 
-#[test]
+#[cargo_test]
 fn test_skips_compiling_bin_with_missing_required_features() {
     let p = project()
         .file(
@@ -1044,7 +1065,7 @@ fn test_skips_compiling_bin_with_missing_required_features() {
         .with_stderr(
             "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] target/debug/deps/foo-[..][EXE]",
         )
         .with_stdout_contains("running 0 tests")
@@ -1064,7 +1085,7 @@ error[E0463]: can't find crate for `bar`",
             .with_stderr(
                 "\
 [COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] target/release/deps/foo-[..][EXE]",
             )
             .with_stdout_contains("running 0 tests")
@@ -1081,7 +1102,7 @@ error[E0463]: can't find crate for `bar`",
     }
 }
 
-#[test]
+#[cargo_test]
 fn run_default() {
     let p = project()
         .file(
@@ -1118,7 +1139,7 @@ Consider enabling them by passing, e.g., `--features=\"a\"`
     p.cargo("run --features a").run();
 }
 
-#[test]
+#[cargo_test]
 fn run_default_multiple_required_features() {
     let p = project()
         .file(
@@ -1154,8 +1175,101 @@ fn run_default_multiple_required_features() {
         .with_status(101)
         .with_stderr(
             "\
-             error: `cargo run` requires that a package only have one executable; \
-             use the `--bin` option to specify which one to run\navailable binaries: foo1, foo2",
+error: `cargo run` could not determine which binary to run[..]
+available binaries: foo1, foo2",
         )
         .run();
 }
+
+#[cargo_test]
+fn renamed_required_features() {
+    // Test that required-features uses renamed package feature names.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            edition = "2018"
+
+            [[bin]]
+            name = "x"
+            required-features = ["a1/f1"]
+
+            [dependencies]
+            a1 = {path="a1", package="a"}
+            a2 = {path="a2", package="a"}
+            "#,
+        )
+        .file(
+            "src/bin/x.rs",
+            r#"
+            fn main() {
+                a1::f();
+                a2::f();
+            }
+            "#,
+        )
+        .file(
+            "a1/Cargo.toml",
+            r#"
+            [package]
+            name = "a"
+            version = "0.1.0"
+
+            [features]
+            f1 = []
+            "#,
+        )
+        .file(
+            "a1/src/lib.rs",
+            r#"
+            pub fn f() {
+                if cfg!(feature="f1") {
+                    println!("a1 f1");
+                }
+            }
+            "#,
+        )
+        .file(
+            "a2/Cargo.toml",
+            r#"
+             [package]
+            name = "a"
+            version = "0.2.0"
+
+            [features]
+            f2 = []
+           "#,
+        )
+        .file(
+            "a2/src/lib.rs",
+            r#"
+            pub fn f() {
+                if cfg!(feature="f2") {
+                    println!("a2 f2");
+                }
+            }
+            "#,
+        )
+        .build();
+
+    p.cargo("run")
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] target `x` in package `foo` requires the features: `a1/f1`
+Consider enabling them by passing, e.g., `--features=\"a1/f1\"`
+",
+        )
+        .run();
+
+    p.cargo("build --features a1/f1").run();
+    p.rename_run("x", "x_with_f1").with_stdout("a1 f1").run();
+
+    p.cargo("build --features a1/f1,a2/f2").run();
+    p.rename_run("x", "x_with_f1_f2")
+        .with_stdout("a1 f1\na2 f2")
+        .run();
+}
