@@ -1,8 +1,7 @@
-use std::fs::File;
-use std::io::prelude::*;
+//! Tests for the `cargo update` command.
 
-use crate::support::registry::Package;
-use crate::support::{basic_manifest, project};
+use cargo_test_support::registry::Package;
+use cargo_test_support::{basic_manifest, project};
 
 #[cargo_test]
 fn minor_update_two_places() {
@@ -40,20 +39,18 @@ fn minor_update_two_places() {
     p.cargo("build").run();
     Package::new("log", "0.1.1").publish();
 
-    File::create(p.root().join("foo/Cargo.toml"))
-        .unwrap()
-        .write_all(
-            br#"
-                [package]
-                name = "foo"
-                version = "0.0.1"
-                authors = []
+    p.change_file(
+        "foo/Cargo.toml",
+        r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
 
-                [dependencies]
-                log = "0.1.1"
-            "#,
-        )
-        .unwrap();
+            [dependencies]
+            log = "0.1.1"
+        "#,
+    );
 
     p.cargo("build").run();
 }
@@ -447,6 +444,7 @@ fn update_precise_first_run() {
       "links": null,
       "manifest_path": "[..]/home/.cargo/registry/src/-[..]/serde-0.2.0/Cargo.toml",
       "metadata": null,
+      "publish": null,
       "name": "serde",
       "readme": null,
       "repository": null,
@@ -494,6 +492,7 @@ fn update_precise_first_run() {
       "links": null,
       "manifest_path": "[..]/foo/Cargo.toml",
       "metadata": null,
+      "publish": null,
       "name": "bar",
       "readme": null,
       "repository": null,
@@ -523,6 +522,12 @@ fn update_precise_first_run() {
         ],
         "deps": [
           {
+            "dep_kinds": [
+              {
+                "kind": null,
+                "target": null
+              }
+            ],
             "name": "serde",
             "pkg": "serde 0.2.0 (registry+https://github.com/rust-lang/crates.io-index)"
           }
@@ -544,7 +549,8 @@ fn update_precise_first_run() {
   "workspace_members": [
     "bar 0.0.1 (path+file://[..]/foo)"
   ],
-  "workspace_root": "[..]/foo"
+  "workspace_root": "[..]/foo",
+  "metadata": null
 }"#,
         )
         .run();
@@ -570,7 +576,7 @@ fn preserve_top_comment() {
     let mut lines = lockfile.lines().collect::<Vec<_>>();
     lines.insert(2, "# some other comment");
     let mut lockfile = lines.join("\n");
-    lockfile.push_str("\n"); // .lines/.join loses the last newline
+    lockfile.push_str("\n\n"); // .lines/.join loses the last newline
     println!("saving Cargo.lock contents:\n{}", lockfile);
 
     p.change_file("Cargo.lock", &lockfile);
@@ -620,7 +626,7 @@ fn dry_run_update() {
         .build();
 
     p.cargo("build").run();
-    let old_lockfile = p.read_file("Cargo.lock");
+    let old_lockfile = p.read_lockfile();
 
     Package::new("log", "0.1.1").publish();
     Package::new("serde", "0.1.1").dep("log", "0.1").publish();
@@ -634,6 +640,6 @@ fn dry_run_update() {
 ",
         )
         .run();
-    let new_lockfile = p.read_file("Cargo.lock");
+    let new_lockfile = p.read_lockfile();
     assert_eq!(old_lockfile, new_lockfile)
 }
