@@ -1,12 +1,9 @@
 //! Tests for workspaces.
 
-use std::env;
-use std::fs::{self, File};
-use std::io::{Read, Write};
-
 use cargo_test_support::registry::Package;
-use cargo_test_support::{basic_lib_manifest, basic_manifest, git, project};
-use cargo_test_support::{sleep_ms, t};
+use cargo_test_support::{basic_lib_manifest, basic_manifest, git, project, sleep_ms};
+use std::env;
+use std::fs;
 
 #[cargo_test]
 fn simple_explicit() {
@@ -1037,8 +1034,7 @@ fn new_warns_you_this_will_not_work() {
         .env("USER", "foo")
         .with_stderr(
             "\
-warning: compiling this new crate may not work due to invalid workspace \
-configuration
+warning: compiling this new crate may not work due to invalid workspace configuration
 
 current package believes it's in a workspace when it's not:
 current: [..]
@@ -1114,13 +1110,11 @@ fn lock_doesnt_change_depending_on_crate() {
 
     p.cargo("build").run();
 
-    let mut lockfile = String::new();
-    t!(t!(File::open(p.root().join("Cargo.lock"))).read_to_string(&mut lockfile));
+    let lockfile = p.read_lockfile();
 
     p.cargo("build").cwd("baz").run();
 
-    let mut lockfile2 = String::new();
-    t!(t!(File::open(p.root().join("Cargo.lock"))).read_to_string(&mut lockfile2));
+    let lockfile2 = p.read_lockfile();
 
     assert_eq!(lockfile, lockfile2);
 }
@@ -1169,8 +1163,7 @@ fn rebuild_please() {
 
     sleep_ms(1000);
 
-    t!(t!(File::create(p.root().join("lib/src/lib.rs")))
-        .write_all(br#"pub fn foo() -> u32 { 1 }"#));
+    p.change_file("lib/src/lib.rs", "pub fn foo() -> u32 { 1 }");
 
     p.cargo("build").cwd("lib").run();
 
@@ -2124,7 +2117,7 @@ fn ws_err_unused() {
 [ERROR] failed to parse manifest at `[..]/foo/Cargo.toml`
 
 Caused by:
-  virtual manifests do not specify {}
+  this virtual manifest specifies a {} section, which is not allowed
 ",
                 key
             ))
@@ -2205,7 +2198,7 @@ fn ws_warn_path() {
 
 #[cargo_test]
 fn invalid_missing() {
-    // Warnings include path to manifest.
+    // Make sure errors are not suppressed with -q.
     let p = project()
         .file(
             "Cargo.toml",
@@ -2225,16 +2218,20 @@ fn invalid_missing() {
         .with_status(101)
         .with_stderr(
             "\
-error: [..]
+[ERROR] failed to get `x` as a dependency of package `foo v0.1.0 [..]`
+
+Caused by:
+  failed to load source for dependency `x`
+
+Caused by:
+  Unable to update [..]/foo/x
+
+Caused by:
+  failed to read `[..]foo/x/Cargo.toml`
 
 Caused by:
   [..]
-
-Caused by:
-  [..]
-
-Caused by:
-  [..]",
+",
         )
         .run();
 }
